/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.config;

import jp.pegasus.main.Main;
import jp.pegasus.util.JsonUtil;
import org.apache.pivot.collections.HashMap;
import org.apache.pivot.collections.Map;
import org.apache.pivot.collections.immutable.ImmutableMap;

import java.io.File;

@SuppressWarnings("unchecked")
public interface Config {

    static class ConfJ {
        public static Map<Object, Object> properties = new HashMap<Object, Object>() {{
            JsonUtil.setProperties(this, Location.CONFIG);
        }};
        public static ImmutableMap<Object, Object> immutableProperties = new ImmutableMap(properties);

        public static Map<Object, Object> getProperties() {
            return immutableProperties;
        }
    }

    static final float DEF_SPLIT_RATIO_LEFT = ((Double) ConfJ.getProperties().get("splitRatioLeft")).floatValue();
    static final float DEF_SPLIT_RATIO_RIGT = ((Double) ConfJ.getProperties().get("splitRatioRight")).floatValue();

    static final String VERSION = (String) ConfJ.getProperties().get("version");
    static final String TITLE = (String) ConfJ.getProperties().get("title");

    static final String APP_DIR = new StringBuilder(System.getenv("LOCALAPPDATA")).append(File.separator).append(TITLE).append("-").append(VERSION).toString();
    static final String APP_DATA_DIR = new StringBuilder(APP_DIR).append(File.separator).append("data").toString();
    static final String APP_DATA_FILE_NAME = (String) ConfJ.getProperties().get("dataFileName");
    static final String APP_DATA_FILE_PATH = APP_DATA_DIR + File.separator + APP_DATA_FILE_NAME;

    static final String GIT_REP_DIR = new StringBuilder(APP_DIR).append(File.separator).append("repository").toString();
    static final String GIT_REP_DATA_DIR = new StringBuilder(GIT_REP_DIR).append(File.separator).append("data").toString();

    static final String APP_CELL_DATA_DIR = new StringBuilder(APP_DATA_DIR).append(File.separator).append("cell").toString();
    static final String APP_TEXT_DATA_DIR = new StringBuilder(APP_DATA_DIR).append(File.separator).append("text").toString();

    static final int HEADER_COL_WIDTH = (Integer) ConfJ.getProperties().get("headerColumnWidth");
    static final int COLUMN_MAX_WIDTH = (Integer) ConfJ.getProperties().get("columnMaxWidth");
    static final int COLUMN_MIN_WIDTH = (Integer) ConfJ.getProperties().get("columnMinWidth");
    static final int MAX_COL_NUMBER = (Integer) ConfJ.getProperties().get("maxColumnNumber");
    static final int MAX_ROW_NUMBER = (Integer) ConfJ.getProperties().get("maxRowNumber");
    static final int COLUMN_WIDTH = (Integer) ConfJ.getProperties().get("columnWidth");

    static final int TEXT_AREA_WIDTH = (Integer) ConfJ.getProperties().get("textAreaWidth");

    static final int PUSH_BUTTON_PREFERRED_WIDTH = (Integer) ConfJ.getProperties().get("pushButtonPreferredWidth");
    static final int PUSH_BUTTON_PREFERRED_HEIGHT = (Integer) ConfJ.getProperties().get("pushButtonPreferredHeight");
    static final int FLOW_PANE_PREFERRED_WIDTH = (Integer) ConfJ.getProperties().get("flowPanePreferredWidth");

    static final long SELECT_ALL_INDEX = 0x00000000FFFFFFFFl & (((long) MAX_COL_NUMBER - 2 << 16) | (long) MAX_ROW_NUMBER - 2);

    static final Class BASE_RESOURCE_CLASS = Main.class;

    static final String DIGEST_ARGORITHM = (String) ConfJ.getProperties().get("digestAlgorithm");

    static final String GIT_CFG_DEF_USR_NM = (String) ConfJ.getProperties().get("gitConfigDefaultUserName");
    static final String GIT_CFG_DEF_USR_ML = (String) ConfJ.getProperties().get("gitConfigDefaultUserMail");

}
