/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *     Pegasus is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     Pegasus is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.deprecated;

import jp.pegasus.pivot.util.SSTableViewUtil;
import org.apache.pivot.collections.HashMap;
import org.apache.pivot.collections.Map;
import org.apache.pivot.json.JSONSerializer;
import org.apache.pivot.serialization.SerializationException;
import org.apache.pivot.wtk.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Iterator;

/**
 * Created by IntelliJ IDEA.
 * User: Kitani7
 * Date: 10/12/28
 * Time: 10:37
 * To change this template use File | Settings | File Templates.
 */
@Deprecated
public class SpreadSheet3 implements Application {
    private Logger log = LoggerFactory.getLogger(SpreadSheet3.class);
    private static long L00;
    private static long L01;

    @Override
    public void startup(Display display, Map<String, String> properties) throws Exception {
        initialize(display);

        L01 = System.currentTimeMillis();
        log.debug("execution time: {} s", (L01 - L00) / 1000.0);
    }

    public void initialize(Display display) {
        HashMap<String, ?> metadataMap = getTableMetadata();

        int rowNumber = (Integer) metadataMap.get("rowNumber");
        int colNumber = (Integer) metadataMap.get("colNumber");

        int headerRowHeight = (Integer) metadataMap.get("headerRowHeight");
        int headerRowWidth = (Integer) metadataMap.get("headerRowWidth");
        int headerColWidth = (Integer) metadataMap.get("headerColWidth");

        String cellBackgroundColor = (String) metadataMap.get("cellBackgroundColor");
        String cellBackgroundColorSelected = (String) metadataMap.get("cellBackgroundColorSelected");
        String cellBorderColor = (String) metadataMap.get("cellBorderColor");
        String cellBorderColorSelected = (String) metadataMap.get("cellBorderColorSelected");


        TablePane tp = createTablePane();
        TablePane.ColumnSequence cols = tp.getColumns();
        for (int colCount = 0; colCount < colNumber; colCount++) {
            TablePane.Column col = new TablePane.Column();
            if (colCount == 0) {
                col.setWidth(headerRowWidth);
            } else {
                col.setWidth(headerColWidth);
            }
            cols.add(col);
        }

        TablePane.RowSequence rows = tp.getRows();
        for (int rowCount = 0; rowCount < rowNumber; rowCount++) {
            TablePane.Row row = new TablePane.Row();
            if (rowCount == 0) {
                TablePane.Filler f = new TablePane.Filler();
                row.add(f);
                for (int colCount = 1; colCount < colNumber; colCount++) {
                    String headerColName = SSTableViewUtil.fromNumber2Alphabet(colCount);
                    Label headerColLabel = getHeaderColLabelMetadata();
                    headerColLabel.setText(headerColName);
                    row.add(headerColLabel);
                }
            } else {
                for (int colCount = 1; colCount < colNumber; colCount++) {
                    TablePane.Filler f = new TablePane.Filler();
                    row.add(f);
                }
            }

            row.setHeight(headerRowHeight);
            rows.add(row);
        }

        ScrollPane sp = new ScrollPane();
        sp.setView(tp);
        Border b = new Border();
        b.setContent(sp);
        Window w = new Window();
        w.setContent(b);
        w.open(display);
    }

    @Override
    public boolean shutdown(boolean optional) throws Exception {
        return false;
    }

    @Override
    public void suspend() throws Exception {
    }

    @Override
    public void resume() throws Exception {
    }

    public static void main(String[] args) {
        L00 = System.currentTimeMillis();
        DesktopApplicationContext.main(SpreadSheet3.class, args);
    }

    @SuppressWarnings("unchecked")
    private Label getHeaderColLabelMetadata() {
        return getLabelMetadata("/json/SSHeaderColLabelPane.json");
    }

    @SuppressWarnings("unchecked")
    private Label getHeaderRowLabelMetadata() {
        return getLabelMetadata("/json/SSHeaderRowLabelPane.json");
    }

    @SuppressWarnings("unchecked")
    private Label getLabelMetadata(String resourcePath) {
        Label label = new Label();

        try {
            URL jsonLocation = getClass().getResource(resourcePath);
            try {
                InputStream inputStream = jsonLocation.openStream();
                JSONSerializer serializer = new JSONSerializer();
                try {
                    Map<String, ?> properties = (Map<String, ?>) serializer.readObject(inputStream);
                    Iterator<String> iterator = properties.iterator();
                    while (iterator.hasNext()) {
                        String key = iterator.next();
                        label.getStyles().put(key, properties.get(key));
                    }
                } catch (SerializationException e) {
                    e.printStackTrace();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        } catch (SecurityException e) {
            e.printStackTrace();
        }
        return label;
    }

    @SuppressWarnings("unchecked")
    private HashMap<String, ?> getTableMetadata() {
        HashMap<String, ?> metadataMap = new HashMap();

        try {
            URL jsonLocation = getClass().getResource("/json/SSTableMetadata.json");
            try {
                InputStream inputStream = jsonLocation.openStream();
                JSONSerializer serializer = new JSONSerializer();
                try {
                    metadataMap = (HashMap<String, ?>) serializer.readObject(inputStream);
                } catch (SerializationException e) {
                    e.printStackTrace();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        } catch (SecurityException e) {
            e.printStackTrace();
        }
        return metadataMap;
    }

    @SuppressWarnings("unchecked")
    private TablePane createTablePane() {
        TablePane tp = new TablePane();

        try {
            URL jsonLocation = getClass().getResource("/json/SSTablePane.json");
            try {
                InputStream inputStream = jsonLocation.openStream();
                JSONSerializer serializer = new JSONSerializer();
                try {
                    Map<String, ?> properties = (Map<String, ?>) serializer.readObject(inputStream);
                    Iterator<String> iterator = properties.iterator();
                    while (iterator.hasNext()) {
                        String key = iterator.next();
                        tp.getStyles().put(key, properties.get(key));
                    }
                } catch (SerializationException e) {
                    e.printStackTrace();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        } catch (SecurityException e) {
            e.printStackTrace();
        }
        return tp;
    }

}
