/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.log;

import ch.qos.logback.classic.spi.ILoggingEvent;
import ch.qos.logback.core.CoreConstants;
import ch.qos.logback.core.LayoutBase;

import java.text.SimpleDateFormat;

public class LogLayout extends LayoutBase<ILoggingEvent> {

    private final String TIME_FORMAT = "yyyy/MM/dd HH:mm:ss";

    public String doLayout(ILoggingEvent event) {
        StringBuilder builder = new StringBuilder(128);
        builder.append(" ");
        builder.append(new SimpleDateFormat(TIME_FORMAT).format(event.getTimeStamp()));
        builder.append(" ");
        builder.append(String.format("%6s", event.getLevel()));
        builder.append(" [");
        builder.append(event.getThreadName());
        builder.append("] ");

        StackTraceElement[] stackTraceElements = event.getCallerData();
        StringBuilder builder2 = new StringBuilder(128);
        for (StackTraceElement stackTraceElement : stackTraceElements) {
            builder2.append(stackTraceElement.getFileName().split("\\.")[0]);
            builder2.append(":");
            builder2.append(stackTraceElement.getMethodName());
            builder2.append(" (");
            builder2.append(String.format("%d", stackTraceElement.getLineNumber()));
            builder2.append(") ");
            break;
        }

        builder.append(String.format("%-50s", builder2.toString()));
        builder.append(event.getFormattedMessage());
        builder.append(CoreConstants.LINE_SEPARATOR);
        return builder.toString();
    }
}