/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */
package jp.pegasus.main;

import jp.pegasus.config.Config;
import jp.pegasus.pivot.model.SSElementalData;
import jp.pegasus.util.JsonUtil;
import org.apache.pivot.collections.ArrayList;
import org.apache.pivot.collections.HashMap;
import org.eclipse.jgit.api.*;
import org.eclipse.jgit.api.errors.*;
import org.eclipse.jgit.lib.Repository;
import org.eclipse.jgit.storage.file.FileBasedConfig;
import org.eclipse.jgit.util.FileUtils;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.Date;

public class Initialize {

    public static void initialize() {
        final File repDataDir = new File(Config.GIT_REP_DATA_DIR);
        final File appDataDir = new File(Config.APP_DATA_DIR);
        final File dataFile = new File(Config.APP_DATA_FILE_PATH);
        final File cellDataDir = new File(Config.APP_CELL_DATA_DIR);
        final File textDataDir = new File(Config.APP_TEXT_DATA_DIR);
        Git git = null;

        try {
            /**
             * Git Init
             */
            if (!repDataDir.exists()) {
                new InitCommand() {{
                    this.setDirectory(repDataDir);
                    this.call();
                }};
            }

            /**
             * Git Clone
             */
            if (!appDataDir.exists()) {
                final URI repUri = repDataDir.toURI();
                final String uriString = repUri.toString();
                CloneCommand clone = new CloneCommand() {{
                    this.setDirectory(appDataDir);
                    this.setURI(uriString);
                }};
                git = clone.call();

                /**
                 * Git Config
                 */
                final Repository repository = git.getRepository();
                final FileBasedConfig config = (FileBasedConfig) repository.getConfig();
                config.setString("user", null, "name", Config.GIT_CFG_DEF_USR_NM);
                config.setString("user", null, "email", Config.GIT_CFG_DEF_USR_ML);
                config.save();
            } else {
                git = Git.open(appDataDir);
            }

            /**
             * Git Add
             */
            if (!dataFile.exists()) {
                /**
                 * Data File
                 */
                FileUtils.createNewFile(dataFile);
                AddCommand add = git.add();
                add.addFilepattern(Config.APP_DATA_FILE_NAME);

                /**
                 * Cell Data File
                 */
                FileUtils.mkdir(cellDataDir, true);
                final ArrayList<HashMap<String, SSElementalData>> cellDataList = SSElementalData.createCellDataList();
                final String cellDataDigest = JsonUtil.getDigest(cellDataList, Config.DIGEST_ARGORITHM);
                final String cellDataFilePath = new StringBuilder(Config.APP_CELL_DATA_DIR).append(File.separator).append(cellDataDigest).toString();
                JsonUtil.writeSheetData(cellDataList, cellDataFilePath);
                add.addFilepattern(new StringBuilder("cell").append('/').append(cellDataDigest).toString());

                /**
                 * Text Data File
                 */
                FileUtils.mkdir(textDataDir, true);
                final SSElementalData elementalData = new SSElementalData();
                final String textDataDigest = JsonUtil.getDigest(elementalData, Config.DIGEST_ARGORITHM);
                final String textDataFilePath = new StringBuilder(Config.APP_TEXT_DATA_DIR).append(File.separator).append(textDataDigest).toString();
                JsonUtil.writeSheetData(elementalData, textDataFilePath);
                add.addFilepattern(new StringBuilder("text").append('/').append(textDataDigest).toString());

                add.call();
            }

            /**
             * Git Commit
             */
            CommitCommand commit = git.commit();
            commit.setMessage("commit: " + new Date()).call();

            /**
             * Git Push
             */
            PushCommand push = git.push();
            push.call();

            /**
             * Git Log
             */
//            LogCommand log = git.log();
//            Iterator<RevCommit> revCommitI = log.call().iterator();
//            while(revCommitI.hasNext()) {
//                RevCommit revCommit = revCommitI.next();
//                revCommit.toString();
//            }


        } catch (ConcurrentRefUpdateException e) {
            e.printStackTrace();
        } catch (InvalidRemoteException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (NoFilepatternException e) {
            e.printStackTrace();
        } catch (NoHeadException e) {
            e.printStackTrace();
        } catch (NoMessageException e) {
            e.printStackTrace();
        } catch (WrongRepositoryStateException e) {
            e.printStackTrace();
        }
    }
}
