/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.content;

import org.apache.pivot.beans.BeanAdapter;
import org.apache.pivot.collections.Dictionary;
import org.apache.pivot.wtk.Border;
import org.apache.pivot.wtk.Component;
import org.apache.pivot.wtk.TableView;
import org.apache.pivot.wtk.TableView.CellRenderer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.awt.*;

public class SSCellBorderRenderer extends Border implements CellRenderer {
    private Logger log = LoggerFactory.getLogger(SSCellBorderRenderer.class);

    public SSCellBorderRenderer() {

        getStyles().put("thickness", 1);
//        getStyles().put("thickness", 0);
        getStyles().put("color", new Color(255, 0, 255, 255));
        getStyles().put("backgroundColor", new Color(0, 0, 255, 255));
    }

    @Override
    public void setSize(int width, int height) {
        super.setSize(width, height);

        validate();
    }

    @Override
    public void render(Object row, int rowIndex, int columnIndex,
                       TableView tableView, String columnName, boolean selected,
                       boolean highlighted, boolean disabled) {
//        log.debug("selected:{}, highlighted:{}", selected, highlighted);
        log.debug("rowIndex:{}, columnIndex:{}", rowIndex, columnIndex);
        //Bounds cellBounds = tableView.getCellBounds(rowIndex, columnIndex);
        renderStyles(tableView, selected, disabled);
    }

    protected void renderStyles(TableView tableView, boolean rowSelected, boolean rowDisabled) {
        Component.StyleDictionary tableViewStyles = tableView.getStyles();
//        Component.StyleDictionary styles = getStyles();

        Color color = new Color(255, 255, 255, 255);
        Color backgroundColor = new Color(255, 255, 255, 255);
        if (tableView.isEnabled() && !rowDisabled) {
            if (rowSelected) {
                if (tableView.isFocused()) {
                    color = new Color(255, 255, 255, 255);
                    backgroundColor = new Color(31, 31, 255, 255);
                } else {
//                    color = new Color(255, 0, 255, 255);
//                    backgroundColor = new Color(255, 0, 255, 255);
                }
            } else {
//                color = new Color(255, 255, 255, 255);
//                backgroundColor = new Color(255, 255, 255, 255);
            }
        } else {
//            color = new Color(255, 255, 255, 255);
//            backgroundColor = new Color(255, 255, 255, 255);
        }

//        tableViewStyles.put("color", color);
//        tableViewStyles.put("backgroundColor", backgroundColor);
    }

    @Override
    @SuppressWarnings("unchecked")
    public String toString(Object row, String columnName) {
        Dictionary<String, Object> dictionary;
        if (row instanceof Dictionary<?, ?>) {
            dictionary = (Dictionary<String, Object>) row;
        } else {
            dictionary = new BeanAdapter(row);
        }

        Object cellData = dictionary.get(columnName);

        return (cellData == null) ? null : cellData.toString();
    }

}
