/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.content;

import jp.pegasus.pivot.ss.SSLabel;
import jp.pegasus.pivot.ss.tablepane.SSTablePane;
import org.apache.pivot.wtk.*;
import org.apache.pivot.wtk.Button;
import org.apache.pivot.wtk.Insets;
import org.apache.pivot.wtk.Menu;
import org.apache.pivot.wtk.content.ButtonData;
import org.apache.pivot.wtk.content.MenuItemData;
import org.apache.pivot.wtk.media.Image;

import java.awt.*;

public class SSMenuItemDataRenderer extends SSTablePane implements Button.DataRenderer {
    protected ImageView imageView = new ImageView();
    protected SSLabel textLabel = new SSLabel();
    protected SSLabel keyboardShortcutLabel = new SSLabel();

    public SSMenuItemDataRenderer() {
        getStyles().put("padding", new Insets(2));

        getColumns().add(new SSTablePane.Column(1, true));
        getColumns().add(new SSTablePane.Column());

        BoxPane boxPane = new BoxPane();
        boxPane.add(imageView);
        boxPane.add(textLabel);
        boxPane.getStyles().put("verticalAlignment", VerticalAlignment.CENTER);
        boxPane.getStyles().put("padding", new Insets(0, 0, 0, 6));

        TablePane.Row row = new TablePane.Row();
        row.add(boxPane);
        row.add(keyboardShortcutLabel);

        getRows().add(row);

        imageView.getStyles().put("backgroundColor", null);
    }

    @Override
    public void setSize(int width, int height) {
        super.setSize(width, height);

        // Since this component doesn't have a parent, it won't be validated
        // via layout; ensure that it is valid here
        validate();
    }

    @Override
    public void render(Object data, Button button, boolean highlighted) {
        Image icon = null;
        String text = null;
        Keyboard.KeyStroke keyboardShortcut = null;

        if (data instanceof ButtonData) {
            ButtonData buttonData = (ButtonData) data;
            icon = buttonData.getIcon();
            text = buttonData.getText();

            if (buttonData instanceof MenuItemData) {
                MenuItemData menuItemData = (MenuItemData) buttonData;
                keyboardShortcut = menuItemData.getKeyboardShortcut();
            }
        } else if (data instanceof Image) {
            icon = (Image) data;
        } else {
            if (data != null) {
                text = data.toString();
            }
        }

        // If the button is selected, icon is a checkmark; otherwise,
        // attempt to retrieve icon from button data
        if (button.isSelected()) {
            icon = (Image) button.getStyles().get("checkmarkImage");
        }

        // Update the image view
        Menu.Item menuItem = (Menu.Item) button;
        Menu menu = (Menu) menuItem.getParent();

        int margin = (Integer) menu.getStyles().get("margin");
        Insets padding = (Insets) getStyles().get("padding");

        imageView.setImage(icon);
        imageView.setPreferredWidth(margin - padding.left * 2);
        imageView.getStyles().put("opacity", button.isEnabled() ? 1.0f : 0.5f);

        // Update the labels
        textLabel.setText(text);

        Font font = (Font) menu.getStyles().get("font");
        textLabel.getStyles().put("font", font);
        keyboardShortcutLabel.getStyles().put("font", font.deriveFont(Font.ITALIC));

        Color color;
        if (button.isEnabled()) {
            if (highlighted) {
                color = (Color) menu.getStyles().get("activeColor");
            } else {
                color = (Color) menu.getStyles().get("color");
            }
        } else {
            color = (Color) menu.getStyles().get("disabledColor");
        }

        textLabel.getStyles().put("color", color);
        keyboardShortcutLabel.getStyles().put("color", color);

        boolean showKeyboardShortcuts = false;
        if (menu.getStyles().containsKey("showKeyboardShortcuts")) {
            showKeyboardShortcuts = (Boolean) menu.getStyles().get("showKeyboardShortcuts");
        }

        if (showKeyboardShortcuts) {
            keyboardShortcutLabel.setVisible(true);
            keyboardShortcutLabel.setText(keyboardShortcut == null ? null : keyboardShortcut.toString());
        } else {
            keyboardShortcutLabel.setVisible(false);
        }
    }

    @Override
    public String toString(Object data) {
        String string = null;

        if (data instanceof ButtonData) {
            ButtonData buttonData = (ButtonData) data;
            string = buttonData.getText();
        } else {
            if (data != null) {
                string = data.toString();
            }
        }

        return string;
    }
}
