/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.content;

import jp.pegasus.pivot.ss.tableview.SSTableView;
import org.apache.pivot.json.JSON;
import org.apache.pivot.wtk.Component;
import org.apache.pivot.wtk.HorizontalAlignment;
import org.apache.pivot.wtk.VerticalAlignment;

import java.awt.*;

public class SSTableViewCellRenderer extends org.apache.pivot.wtk.Label implements SSTableView.CellRenderer {

    public SSTableViewCellRenderer() {
        getStyles().put("font", "ＭＳ Ｐゴシック 12");
        getStyles().put("verticalAlignment", VerticalAlignment.CENTER);
        getStyles().put("horizontalAlignment", HorizontalAlignment.CENTER);
        getStyles().put("padding", new org.apache.pivot.wtk.Insets(2));
    }

    @Override
    public void setSize(int width, int height) {
        super.setSize(width, height);

        validate();
    }

    @Override
    public void render(Object row, int rowIndex, int columnIndex,
                       SSTableView tableView, String columnName, boolean isSelected, boolean isCopied, boolean isCutOut, boolean isPasted,
                       boolean isHighlighted, boolean disabled) {
        //Bounds cellBounds = tableView.getCellBounds(rowIndex, columnIndex);
        renderStyles(tableView, isSelected, disabled);

        String text = null;
        if (row != null && columnName != null) {
            text = toString(row, columnName);
        }

        setText(text == null ? "" : text);
    }

    protected void renderStyles(SSTableView tableView, boolean rowSelect, boolean rowDisabled) {
        Component.StyleDictionary tableViewStyles = tableView.getStyles();
        Component.StyleDictionary styles = getStyles();

        Font font = (Font) tableViewStyles.get("font");
        styles.put("font", font);

        Color color;
        if (tableView.isEnabled() && !rowDisabled) {
            if (rowSelect) {
                if (tableView.isFocused()) {
                    color = (Color) tableViewStyles.get("selectColor");
                } else {
                    color = (Color) tableViewStyles.get("inactiveSelectColor");
                }
            } else {
                color = (Color) tableViewStyles.get("color");
            }
        } else {
            color = (Color) tableViewStyles.get("disabledColor");
        }

        styles.put("color", color);
    }

    @Override
    public String toString(Object row, String columnName) {
        Object cellData = JSON.get(row, columnName);
        return (cellData == null) ? null : cellData.toString();

//        SSData.CellData cellData = (SSData.CellData)JSON.get(row, columnName);
//        return (cellData == null) ? null : cellData.text;
//        Object cellData = JSON.get(row, columnName);
//        return (cellData == null) ? null : cellData.toString();
    }
}
