/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.content;

import jp.pegasus.pivot.ss.tableview.SSTableView;
import jp.pegasus.pivot.ss.tableview.SSTableViewHeader;
import org.apache.pivot.wtk.*;
import org.apache.pivot.wtk.Component;
import org.apache.pivot.wtk.Insets;
import org.apache.pivot.wtk.Label;
import org.apache.pivot.wtk.content.TableViewHeaderData;
import org.apache.pivot.wtk.media.Image;

import java.awt.*;

public class SSTableViewHeaderDataRenderer extends BoxPane implements SSTableView.HeaderDataRenderer {
    protected ImageView imageView = new ImageView();
    protected Label label = new Label();

    public SSTableViewHeaderDataRenderer() {
        getStyles().put("horizontalAlignment", HorizontalAlignment.CENTER);
        getStyles().put("verticalAlignment", VerticalAlignment.CENTER);
        getStyles().put("padding", new Insets(1, 2, 1, 2));

        add(imageView);
        add(label);
    }

    @Override
    public void setSize(int width, int height) {
        super.setSize(width, height);

        // Since this component doesn't have a parent, it won't be validated
        // via layout; ensure that it is valid here
        validate();
    }

    @Override
    public void render(Object data, int columnIndex, SSTableViewHeader tableViewHeader,
                       String columnName, boolean highlighted) {
        Image icon = null;
        String text = null;

        if (data instanceof TableViewHeaderData) {
            TableViewHeaderData tableViewHeaderData = (TableViewHeaderData) data;
            icon = tableViewHeaderData.getIcon();
            text = tableViewHeaderData.getText();
        } else if (data instanceof Image) {
            icon = (Image) data;
        } else {
            if (data != null) {
                text = data.toString();
            }
        }

        // Update the icon image view
        imageView.setImage(icon);

        if (icon == null) {
            imageView.setVisible(false);
        } else {
            imageView.setVisible(true);
            imageView.getStyles().put("opacity", tableViewHeader.isEnabled() ? 1.0f : 0.5f);
        }

        // Show/hide the label
        label.setText(text);

        if (text == null) {
            label.setVisible(false);
        } else {
            label.setVisible(true);

            // Update the label styles
            Component.StyleDictionary labelStyles = label.getStyles();
            //label.getStyles().put("horizontalAlignment", HorizontalAlignment.CENTER);

            Object labelFont = tableViewHeader.getStyles().get("font");
            if (labelFont instanceof Font) {
                labelStyles.put("font", labelFont);
            }

            Object color = null;
            if (highlighted) {
                color = Color.WHITE;
            } else {
                if (tableViewHeader.isEnabled()) {
                    color = tableViewHeader.getStyles().get("color");
                } else {
                    color = tableViewHeader.getStyles().get("disabledColor");
                }
            }
            if (color instanceof Color) {
                labelStyles.put("color", color);
            }
        }
    }

    public String toString(Object data) {
        if (data == null) {
            throw new IllegalArgumentException();
        }

        String string;
        if (data instanceof TableViewHeaderData) {
            TableViewHeaderData tableViewHeaderData = (TableViewHeaderData) data;
            string = tableViewHeaderData.getText();
        } else {
            string = data.toString();
        }

        return string;
    }
}
