/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.content;

import jp.pegasus.pivot.model.SSElementalData;
import jp.pegasus.pivot.ss.SSTextArea;
import jp.pegasus.pivot.ss.tableview.SSTableView;
import jp.pegasus.util.ColorUtil;
import org.apache.pivot.collections.HashMap;
import org.apache.pivot.json.JSON;
import org.apache.pivot.wtk.Component;
import org.apache.pivot.wtk.Insets;

import java.awt.*;

public class SSTableViewTextAreaCellRenderer extends SSTextArea implements SSTableView.CellRenderer {

    public SSTableViewTextAreaCellRenderer() {
        getStyles().put("font", "ＭＳ Ｐゴシック 12");
        getStyles().put("margin", new Insets(2));
    }

    @Override
    public void setSize(int width, int height) {
        super.setSize(width, height);

        // Since this component doesn't have a parent, it won't be validated
        // via layout; ensure that it is valid here
        validate();
    }

    public void render(Object row, int rowIndex, int columnIndex,
                       SSTableView tableView, String columnName,
                       boolean isSelect, boolean isCopied, boolean isCutOut, boolean isPasted, boolean highlighted, boolean isDisabled) {
        renderStyles(tableView, isSelect, isCopied, isCutOut, isPasted, isDisabled, rowIndex, columnIndex);

        String text = null;
        if (row != null && columnName != null) {
            text = toString(row, columnName);
        }

        setText(text == null ? "" : text);
    }

    private void renderStyles(SSTableView tableView, boolean isSelect, boolean isCopied, boolean isCutOut, boolean isPasted, boolean rowDisabled, int rowIndex, int columnIndex) {
        Component.StyleDictionary tableViewStyles = tableView.getStyles();
        Component.StyleDictionary styles = getStyles();

        Font font = (Font) tableViewStyles.get("font");
        styles.put("font", font);

        Color color;
        if (false) {
        }
        /**
         * Copy + Paste + Select
         */
        else if (isCopied && !isCutOut && isPasted && isSelect) {
            Color copyColor = (Color) tableViewStyles.get("copyColor");
            Color pasteColor = (Color) tableViewStyles.get("pasteColor");
            Color selectColor = (Color) tableViewStyles.get("selectColor");

            color = ColorUtil.blend(ColorUtil.blend(copyColor, pasteColor), selectColor);
        }
        /**
         * Cut + Paste + Select
         */
        else if (!isCopied && isCutOut && isPasted && isSelect) {
            Color cutColor = (Color) tableViewStyles.get("cutColor");
            Color pasteColor = (Color) tableViewStyles.get("pasteColor");
            Color selectColor = (Color) tableViewStyles.get("selectColor");

            color = ColorUtil.blend(ColorUtil.blend(cutColor, pasteColor), selectColor);
        }
        /**
         * Paste + Select
         */
        else if (!isCopied && isPasted && isSelect) {
            Color pasteColor = (Color) tableViewStyles.get("pasteColor");
            Color selectColor = (Color) tableViewStyles.get("selectColor");

            color = ColorUtil.blend(pasteColor, selectColor);
        }
        /**
         * Copy + Select
         */
        else if (isCopied && !isCutOut && !isPasted && isSelect) {
            Color copyColor = (Color) tableViewStyles.get("copyColor");
            Color selectColor = (Color) tableViewStyles.get("selectColor");

            color = ColorUtil.blend(copyColor, selectColor);
        }
        /**
         * Cut + Select
         */
        else if (!isCopied && isCutOut && !isPasted && isSelect) {
            Color cutColor = (Color) tableViewStyles.get("cutColor");
            Color selectColor = (Color) tableViewStyles.get("selectColor");

            color = ColorUtil.blend(cutColor, selectColor);
        }
        /**
         * Select
         */
        else if (!isCopied && !isPasted && isSelect) {
            color = (Color) tableViewStyles.get("selectColor");
        }
        /**
         * Copy + Paste
         */
        else if (isCopied && !isCutOut && isPasted && !isSelect) {
            Color copyColor = (Color) tableViewStyles.get("copyColor");
            Color pasteColor = (Color) tableViewStyles.get("pasteColor");

            color = ColorUtil.blend(copyColor, pasteColor);
        }
        /**
         * Cut + Paste
         */
        else if (!isCopied && isCutOut && isPasted && !isSelect) {
            Color cutColor = (Color) tableViewStyles.get("cutColor");
            Color pasteColor = (Color) tableViewStyles.get("pasteColor");

            color = ColorUtil.blend(cutColor, pasteColor);
        }
        /**
         * Paste
         */
        else if (!isCopied && isPasted && !isSelect) {
            color = (Color) tableViewStyles.get("pasteColor");
        }
        /**
         * Copy
         */
        else if (isCopied && !isCutOut && !isPasted && !isSelect) {
            color = (Color) tableViewStyles.get("copyColor");
        }
        /**
         * Cut
         */
        else if (!isCopied && isCutOut && !isPasted && !isSelect) {
            color = (Color) tableViewStyles.get("copyColor");
        }
        /**
         * Other
         */
        else if (!isCopied && !isPasted && !isSelect) {
            color = (Color) tableViewStyles.get("color");
        }
        /**
         * Other
         */
        else {
            color = (Color) tableViewStyles.get("color");
        }

        styles.put("color", color);
    }

    @Override
    public String toString(Object row, String columnName) {
//        Logger.log.debug("columnName:{}, row:{}", columnName, row.toString());
        String text = null;
        Object object = JSON.get(row, columnName);

        if (object instanceof SSElementalData) {
            SSElementalData elementalData = (SSElementalData) object;
            text = (elementalData == null) ? null : elementalData.getText();
        } else if (object instanceof HashMap) {
            HashMap cellData = (HashMap) object;
            text = (String) cellData.get("text");
        }

//        String text = null;
//        Object object = null;
//
//        CacheManager manager = CacheManager.getInstance();
//        try {
//            Cache cache = manager.getCache("sampleCache1");
//            Element element = cache.get(columnName);
//
//            if(element == null) {
//                object = JSON.get(row, columnName);
//                cache.put(new Element(columnName, object));
//            } else {
//                object = element.getValue();
//            }
//
//        } finally {
//            manager.shutdown();
//        }

        return text;
    }

}
