/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */
package jp.pegasus.pivot.effects;

import org.apache.pivot.wtk.Bounds;
import org.apache.pivot.wtk.Component;
import org.apache.pivot.wtk.effects.Decorator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.awt.*;
import java.awt.geom.AffineTransform;

public class SSBorderDecorator implements Decorator {
    private Logger log = LoggerFactory.getLogger(SSBorderDecorator.class);

    private final Color SELECT_CELL_COLOR = new Color(223, 127, 127, 127);
    private final Color SELECT_COLOR = new Color(255, 127, 63, 31);
    private final Color SELECT_RECT_COLOR = new Color(255, 63, 63, 192);
    private final BasicStroke BASIC_STROKE = new BasicStroke(2);

    private Graphics2D graphics = null;
    private Component component = null;

    private int selectX;
    private int selectY;
    private int selectWidth;
    private int selectHeight;

    private int selectCellX;
    private int selectCellY;
    private int selectCellWidth;
    private int selectCellHeight;

    @Override
    public Graphics2D prepare(Component component, Graphics2D graphics) {
        this.component = component;
        this.graphics = graphics;

        //log.debug("");

        graphics.setColor(SELECT_CELL_COLOR);
        graphics.fillRect(
        selectCellX,
        selectCellY,
        selectCellWidth,
        selectCellHeight
        );

        graphics.setColor(SELECT_COLOR);
        graphics.fillRect(
        selectX,
        selectY,
        selectWidth,
        selectHeight
        );

        graphics.setColor(SELECT_RECT_COLOR);
        graphics.setStroke(BASIC_STROKE);
        graphics.drawRect(
        selectX - 1,
        selectY - 1,
        selectWidth + 1,
        selectHeight + 1
        );

        graphics.drawRect(
        selectX + selectWidth - 1,
        selectY + selectHeight - 1,
        3,
        3
        );

        return graphics;
    }

    @Override
    public void update() {
//    	component.paint(graphics);
//        graphics = null;
        prepare(component, graphics);
    }

    @Override
    public Bounds getBounds(Component component) {
        return new Bounds(-1, -1, component.getWidth() + 1, component.getHeight() + 1);
    }

    @Override
    public AffineTransform getTransform(Component component) {
        return new AffineTransform();
    }

    public Bounds getSelectBounds() {
        return new Bounds(selectX, selectY, selectWidth, selectHeight);
    }

    public void setSelectBounds(Bounds selectBounds) {
        selectX = selectBounds.x;
        selectY = selectBounds.y;
        selectWidth = selectBounds.width;
        selectHeight = selectBounds.height;
    }

    public void setSelectX(int selectX) {
        this.selectX = selectX;
    }

    public void setSelectY(int selectY) {
        this.selectY = selectY;
    }

    public void setSelectWidth(int selectWidth) {
        this.selectWidth = selectWidth;
    }

    public void setSelectHeight(int selectHeight) {
        this.selectHeight = selectHeight;
    }

    public Bounds getSelectCellBounds() {
        return new Bounds(selectCellX, selectCellY, selectCellWidth, selectCellHeight);
    }

    public void setSelectCellBounds(Bounds selectCellBounds) {
        selectCellX = selectCellBounds.x;
        selectCellY = selectCellBounds.y;
        selectCellWidth = selectCellBounds.width;
        selectCellHeight = selectCellBounds.height;
    }

    public void setSelectCellX(int selectCellX) {
        this.selectCellX = selectCellX;
    }

    public void setSelectCellY(int selectCellY) {
        this.selectCellY = selectCellY;
    }

    public void setSelectCellWidth(int selectCellWidth) {
        this.selectCellWidth = selectCellWidth;
    }

    public void setSelectCellHeight(int selectCellHeight) {
        this.selectCellHeight = selectCellHeight;
    }
}
