/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.factory;

import jp.pegasus.config.Config;
import jp.pegasus.config.Location;
import jp.pegasus.log.Logger;
import jp.pegasus.pivot.factory.tableview.SSTableViewColumnHeaderFactory;
import jp.pegasus.pivot.factory.tableview.SSTableViewFactory;
import jp.pegasus.pivot.factory.tableview.SSTableViewRowHeaderFactory;
import jp.pegasus.pivot.ss.SSCorner;
import jp.pegasus.pivot.ss.SSScrollPane;
import jp.pegasus.pivot.ss.tableview.SSTableView;
import jp.pegasus.pivot.ss.tableview.SSTableViewHeader;
import jp.pegasus.util.JsonUtil;
import org.apache.pivot.wtk.Component;
import org.apache.pivot.wtk.ComponentMouseButtonListener;
import org.apache.pivot.wtk.ComponentMouseListener;
import org.apache.pivot.wtk.Mouse;

public class SSCornerFactory {

    public static SSCorner createCorner() {
        SSCorner corner = new SSCorner();
        JsonUtil.setStyles(corner, Location.CORNER);

        corner.getComponentMouseButtonListeners().add(new ComponentMouseButtonListener() {

            @Override
            public boolean mouseDown(Component component, Mouse.Button button, int x, int y) {
                SSCorner corner = (SSCorner) component;
                SSScrollPane scrollPane = (SSScrollPane) corner.getParent();
                SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) scrollPane.getColumnHeader();
                SSTableView tableViewRowHeader = (SSTableView) scrollPane.getRowHeader();
                SSTableView tableView = (SSTableView) scrollPane.getView();
                tableView.setSelect(true);

                if (!tableView.isMouseDown() && button == Mouse.Button.LEFT) {

                    if (corner.isPressed()) {
                        tableView.setSelectCurrentIndexQuartet(0x0000000000000000l);
                        setDefaultSkin(corner);
                    } else {
                        tableView.setSelectCurrentIndexQuartet(Config.SELECT_ALL_INDEX);
//                        tableView.setSelectCurrentIndex(0xFFFFFFFF00000000l & (((long) (MAX_COLUMN_NUMBER - 2) << 48) | (long) (MAX_ROW_NUMBER - 2) << 32));
                        //Logger.log.info("setSelectCurrentIndex:{}", 0x00000000FFFFFFFF & (((long) MAX_COLUMN_NUMBER - 2 << 16) | (long) MAX_ROW_NUMBER - 2));
                        setSelectSkin(corner);
                    }

                    tableViewRowHeader.setSelect(true);
                    tableViewColumnHeader.setHeaderPressed(true);
                    tableViewColumnHeader.setSelect(true);
                    tableViewColumnHeader.getStyles().put("columnsResizable", false);
                    tableView.setMouseDown(true);

                    SSTableViewColumnHeaderFactory.refreshDecorator(tableViewColumnHeader);
                    SSTableViewRowHeaderFactory.refreshDecorator(tableViewRowHeader);
                    SSTableViewFactory.refreshDecorator(tableView);
                }

                return false;
            }

            @Override
            public boolean mouseUp(Component component, Mouse.Button button, int x, int y) {
                SSCorner corner = (SSCorner) component;
                SSScrollPane scrollPane = (SSScrollPane) corner.getParent();
                SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) scrollPane.getColumnHeader();
                SSTableView tableViewRowHeader = (SSTableView) scrollPane.getRowHeader();
                SSTableView tableView = (SSTableView) scrollPane.getView();

                if (tableView.isSelect()) {
                    SSTableViewColumnHeaderFactory.refreshDecorator(tableViewColumnHeader);
                    SSTableViewRowHeaderFactory.refreshDecorator(tableViewRowHeader);
                    SSTableViewFactory.refreshDecorator(tableView);

                    tableViewColumnHeader.setHeaderPressed(false);
                    tableViewColumnHeader.getStyles().put("columnsResizable", true);
                    tableViewRowHeader.setHeaderPressed(false);
                    tableView.setMouseDown(false);
                }
                return false;
            }

            @Override
            public boolean mouseClick(Component component, Mouse.Button button, int x, int y, int count) {
//                Logger.log.debug("");
                return false;
            }
        });

        corner.getComponentMouseListeners().add(new ComponentMouseListener() {

            @Override
            public boolean mouseMove(Component component, int x, int y) {
                //ScrollPaneFactory.dumpIndexes("corner:mouseMove");

//                if (ScrollPaneFactory.getMouseDown()) {
//                    SSCorner corner = (SSCorner) component;
//                    SSScrollPane scrollPane = (SSScrollPane) corner.getParent();
//                    SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) scrollPane.getColumnHeader();
//                    SSTableView tableViewRowHeader = (SSTableView) scrollPane.getRowHeader();
//                    SSTableView tableView = (SSTableView) scrollPane.getView();
//
//                    ScrollPaneFactory.setIndexes(0, 0, 0, 0);
//
//                    SSTableViewColumnHeaderFactory.refreshDecorator(tableViewColumnHeader);
//                    SSTableViewRowHeaderFactory.refreshDecorator(tableViewRowHeader);
//                    SSTableViewFactory.refreshDecorator(tableView);
//                    setDefaultSkin(corner);
//                }

//                if (
//                        !ScrollPaneFactory.getMouseDown() &&
//                                ScrollPaneFactory.getCurrentRowIndex() != -1 &&
//                                ScrollPaneFactory.getCurrentColumnIndex() != -1 &&
//                                ScrollPaneFactory.getSelectRowIndex() != -1 &&
//                                ScrollPaneFactory.getSelectColumnIndex() != -1) {
//                    SSCorner corner = (SSCorner) component;
//                    SSScrollPane scrollPane = (SSScrollPane) corner.getParent();
//                    SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) scrollPane.getColumnHeader();
//                    SSTableView tableViewRowHeader = (SSTableView) scrollPane.getRowHeader();
//                    SSTableView tableView = (SSTableView) scrollPane.getView();
//
//                    SSTableViewColumnHeaderFactory.refreshDecorator(tableViewColumnHeader);
//                    SSTableViewRowHeaderFactory.refreshDecorator(tableViewRowHeader);
//                    SSTableViewFactory.refreshDecorator(tableView);
//                }
                return false;
            }

            @Override
            public void mouseOver(Component component) {
                Logger.log.debug("");
            }

            @Override
            public void mouseOut(Component component) {
                Logger.log.debug("");
            }
        });

//        corner.getComponentMouseButtonListeners().add(new ComponentMouseButtonListener(){
//            @Override
//            public boolean mouseDown(Component component, Mouse.Button button, int x, int y) {
//                log.debug("");
//                SSCorner corner = (SSCorner)component;
//                JsonUtil.setStyles(corner, LOCATION_SELECTED);
//                return false;
//            }
//
//            @Override
//            public boolean mouseUp(Component component, Mouse.Button button, int x, int y) {
//                log.debug("");
//                SSCorner corner = (SSCorner)component;
//                JsonUtil.setStyles(corner, LOCATION);
//                return false;
//            }
//
//            @Override
//            public boolean mouseClick(Component component, Mouse.Button button, int x, int y, int count) {
//                log.debug("");
//                return false;
//            }
//        });

        return corner;
    }

    public static void setDefaultSkin(SSCorner corner) {
        corner.setPressed(false);
        JsonUtil.setStyles(corner, Location.CORNER);
    }

    public static void setSelectSkin(SSCorner corner) {
        corner.setPressed(true);
        JsonUtil.setStyles(corner, Location.CORNER_SELECTED);
    }

}
