/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.factory;

import jp.pegasus.config.Config;
import jp.pegasus.config.Location;
import jp.pegasus.log.Logger;
import jp.pegasus.main.Initialize;
import jp.pegasus.pivot.factory.treeview.SSTreeViewOfPopupSpreadSheetFactory;
import jp.pegasus.pivot.factory.treeview.SSTreeViewOfPopupTabPaneFactory;
import jp.pegasus.pivot.factory.treeview.SSTreeViewOfPopupWindowMenuFactory;
import jp.pegasus.pivot.model.SSData;
import jp.pegasus.pivot.skin.SSTabPaneSkin;
import jp.pegasus.pivot.ss.*;
import jp.pegasus.pivot.ss.tableview.SSTableView;
import jp.pegasus.pivot.util.SSTreeViewUtil;
import jp.pegasus.util.JsonUtil;
import org.apache.pivot.collections.ArrayList;
import org.apache.pivot.wtk.*;
import org.apache.pivot.wtk.content.ButtonData;

public class SSFrameFactory {

    public static SSFrame createFrame() {
        final SSFrame frame = new SSFrame() {
            //            SSData data = JsonUtil.readGrandData(Config.APP_DATA_FILE_PATH);
            {
//                this.setAttribute(SSData.DEFAULT_TYPE, data);
                this.setTitle(String.format("%s Ver %s", Config.TITLE, Config.VERSION));
                this.setMaximized(true);

                JsonUtil.setStyles(this, Location.FRAME);
                Initialize.initialize();
            }
        };

        final SSTreeNode.NodeType mnuFld = SSTreeNode.NodeType.MnuFld;
        final SSTreeNode.NodeType mnuFilGrid = SSTreeNode.NodeType.MnuFilGrid;
        final SSTreeNode.NodeType mnuFilText = SSTreeNode.NodeType.MnuFilText;
        final SSTreeNode.NodeType mnuCpy = SSTreeNode.NodeType.MnuCpy;
        final SSTreeNode.NodeType mnuDel = SSTreeNode.NodeType.MnuDel;
        final SSTreeNode.NodeType mnuExt = SSTreeNode.NodeType.MnuExt;
        final SSTreeNode.NodeType mnuSrh = SSTreeNode.NodeType.MnuSrh;
        final SSTreeNode.NodeType mnuHlp = SSTreeNode.NodeType.MnuHlp;

        frame.setMenuHandler(new MenuHandler() {
            @Override
            public void configureMenuBar(Component component, MenuBar menuBar) {
                if (component instanceof SSFrame) {
                    SSTreeView treeView = getTreeViewFromFrame((SSFrame) component);

                    if (treeView == null || treeView.getSelectedNode() == null || treeView.getSelectedPath() == null) {
                        Action.getNamedActions().get(mnuCpy.toString()).setEnabled(false);
                        Action.getNamedActions().get(mnuDel.toString()).setEnabled(false);
                    } else {
                        Action.getNamedActions().get(mnuCpy.toString()).setEnabled(true);
                        Action.getNamedActions().get(mnuDel.toString()).setEnabled(true);
                    }
                }
            }

            @Override
            public void cleanupMenuBar(Component component, MenuBar menuBar) {
            }

            @Override
            public boolean configureContextMenu(Component component, Menu menu, int x, int y) {
                return false;
            }
        });

        Action.getNamedActions().put(mnuFld.toString(), new Action() {
            @Override
            public void perform(Component source) {
            }
        });
        Action.getNamedActions().put(mnuFilGrid.toString(), new Action() {
            @Override
            public void perform(Component source) {
                if (source != null & source.getWindow() != null) {
                    Window frame = source.getWindow().getOwner();
                    if (frame instanceof SSFrame) {
                        SSTreeView treeView = getTreeViewFromFrame((SSFrame) frame);
                        SSTreeViewUtil.addFile(treeView, SSTreeNode.NodeType.MnuFilGrid);
                    } else {
                        Window frame2 = frame.getOwner();
                        if (frame2 instanceof SSFrame) {
                            SSTreeView treeView = getTreeViewFromFrame((SSFrame) frame2);
                            SSTreeViewUtil.addFile(treeView, SSTreeNode.NodeType.MnuFilGrid);
                        }
                    }
                }
            }
        });
        Action.getNamedActions().put(mnuFilText.toString(), new Action() {
            @Override
            public void perform(Component source) {
                if (source != null & source.getWindow() != null) {
                    Window frame = source.getWindow().getOwner();
                    if (frame instanceof SSFrame) {
                        SSTreeView treeView = getTreeViewFromFrame((SSFrame) frame);
                        SSTreeViewUtil.addFile(treeView, SSTreeNode.NodeType.MnuFilText);
                    } else {
                        Window frame2 = frame.getOwner();
                        if (frame2 instanceof SSFrame) {
                            SSTreeView treeView = getTreeViewFromFrame((SSFrame) frame2);
                            SSTreeViewUtil.addFile(treeView, SSTreeNode.NodeType.MnuFilText);
                        }
                    }
                }
            }
        });
        Action.getNamedActions().put(mnuCpy.toString(), new Action() {
            @Override
            public void perform(Component source) {
                if (source != null & source.getWindow() != null) {
                    Window frame = source.getWindow().getOwner();
                    if (frame instanceof SSFrame) {
                        SSTreeView treeView = getTreeViewFromFrame((SSFrame) frame);
                        SSTreeViewUtil.cpyFile(treeView);
                    }
                }
            }
        });
        Action.getNamedActions().put(mnuDel.toString(), new Action() {
            @Override
            public void perform(Component source) {
                if (source != null & source.getWindow() != null) {
                    Window frame = source.getWindow().getOwner();
                    if (frame instanceof SSFrame) {
                        SSTreeView treeView = getTreeViewFromFrame((SSFrame) frame);
                        SSTreeViewUtil.delFile(treeView);
                    }
                }
            }
        });
        Action.getNamedActions().put(mnuExt.toString(), new Action() {
            @Override
            public void perform(Component source) {
                if (source instanceof SSFrame) {
                    SSData data = (SSData) source.getAttribute(SSData.DEFAULT_TYPE);
                    JsonUtil.writeGrandData(data, Config.APP_DATA_FILE_PATH);
                }
                DesktopApplicationContext.exit();
            }
        });
        Action.getNamedActions().put(mnuSrh.toString(), new Action() {
            @Override
            public void perform(Component source) {
            }
        });
        Action.getNamedActions().put(mnuHlp.toString(), new Action() {
            @Override
            public void perform(Component source) {
            }
        });
        Action.getNamedActions().get(mnuFld.toString()).setEnabled(false);
        Action.getNamedActions().get(mnuFilGrid.toString()).setEnabled(true);
        Action.getNamedActions().get(mnuFilText.toString()).setEnabled(true);
        Action.getNamedActions().get(mnuCpy.toString()).setEnabled(false);
        Action.getNamedActions().get(mnuDel.toString()).setEnabled(false);
        Action.getNamedActions().get(mnuExt.toString()).setEnabled(true);
        Action.getNamedActions().get(mnuSrh.toString()).setEnabled(false);
        Action.getNamedActions().get(mnuHlp.toString()).setEnabled(false);

        frame.getActionMappings().add(new Window.ActionMapping() {{
            setAction(mnuFld.toString());
            setKeyStroke(mnuFld.getShortCutText());
        }});
        frame.getActionMappings().add(new Window.ActionMapping() {{
            setAction(mnuFilGrid.toString());
            setKeyStroke(mnuFilGrid.getShortCutText());
        }});
        frame.getActionMappings().add(new Window.ActionMapping() {{
            setAction(mnuFilText.toString());
            setKeyStroke(mnuFilText.getShortCutText());
        }});
        frame.getActionMappings().add(new Window.ActionMapping() {{
            setAction(mnuCpy.toString());
            setKeyStroke(mnuCpy.getShortCutText());
        }});
        frame.getActionMappings().add(new Window.ActionMapping() {{
            setAction(mnuDel.toString());
            setKeyStroke(mnuDel.getShortCutText());
        }});
        frame.getActionMappings().add(new Window.ActionMapping() {{
            setAction(mnuExt.toString());
            setKeyStroke(mnuExt.getShortCutText());
        }});
        frame.getActionMappings().add(new Window.ActionMapping() {{
            setAction(mnuSrh.toString());
            setKeyStroke(mnuSrh.getShortCutText());
        }});
        frame.getActionMappings().add(new Window.ActionMapping() {{
            setAction(mnuHlp.toString());
            setKeyStroke(mnuHlp.getShortCutText());
        }});

        frame.getComponentKeyListeners().add(new ComponentKeyListener() {
            @Override
            public boolean keyTyped(Component component, char character) {
                Logger.log.debug("character:{}", character);
                return false;
            }

            @Override
            public boolean keyPressed(Component component, int keyCode, Keyboard.KeyLocation keyLocation) {
                Logger.log.debug("keyCode:{}", keyCode);
                if (keyCode == Keyboard.KeyCode.D &&
                        Keyboard.isPressed(Keyboard.Modifier.CTRL)) {
                    DesktopApplicationContext.exit();
                }
                return false;
            }

            @Override
            public boolean keyReleased(Component component, int keyCode, Keyboard.KeyLocation keyLocation) {
                Logger.log.debug("keyCode:{}", keyCode);
                return false;
            }
        });

        frame.getComponentMouseButtonListeners().add(new ComponentMouseButtonListener() {
            private ArrayList<SSPalette> paletteList = new ArrayList<SSPalette>();

            @Override
            public boolean mouseDown(Component component, Mouse.Button button, int x, int y) {
                SSFrame frame = (SSFrame) component;

                if (button == Mouse.Button.RIGHT) {
                    Component descendant = frame.getDescendantAt(x, y);
                    Logger.log.debug("descendant:{}", descendant.toString());

                    /**
                     * Sheet
                     */
                    if (descendant instanceof SSTableView &&
                            SSTableView.Type.SpreadSheet.equals(((SSTableView) descendant).getTableViewType())) {
                        SSPalette palette = ((SSTableView) descendant).getPalette();
                        if (palette == null) {
                            palette = SSPaletteFactory.createPalette(SSPalette.Type.PopupSpreadSheet);
                            palette.setContent(SSTreeViewOfPopupSpreadSheetFactory.createTreeViewOfPopupSpreadSheet((SSTableView) descendant));
                            palette.setLocation(x, y);
                            palette.open(frame);

                            closeAllPallete();

                            ((SSTableView) descendant).setPalette(palette);
                            paletteList.add(palette);
                        } else if (palette.isOpen()) {
                            palette.setLocation(x, y);
                        } else if (palette.isClosed()) {
                            closeAllPallete();
                            palette.setLocation(x, y);
                            palette.open(frame);
                        }
                    }
                    /**
                     * Menu
                     */
                    else if (descendant instanceof SSTreeView &&
                            SSTreeView.Type.WindowMenu.equals(((SSTreeView) descendant).getTreeViewType())) {
                        SSPalette palette = ((SSTreeView) descendant).getPalette();
                        if (palette == null) {
                            palette = SSPaletteFactory.createPalette(SSPalette.Type.PopupWindowMenu);
                            palette.setContent(SSTreeViewOfPopupWindowMenuFactory.createTreeViewOfPopupWindowMenu((SSTreeView) descendant));
                            palette.setLocation(x, y);
                            palette.open(frame);

                            closeAllPallete();

                            ((SSTreeView) descendant).setPalette(palette);
                            paletteList.add(palette);
                        } else if (palette.isOpen()) {
                            palette.setLocation(x, y);
                        } else if (palette.isClosed()) {
                            closeAllPallete();
                            palette.setLocation(x, y);
                            palette.open(frame);
                        }
                    }
                    /**
                     * Tab
                     */
                    else if (descendant instanceof SSTabPaneSkin.TabButton &&
                            SSTabPaneSkin.Type.Sheet.equals(
                                    ((ButtonData) ((SSTabPaneSkin.TabButton) descendant).getButtonData()).getUserData()
                            )
                            ) {
                        SSPalette palette = ((SSTabPaneSkin.TabButton) descendant).getPalette();
                        if (palette == null) {
                            palette = SSPaletteFactory.createPalette(SSPalette.Type.PopupTabPane);
                            palette.setContent(SSTreeViewOfPopupTabPaneFactory.createTreeViewOfPopupTabPane((SSTabPaneSkin.TabButton) descendant));
                            palette.setLocation(x, y);
                            palette.open(frame);

                            closeAllPallete();

                            ((SSTabPaneSkin.TabButton) descendant).setPalette(palette);
                            paletteList.add(palette);
                        } else if (palette.isOpen()) {
                            palette.setLocation(x, y);
                        } else if (palette.isClosed()) {
                            closeAllPallete();
                            palette.setLocation(x, y);
                            palette.open(frame);
                        }
                    } else {
                        Logger.log.info("descendant:{}", descendant.toString());
                    }
                } else {
                    // Close All
                    closeAllPallete();
                }

                return false;
            }

            private void closeAllPallete() {
                for (SSPalette palette : paletteList) {
                    palette.close();
                }
            }

            @Override
            public boolean mouseUp(Component component, Mouse.Button button, int x, int y) {
                return false;
            }

            @Override
            public boolean mouseClick(Component component, Mouse.Button button, int x, int y, int count) {
                return false;
            }
        });

        return frame;
    }

    /**
     * Save Data
     *
     * @param frame
     */
    public static void save(SSFrame frame) {
        final long start = System.currentTimeMillis();

        SSData data = (SSData) frame.getAttribute(SSData.DEFAULT_TYPE);
        JsonUtil.writeGrandData(data, Config.APP_DATA_FILE_PATH);

        Logger.log.debug("Save Time {} ms.", System.currentTimeMillis() - start);
    }

    private static SSTreeView getTreeViewFromFrame(SSFrame frame) {
        SplitPane splitPaneRigt = (SplitPane) frame.getContent();
        SplitPane splitPaneLeft = (SplitPane) splitPaneRigt.getLeft();
        if (splitPaneLeft == null) {
            return null;
        }
        SSTabPane leftTab = (SSTabPane) splitPaneLeft.getLeft();
        if (leftTab == null) {
            return null;
        }
        SSBorder border = (SSBorder) leftTab.getSelectedTab();
        SSScrollPane scrollPane = (SSScrollPane) border.getContent();
        SSTreeView treeView = (SSTreeView) scrollPane.getView();

        return treeView;
    }

}
