/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.factory;

import jp.pegasus.pivot.ss.SSTreeNode;
import org.apache.pivot.wtk.ApplicationContext;
import org.apache.pivot.wtk.Menu;
import org.apache.pivot.wtk.MenuBar;
import org.apache.pivot.wtk.content.MenuItemData;

import java.text.NumberFormat;
import java.util.Date;

public class SSMenuBarItemFactory {

    private static final String DEF_TITLE_FILE_MENU = "File";
    private static final String DEF_TITLE_FILE_SEARCH = "Search";
    private static final String DEF_TITLE_FILE_TOOLS = "Tools";
    private static final String DEF_TITLE_FILE_PREFERENCES = "Preferences";
    private static final String DEF_TITLE_FILE_VERSION_CONTROL = "Version Control";
    private static final String DEF_TITLE_FILE_SETTINGS = "Settings";
    private static final String DEF_TITLE_FILE_HELP = "Help";

    public static MenuBar.Item createFileMenuBarItem() {
        MenuBar.Item menuBarItem = new MenuBar.Item();
        menuBarItem.setButtonData(DEF_TITLE_FILE_MENU);
        menuBarItem.setEnabled(true);

        final SSTreeNode.NodeType mnuGrp = SSTreeNode.NodeType.MnuGrp;
        final SSTreeNode.NodeType mnuFld = SSTreeNode.NodeType.MnuFld;
        final SSTreeNode.NodeType mnuFilGrid = SSTreeNode.NodeType.MnuFilGrid;
        final SSTreeNode.NodeType mnuFilText = SSTreeNode.NodeType.MnuFilText;
        final SSTreeNode.NodeType mnuCpy = SSTreeNode.NodeType.MnuCpy;
        final SSTreeNode.NodeType mnuDel = SSTreeNode.NodeType.MnuDel;
        final SSTreeNode.NodeType mnuExt = SSTreeNode.NodeType.MnuExt;

        menuBarItem.setMenu(new Menu() {{
            getSections().add(new Menu.Section() {{
                add(new Menu.Item() {{
                    setButtonData(new MenuItemData() {{
                        setAction(mnuFld.toString());
                        setIcon(SSImageFactory.getIconImage(mnuFld));
                        setText(mnuFld.toString());
                        setKeyboardShortcut(mnuFld.getShortCutText());
                    }});
                }});
                add(new Menu.Item() {{
                    setMenu(new Menu() {{
                        getSections().add(new Menu.Section() {{
                            add(new Menu.Item() {{
                                setEnabled(true);
                                setButtonData(new MenuItemData() {{
                                    setAction(mnuFilGrid.toString());
                                    setIcon(SSImageFactory.getIconImage(mnuFilGrid));
                                    setText("Grid Sheet");
                                    setKeyboardShortcut(mnuFilGrid.getShortCutText());
                                }});
                            }});
                            add(new Menu.Item() {{
                                setEnabled(true);
                                setButtonData(new MenuItemData() {{
                                    setAction(mnuFilText.toString());
                                    setIcon(SSImageFactory.getIconImage(mnuFilText));
                                    setText("Text Sheet");
                                    setKeyboardShortcut(mnuFilText.getShortCutText());
                                }});
                            }});
                            add(new Menu.Item() {{
                                setEnabled(false);
                                setButtonData("Drawing");
                            }});
                            add(new Menu.Item() {{
                                setEnabled(false);
                                setButtonData("Schaduler");
                            }});
                        }});
                    }});
                    setButtonData(new MenuItemData() {{
                        setIcon(SSImageFactory.getIconImage(mnuGrp));
                        setText(mnuGrp.toString());
                    }});
                }});
                add(new Menu.Item() {{
                    setButtonData(new MenuItemData() {{
                        setAction(mnuCpy.toString());
                        setIcon(SSImageFactory.getIconImage(mnuCpy));
                        setText(mnuCpy.toString());
                        setKeyboardShortcut(mnuCpy.getShortCutText());
                    }});
                }});
                add(new Menu.Item() {{
                    setButtonData(new MenuItemData() {{
                        setAction(mnuDel.toString());
                        setIcon(SSImageFactory.getIconImage(mnuDel));
                        setText(mnuDel.toString());
                        setKeyboardShortcut(mnuDel.getShortCutText());
                    }});
                }});
            }});
            getSections().add(new Menu.Section() {{
                add(new Menu.Item() {{
                    setButtonData(new MenuItemData() {{
                        setAction(mnuExt.toString());
                        setIcon(SSImageFactory.getIconImage(mnuExt));
                        setText(mnuExt.toString());
                        setKeyboardShortcut(mnuExt.getShortCutText());
                    }});
                }});
            }});
        }});

        return menuBarItem;
    }

    public static MenuBar.Item createSearchMenuBarItem() {
        MenuBar.Item menuBarItem = new MenuBar.Item();
        menuBarItem.setButtonData(DEF_TITLE_FILE_SEARCH);
        menuBarItem.setEnabled(true);

        final SSTreeNode.NodeType mnuSrh = SSTreeNode.NodeType.MnuSrh;

        menuBarItem.setMenu(new Menu() {{
            getSections().add(new Menu.Section() {{
                add(new Menu.Item() {{
                    setButtonData(new MenuItemData() {{
                        setAction(mnuSrh.toString());
                        setIcon(SSImageFactory.getIconImage(mnuSrh));
                        setText(mnuSrh.toString());
                        setKeyboardShortcut(mnuSrh.getShortCutText());
                    }});
                }});
            }});
        }});

        return menuBarItem;
    }

    public static MenuBar.Item createToolsMenuBarItem() {
        MenuBar.Item menuBarItem = new MenuBar.Item();
        menuBarItem.setButtonData(DEF_TITLE_FILE_TOOLS);
        menuBarItem.setEnabled(true);
        menuBarItem.setMenu(new Menu() {{
            getSections().add(new Menu.Section() {{
                for (int i = 0; i < 10; i++) {
                    add(new Menu.Item() {{
                        setEnabled(false);
                        setButtonData("Under Construction");
                    }});
                }
            }});
        }});

        return menuBarItem;
    }

    public static MenuBar.Item createPreferenceMenuBarItem() {
        final MenuBar.Item menuBarItem = new MenuBar.Item();
        menuBarItem.setButtonData(DEF_TITLE_FILE_PREFERENCES);
        menuBarItem.setEnabled(true);
        menuBarItem.setMenu(new Menu() {{
            getSections().add(new Menu.Section() {{
                add(new Menu.Item() {{
                    setEnabled(false);
                    setButtonData("Under Construction1");
                }});
                add(new Menu.Item() {{
                    setButtonData("Under Construction2");
                    setMenu(new Menu() {{
                        getSections().add(new Menu.Section() {{
                            add(new Menu.Item() {{
                                setEnabled(false);
                                setButtonData("Under Construction2-1");
                            }});
                            add(new Menu.Item() {{
                                setEnabled(false);
                                setButtonData("Under Construction2-2");
                            }});
                        }});
                    }});
                }});
            }});
        }});

        return menuBarItem;
    }

    public static MenuBar.Item createSettingMenuBarItem() {
        MenuBar.Item menuBarItem = new MenuBar.Item();
        menuBarItem.setButtonData(DEF_TITLE_FILE_SETTINGS);
        menuBarItem.setEnabled(true);
        menuBarItem.setMenu(new Menu() {{
            getSections().add(new Menu.Section() {{
                for (int i = 0; i < 10; i++) {
                    add(new Menu.Item() {{
                        setEnabled(false);
                        setButtonData("Under Construction");
                    }});
                }
            }});
        }});

        return menuBarItem;
    }

    public static MenuBar.Item createVersionControlMenuBarItem() {
        MenuBar.Item menuBarItem = new MenuBar.Item();
        menuBarItem.setButtonData(DEF_TITLE_FILE_VERSION_CONTROL);
        menuBarItem.setEnabled(true);

        menuBarItem.setMenu(new Menu() {{
            getSections().add(new Menu.Section() {{
                add(new Menu.Item() {{
                    setEnabled(false);
                    setButtonData("Repository Settings");
                }});
                add(new Menu.Item() {{
                    setEnabled(false);
                    setButtonData("Commit Changes");
                }});
            }});
            getSections().add(new Menu.Section() {{
                add(new Menu.Item() {{
                    setButtonData("Git");
                    setMenu(new Menu() {{
                        getSections().add(new Menu.Section() {{
                            add(new Menu.Item() {{
                                setEnabled(false);
                                setButtonData("Show History");
                            }});
                        }});
                        getSections().add(new Menu.Section() {{
                            add(new Menu.Item() {{
                                setEnabled(false);
                                setButtonData("Revert");
                            }});
                        }});
                        getSections().add(new Menu.Section() {{
                            add(new Menu.Item() {{
                                setEnabled(false);
                                setButtonData("Fetch Changes");
                            }});
                            add(new Menu.Item() {{
                                setEnabled(false);
                                setButtonData("Pull Changes");
                            }});
                            add(new Menu.Item() {{
                                setEnabled(false);
                                setButtonData("Push Changes");
                            }});
                        }});
                    }});
                }});
            }});
        }});

        return menuBarItem;
    }


    public static MenuBar.Item createHelpMenuBarItem() {
        MenuBar.Item menuBarItem = new MenuBar.Item();
        menuBarItem.setButtonData(DEF_TITLE_FILE_HELP);
        menuBarItem.setEnabled(true);

        final SSTreeNode.NodeType mnuHlp = SSTreeNode.NodeType.MnuHlp;

        menuBarItem.setMenu(new Menu() {{
            getSections().add(new Menu.Section() {{
                add(new Menu.Item() {{
                    setButtonData(new MenuItemData() {{
                        setAction(mnuHlp.toString());
                        setIcon(SSImageFactory.getIconImage(mnuHlp));
                        setText(mnuHlp.toString());
                        setKeyboardShortcut(mnuHlp.getShortCutText());
                    }});
                }});
            }});
        }});

        return menuBarItem;
    }

    public static MenuBar.Item createSeparatorMenuBarItem() {
        final MenuBar.Item menuBarItem = new MenuBar.Item();
        menuBarItem.setEnabled(false);
        menuBarItem.setButtonData(" | ");

        return menuBarItem;
    }

    public static MenuBar.Item createDateInfoMenuBarItem() {
        final MenuBar.Item menuBarItem = new MenuBar.Item();
        menuBarItem.setEnabled(true);

        ApplicationContext.scheduleRecurringCallback(new Runnable() {
            @Override
            public void run() {
                menuBarItem.setButtonData(new Object() {
                    @Override
                    public String toString() {
                        return new Date().toString();
                    }
                });
            }
        }, 1000, 1000);

        return menuBarItem;
    }

    public static MenuBar.Item createMemoryIndicatorMenuBarItem() {
        final MenuBar.Item menuBarItem = new MenuBar.Item();
        menuBarItem.setEnabled(true);

        ApplicationContext.scheduleRecurringCallback(new Runnable() {
            @Override
            public void run() {
                menuBarItem.setButtonData(new Object() {
                    @Override
                    public String toString() {
                        NumberFormat form = NumberFormat.getNumberInstance();
                        double totalMem = Runtime.getRuntime().totalMemory() / 1000000.0;
                        double freeMem = Runtime.getRuntime().freeMemory() / 1000000.0;
                        return String.format("Total Memory: %s MB, Memory Usage: %s MB", form.format(totalMem), form.format(totalMem - freeMem));
                    }
                });
            }
        }, 1000, 1000);

        return menuBarItem;
    }


    public static MenuBar.Item createMenuBarItem(Object menuData) {
        MenuBar.Item menuBarItem = new MenuBar.Item();
        menuBarItem.setButtonData(menuData);
        menuBarItem.setEnabled(true);

        Menu.Section menuSection = new Menu.Section();

        Menu.Item menuItem = new Menu.Item();
        menuItem.setName("");
        menuItem.setButtonData("aaaaa");
        menuSection.add(menuItem);

        Menu menu = new Menu();
        menu.setName("menu");
        menu.getSections().add(menuSection);

        menuBarItem.setMenu(menu);

//        ColorChooserButton colorChooserButton = new ColorChooserButton();
//        Color selectedColor = colorChooserButton.getSelectedColor();
//        colorChooserButton.setName("Color");
//        colorChooserButton.setButtonData(Color.BLACK);

//        ButtonGroup buttonGroup = new ButtonGroup();
//        buttonGroup.add(new Menu.Item("aaa"));
//        buttonGroup.add(new Menu.Item("bbb"));
//        menuBarItem.setButtonGroup(buttonGroup);

        return menuBarItem;
    }
}
