/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.factory;

import jp.pegasus.config.Location;
import jp.pegasus.log.Logger;
import jp.pegasus.pivot.ss.SSPalette;
import jp.pegasus.pivot.ss.SSTreeView;
import jp.pegasus.pivot.ss.tableview.SSTableView;
import jp.pegasus.util.JsonUtil;
import org.apache.pivot.collections.Sequence;
import org.apache.pivot.util.Vote;
import org.apache.pivot.wtk.*;
import org.apache.pivot.wtk.media.Image;

public class SSPaletteFactory {

    public static SSPalette createPalette(SSPalette.Type type) {
        SSPalette palette = new SSPalette(type);
        JsonUtil.setStyles(palette, Location.PALETTE);

        palette.getComponentStateListeners().add(new ComponentStateListener() {
            @Override
            public void enabledChanged(Component component) {
                Logger.log.info("");
            }

            @Override
            public void focusedChanged(Component component, Component obverseComponent) {
                Logger.log.info("");
            }
        });

        palette.getWindowStateListeners().add(new WindowStateListener() {
            @Override
            public void windowOpened(Window window) {
                Logger.log.info("");
                SSPalette palette = (SSPalette) window;
                SSTreeView treeView = (SSTreeView) palette.getContent();

                switch (palette.getPaletteType()) {
                    case Default:
                        break;
                    case PopupSpreadSheet:
                        SSTableView tableView = treeView.getTableView();
                        treeView.setDisabledNodeFilter(new SSTreeView.DisabledNodeFilter(tableView));
                        break;
                    case PopupWindowMenu:
                        break;
                    default:
                        break;
                }
            }

            @Override
            public Vote previewWindowClose(Window window) {
                Logger.log.info("");
                Vote vote = Vote.APPROVE;
                return vote;
            }

            @Override
            public void windowCloseVetoed(Window window, Vote reason) {
                Logger.log.info("");
            }

            @Override
            public void windowClosed(Window window, Display display, Window owner) {
                Logger.log.info("");
            }
        });

        return palette;
    }

    public static SSPalette createPalette() {
        SSPalette palette = new SSPalette();
        JsonUtil.setStyles(palette, Location.PALETTE);

        palette.getComponentStateListeners().add(new ComponentStateListener() {
            @Override
            public void enabledChanged(Component component) {
                Logger.log.info("");
            }

            @Override
            public void focusedChanged(Component component, Component obverseComponent) {
                Logger.log.info("");
            }
        });

        palette.getWindowListeners().add(new WindowListener() {
            @Override
            public void titleChanged(Window window, String previousTitle) {
                Logger.log.info("");
            }

            @Override
            public void iconAdded(Window window, Image addedIcon) {
                Logger.log.info("");
            }

            @Override
            public void iconInserted(Window window, Image addedIcon, int index) {
                Logger.log.info("");
            }

            @Override
            public void iconsRemoved(Window window, int index, Sequence<Image> removed) {
                Logger.log.info("");
            }

            @Override
            public void contentChanged(Window window, Component previousContent) {
                Logger.log.info("");
            }

            @Override
            public void activeChanged(Window window, Window obverseWindow) {
                Logger.log.info("");
            }

            @Override
            public void maximizedChanged(Window window) {
                Logger.log.info("");
            }
        });

        palette.getWindowStateListeners().add(new WindowStateListener() {
            @Override
            public void windowOpened(Window window) {
                Logger.log.info("");
                SSPalette palette = (SSPalette) window;
                SSTreeView treeView = (SSTreeView) palette.getContent();
                SSTableView tableView = treeView.getTableView();
                treeView.setDisabledNodeFilter(new SSTreeView.DisabledNodeFilter(tableView));
            }

            @Override
            public Vote previewWindowClose(Window window) {
                Logger.log.info("");
                Vote vote = Vote.APPROVE;
                return vote;
            }

            @Override
            public void windowCloseVetoed(Window window, Vote reason) {
                Logger.log.info("");
            }

            @Override
            public void windowClosed(Window window, Display display, Window owner) {
                Logger.log.info("");
            }
        });

        return palette;
    }
}
