/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.factory;

import jp.pegasus.config.Config;
import jp.pegasus.config.Location;
import jp.pegasus.log.Logger;
import jp.pegasus.pivot.model.SSData;
import jp.pegasus.pivot.model.SSElementalData;
import jp.pegasus.pivot.model.SSFileData;
import jp.pegasus.pivot.model.SSTextData;
import jp.pegasus.pivot.ss.SSTextArea;
import jp.pegasus.util.JsonUtil;
import org.apache.pivot.collections.HashMap;
import org.apache.pivot.wtk.*;

import java.io.File;

public class SSTextAreaFactory {

    public static SSTextArea createTextArea(SSTextArea.TextAreaType textAreaType) {
        SSTextArea textArea = new SSTextArea();
        JsonUtil.setStyles(textArea, Location.TEXT_AREA);
        textArea.setTextAreaType(textAreaType);
        textArea.setEnabled(true);
        textArea.setEditable(true);
        textArea.setTextBindType(BindType.LOAD);

        textArea.getComponentMouseButtonListeners().add(new ComponentMouseButtonListener() {
            @Override
            public boolean mouseDown(Component component, Mouse.Button button, int i, int i1) {
                return false;
            }

            @Override
            public boolean mouseUp(Component component, Mouse.Button button, int i, int i1) {
                return false;
            }

            @Override
            public boolean mouseClick(Component component, Mouse.Button button, int i, int i1, int i2) {
                Logger.log.debug("component:" + component.toString());
                return false;
            }
        });

        textArea.getComponentKeyListeners().add(new ComponentKeyListener() {
            @Override
            public boolean keyTyped(Component component, char c) {
                SSTextArea textArea = (SSTextArea) component;
                if (SSTextArea.TextAreaType.Sheet.equals(textArea.getTextAreaType())) {

                }

                return false;
            }

            @Override
            public boolean keyPressed(Component component, int keyCode, Keyboard.KeyLocation keyLocation) {
                final SSTextArea textArea = (SSTextArea) component;
                if (SSTextArea.TextAreaType.Sheet.equals(textArea.getTextAreaType())) {
                    Logger.log.debug("keyCode:" + keyCode);

                    if (false) {
                    }
                    /**
                     * Save
                     */
                    else if (keyCode == Keyboard.KeyCode.S && Keyboard.isPressed(Keyboard.Modifier.CTRL)) {
                        final Window window = textArea.getWindow();
                        SSData data = (SSData) window.getAttribute(SSData.DEFAULT_TYPE);

                        final int selectFileIndex = data.getSelectFileIndex();
                        SSFileData fileData = data.getFileDataList().get(selectFileIndex);

                        final int selectSheetIndex = fileData.getSelectSheetIndex();
                        Object sheetData = fileData.getSheetDataList().get(selectSheetIndex);
                        SSTextData textData = null;
                        if (sheetData instanceof HashMap) {
                            textData = SSTextData.convertMapToObject((HashMap<String, ?>) sheetData);
                        } else if (sheetData instanceof SSTextData) {
                            textData = (SSTextData) sheetData;
                        }
                        SSElementalData elementalData = new SSElementalData() {{
                            this.setText(textArea.getText());
                        }};

                        String digest = JsonUtil.getDigest(elementalData, Config.DIGEST_ARGORITHM);
                        StringBuilder textDataFilePath = new StringBuilder(Config.APP_TEXT_DATA_DIR).append(File.separator).append(digest);
                        JsonUtil.writeSheetData(elementalData, textDataFilePath.toString());

                        if (sheetData instanceof HashMap) {
                            ((HashMap<String, String>) sheetData).put("textDataDigest", digest);
                        } else if (sheetData instanceof SSTextData) {
                            ((SSTextData) sheetData).setTextDataDigest(digest);
                        }
                        JsonUtil.writeGrandData(data, Config.APP_DATA_FILE_PATH);

                        return false;
                    }
                }

                return false;
            }

            @Override
            public boolean keyReleased(Component component, int keyCode, Keyboard.KeyLocation keyLocation) {
                return false;
            }
        });

        return textArea;
    }

    public static void setSelect(SSTextArea textArea) {
        JsonUtil.setStyles(textArea, Location.TEXT_AREA_SELECT);
        textArea.setEnabled(true);
        textArea.setEditable(true);
    }

    public static void setAlternate(SSTextArea textArea) {
        JsonUtil.setStyles(textArea, Location.TEXT_AREA_ALTERNATE);
    }

}
