/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.factory;

import jp.pegasus.pivot.factory.tabpane.SSTabPaneContentFactory;
import jp.pegasus.pivot.model.SSData;
import jp.pegasus.pivot.model.SSFileData;
import jp.pegasus.pivot.ss.SSTabPane;
import jp.pegasus.pivot.ss.SSTreeBranch;
import jp.pegasus.pivot.ss.SSTreeNode;
import org.apache.pivot.collections.ArrayList;
import org.apache.pivot.collections.List;

public class SSTreeNodeFactory {

    public static SSTreeNode createTreeNode(SSTreeNode.NodeType nodeType) {
        SSTreeNode treeNode = new SSTreeNode(nodeType);

        return treeNode;
    }

    public static ArrayList<SSTreeNode> constructTreeNodeList(SSData data) {
        /**
         * File data
         */
        ArrayList<SSTreeNode> treeNodeList = new ArrayList<SSTreeNode>();
        ArrayList<SSFileData> fileDataList = data.getFileDataList();
        for (int i = 0, n = fileDataList.getLength(); i < n; i++) {
            List<Object> sheetDataList = null;
            SSFileData fileData = fileDataList.get(i);

            final String fileName = fileData.getFileName();
            final long fileCreateTime = fileData.getCreateTime();
            final int sheetType = fileData.getSheetType();

            SSTreeNode.NodeType nodeType = null;
            switch (sheetType) {
                case 0:
                    nodeType = SSTreeNode.NodeType.MnuFilGrid;
                    break;
                case 1:
                    nodeType = SSTreeNode.NodeType.MnuFilText;
                    break;
                default:
                    throw new RuntimeException();
            }
            SSTreeBranch treeBranch = SSTreeBranchFactory.createTreeBranch(fileName, nodeType, sheetDataList);

            SSTabPane tabPane = SSTabPaneContentFactory.constructTabPaneContent(fileData);
            treeBranch.setTabPane(tabPane);
            treeNodeList.add(treeBranch);
        }

        return treeNodeList;
    }
}
