/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.factory;

import jp.pegasus.config.Config;
import jp.pegasus.config.Location;
import jp.pegasus.log.Logger;
import jp.pegasus.pivot.factory.tableview.SSTableViewColumnHeaderFactory;
import jp.pegasus.pivot.factory.tableview.SSTableViewFactory;
import jp.pegasus.pivot.factory.tableview.SSTableViewRowHeaderFactory;
import jp.pegasus.pivot.model.SSElementalData;
import jp.pegasus.pivot.model.SSGridData;
import jp.pegasus.pivot.model.SSTextData;
import jp.pegasus.pivot.skin.SSTabPaneSkin;
import jp.pegasus.pivot.ss.*;
import jp.pegasus.pivot.ss.tableview.SSTableView;
import jp.pegasus.pivot.ss.tableview.SSTableViewHeader;
import jp.pegasus.util.JsonUtil;
import org.apache.pivot.collections.ArrayList;
import org.apache.pivot.collections.HashMap;
import org.apache.pivot.wtk.*;
import org.apache.pivot.wtk.content.ButtonData;

import java.io.File;
import java.util.Date;

public class ScrollPaneFactory implements Config {

    public static SSScrollPane createScrollPane(ScrollPane.ScrollBarPolicy horizontalScrollBarPolicy, ScrollPane.ScrollBarPolicy verticalScrollBarPolicy) {
        SSScrollPane scrollPane = new SSScrollPane(horizontalScrollBarPolicy, verticalScrollBarPolicy);

        scrollPane.getComponentMouseListeners().add(new ComponentMouseListener() {
            @Override
            public boolean mouseMove(Component component, int x, int y) {
                return false;
            }

            @Override
            public void mouseOver(Component component) {
            }

            @Override
            public void mouseOut(Component component) {
            }
        });

        scrollPane.getComponentKeyListeners().add(new ComponentKeyListener() {
            @Override
            public boolean keyTyped(Component component, char character) {
                switch (character) {
                    case '\t':
                        break;
                    default:
                        break;
                }
                return false;
            }

            @Override
            public boolean keyPressed(Component component, int keyCode, Keyboard.KeyLocation keyLocation) {
                Logger.log.debug("");

                return false;
            }

            @Override
            public boolean keyReleased(Component component, int keyCode, Keyboard.KeyLocation keyLocation) {
                //log.debug("keyCode:{}", keyCode);
                return false;
            }
        });

        scrollPane.getComponentMouseWheelListeners().add(new ComponentMouseWheelListener() {

            @Override
            public boolean mouseWheel(Component component, Mouse.ScrollType scrollType, int scrollAmount, int wheelRotation, int x, int y) {
//                LinkedHashMap<String, Object> logAttr = new LinkedHashMap<String, Object>();
//                logAttr.put("scrollType", scrollType.toString());
//                logAttr.put("scrollAmount", scrollAmount);
//                logAttr.put("wheelRotation", wheelRotation);
//                logAttr.put("x", x);
//                logAttr.put("y", y);
//                log.debug("{}", logAttr);

                return false;
            }
        });

        scrollPane.getComponentMouseButtonListeners().add(new ComponentMouseButtonListener() {

            @Override
            public boolean mouseDown(Component component, Mouse.Button button, int x, int y) {
                Logger.log.debug("x:{}, y:{}", x, y);

                return false;
            }

            @Override
            public boolean mouseUp(Component component, Mouse.Button button, int x, int y) {
                Logger.log.debug("x:{}, y:{}", x, y);

                return false;
            }

            @Override
            public boolean mouseClick(Component component, Mouse.Button button, int x, int y, int count) {
                return false;
            }
        });

        scrollPane.getViewportListeners().add(new ViewportListener() {

            @Override
            public void scrollTopChanged(Viewport scrollPane, int previousScrollTop) {
                //log.debug("scrollPane:{}", scrollPane.toString());
                //log.debug("previousScrollTop:{}", previousScrollTop);
            }

            @Override
            public void scrollLeftChanged(Viewport scrollPane, int previousScrollLeft) {
                Logger.log.debug("");
//                if (currentColumnIndex != -1 && currentRowIndex != -1 &&
//                        selectColumnIndex != -1 && selectRowIndex != -1) {
//                    SSTableViewColumnHeaderFactory.refreshDecorator((SSTableViewHeader) ((SSScrollPane) scrollPane).getColumnHeader());
//                }
            }

            @Override
            public void viewChanged(Viewport scrollPane, Component previousView) {
                Logger.log.debug("");
            }
        });

        return scrollPane;
    }

    public static SSScrollPane constructGridScrollPane(final SSTabPane tabPane, final SSGridData gridData) {
        final String sheetName = gridData.getSheetName();
        final long createTime = gridData.getCreateTime();
        final long selectCurrentIndexQuartet = gridData.getSelectCurrentIndexQuartet();
        final String cellDataDigest = gridData.getCellDataDigest();

        /**
         * Table View
         */
        final SSTableView tableView = SSTableViewFactory.createTableViewOfSpreadSheet();

        /**
         * Set Cell Data
         */
        ArrayList<HashMap<String, SSElementalData>> cellDataList = null;
        if (cellDataDigest == null) {
            cellDataList = SSElementalData.createCellDataList();
        } else {
            final String cellDataFilePath = new StringBuilder(Config.APP_CELL_DATA_DIR).append(File.separator).append(cellDataDigest).toString();
            cellDataList = JsonUtil.readCellDataList(cellDataFilePath);
        }
        tableView.setTableData(cellDataList);

        /**
         * Row Header
         */
        SSTableView tableViewRowHeader = SSTableViewRowHeaderFactory.createTableViewOfRowHeader();

        /**
         * Column Header
         */
        SSTableViewHeader tableViewColumnHeader = SSTableViewColumnHeaderFactory.createTableViewColumnHeader(tableView);

        /**
         * Corner
         */
        SSCorner corner = SSCornerFactory.createCorner();

        /**
         * Scroll
         */
        SSScrollPane scrollPane = ScrollPaneFactory.createScrollPane(ScrollPane.ScrollBarPolicy.AUTO, ScrollPane.ScrollBarPolicy.AUTO);
        scrollPane.setView(tableView);
        scrollPane.setColumnHeader(tableViewColumnHeader);
        scrollPane.setRowHeader(tableViewRowHeader);
        scrollPane.setCorner(corner);

        /**
         * Tab
         */
        SSBorder border = new SSBorder();
        JsonUtil.setStyles(border, Location.BORDER);
        border.setContent(scrollPane);

        SSTabPane.setTabData(border, new ButtonData(sheetName) {{
            setUserData(SSTabPaneSkin.Type.Sheet);
        }});
        SSTabPane.setTooltipText(border, new Date(createTime).toString());
        tabPane.getTabs().add(border);

        return scrollPane;
    }

    public static SSScrollPane constructTextScrollPane(SSTabPane tabPane, SSTextData textData) {
        /**
         * Text Area
         */
        SSTextArea textArea = SSTextAreaFactory.createTextArea(SSTextArea.TextAreaType.Sheet);
        textArea.setEnabled(true);
        textArea.setEditable(true);
        textArea.setMaximumWidth(Config.TEXT_AREA_WIDTH);

        /**
         * Set Text Data
         */
        StringBuilder textDataFilePath = new StringBuilder(Config.APP_TEXT_DATA_DIR).append(File.separator);
        String digest = textData.getTextDataDigest();
        Logger.log.debug("digest:{}", digest);
        if (digest == null) {
            SSElementalData elementalData = new SSElementalData();
            String newDigest = JsonUtil.getDigest(elementalData, Config.DIGEST_ARGORITHM);
            textDataFilePath.append(newDigest);
            JsonUtil.writeSheetData(elementalData, textDataFilePath.toString());
            textData.setTextDataDigest(newDigest);
        } else {
            textDataFilePath.append(digest);
            SSElementalData elementalData = JsonUtil.readTextData(textDataFilePath.toString());
            if (elementalData.getText() == null) {
                textArea.setText("");
            } else {
                textArea.setText(elementalData.getText());
            }
        }

        /**
         * Tab
         */
        final String sheetName = textData.getSheetName();
        final long sheetCreateTime = textData.getCreateTime();

        /**
         * Scroll
         */
        SSScrollPane scrollPane = ScrollPaneFactory.createScrollPane(ScrollPane.ScrollBarPolicy.AUTO, ScrollPane.ScrollBarPolicy.AUTO);
        scrollPane.setView(textArea);

        SSBorder border = new SSBorder();
        JsonUtil.setStyles(border, Location.BORDER);
        border.setContent(scrollPane);

        SSTabPane.setTabData(border, new ButtonData(sheetName) {{
            setUserData(SSTabPaneSkin.Type.Sheet);
        }});
        SSTabPane.setTooltipText(border, new Date(sheetCreateTime).toString());
        tabPane.getTabs().add(border);

        return scrollPane;
    }
}
