/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.factory.tableview;

import jp.pegasus.config.Config;
import jp.pegasus.config.Location;
import jp.pegasus.pivot.factory.SSCornerFactory;
import jp.pegasus.pivot.ss.SSCorner;
import jp.pegasus.pivot.ss.SSScrollPane;
import jp.pegasus.pivot.ss.tableview.SSTableView;
import jp.pegasus.pivot.ss.tableview.SSTableViewHeader;
import jp.pegasus.util.JsonUtil;
import org.apache.pivot.wtk.*;

public class SSTableViewColumnHeaderFactory {

    @SuppressWarnings("unchecked")
    public static SSTableViewHeader createTableViewColumnHeader(SSTableView tableView) {
        SSTableViewHeader tableViewColumnHeader = new SSTableViewHeader();
        JsonUtil.setStyles(tableViewColumnHeader, Location.TABLE_VIEW_COL_HEADER);
        tableViewColumnHeader.setTableView(tableView);

        tableViewColumnHeader.getComponentMouseButtonListeners().add(new ComponentMouseButtonListener() {
            @Override
            public boolean mouseDown(Component component, Mouse.Button button, int x, int y) {
                SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) component;
                SSScrollPane scrollPane = (SSScrollPane) tableViewColumnHeader.getParent();
                SSTableView tableView = (SSTableView) scrollPane.getView();

                if (button == Mouse.Button.LEFT) {

                    Cursor cursor = tableViewColumnHeader.getCursor();
                    if (cursor != Cursor.RESIZE_EAST) {
                        long selectCurrentIndexQuartet = tableView.getSelectCurrentIndexQuartet();
                        int selectColumnIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 48));

                        final int columnIndex = tableViewColumnHeader.getHeaderAt(x);
                        if (!Keyboard.isPressed(Keyboard.Modifier.SHIFT)) {
                            selectColumnIndex = columnIndex;
                        }

                        final int maxRowNumber = Config.MAX_ROW_NUMBER;
                        selectCurrentIndexQuartet = ((long) selectColumnIndex << 48) | (0l << 32) | ((long) columnIndex << 16) | (long) maxRowNumber - 2;
                        tableView.setSelectCurrentIndexQuartet(selectCurrentIndexQuartet);

                        tableViewColumnHeader.setSelect(true);
                        tableViewColumnHeader.setHeaderPressed(true);
                        tableViewColumnHeader.getStyles().put("columnsResizable", false);
                        tableView.setSelect(true);

                        SSTableView tableViewRowHeader = (SSTableView) scrollPane.getRowHeader();
                        tableViewRowHeader.setSelect(true);
                        SSTableViewRowHeaderFactory.refreshDecorator(tableViewRowHeader);

                        SSTableViewFactory.refreshDecorator(tableView);
                        refreshDecorator(tableViewColumnHeader);

                        SSCorner corner = (SSCorner) scrollPane.getCorner();
                        SSCornerFactory.setDefaultSkin(corner);
                    }
                    tableView.setMouseDown(true);
                }

                return false;
            }

            @Override
            public boolean mouseUp(Component component, Mouse.Button button, int x, int y) {
                //log.debug("");
                SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) component;
                SSScrollPane scrollPane = (SSScrollPane) tableViewColumnHeader.getParent();
                SSTableView tableView = (SSTableView) scrollPane.getView();

                if (tableView.isSelect()) {
                    SSTableView tableViewRowHeader = (SSTableView) scrollPane.getRowHeader();
                    SSTableViewRowHeaderFactory.refreshDecorator(tableViewRowHeader);
                    SSTableViewFactory.refreshDecorator(tableView);
                    refreshDecorator(tableViewColumnHeader);

                    tableViewColumnHeader.setHeaderPressed(false);
                    tableViewColumnHeader.getStyles().put("columnsResizable", true);
                    tableViewRowHeader.setHeaderPressed(false);
                    tableView.setMouseDown(false);
                    //log.debug("tableView.isMouseDown:{}", tableView.isMouseDown());
                }

                return false;
            }

            @Override
            public boolean mouseClick(Component component, Mouse.Button
            button, int x, int y, int count) {
                //log.debug("");
                return false;
            }
        });

        tableViewColumnHeader.getComponentMouseListeners().add(new ComponentMouseListener() {

            @Override
            public boolean mouseMove(Component component, int x, int y) {
                SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) component;
                SSScrollPane scrollPane = (SSScrollPane) tableViewColumnHeader.getParent();
                SSTableView tableView = (SSTableView) scrollPane.getView();
                SSTableView tableViewRowHeader = (SSTableView) scrollPane.getRowHeader();
                //log.debug("tableView.isMouseDown:{}", tableView.isMouseDown());

                if (tableView.isMouseDown()) {
                    Cursor cursor = tableViewColumnHeader.getCursor();

                    if (cursor == Cursor.RESIZE_EAST) {
                        SSTableViewRowHeaderFactory.refreshDecorator(tableViewRowHeader);
                        SSTableViewFactory.refreshDecorator(tableView);
                        refreshDecorator(tableViewColumnHeader);
                    } else {
                        int columnIndex = tableViewColumnHeader.getHeaderAt(x);
                        mouseMoveOnTableViewColumnHeader(tableViewColumnHeader, columnIndex);
                    }
                }

                if (!tableView.isMouseDown() && tableView.isSelect()) {
                    SSTableViewRowHeaderFactory.refreshDecorator(tableViewRowHeader);
                    SSTableViewFactory.refreshDecorator(tableView);
                    refreshDecorator(tableViewColumnHeader);
                }

                return false;
            }

            @Override
            public void mouseOver(Component component) {
            }

            @Override
            public void mouseOut(Component component) {
            }
        });

        return tableViewColumnHeader;
    }

    public static void refreshDecorator(SSTableViewHeader tableViewColumnHeader) {
        SSScrollPane scrollPane = (SSScrollPane) tableViewColumnHeader.getParent();
        SSTableView tableView = (SSTableView) scrollPane.getView();

        final long selectCurrentIndexQuartet = tableView.getSelectCurrentIndexQuartet();
        final int selColumnIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 48));
        final int curColumnIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 16));

        int[] selectRange = {-1, -1, -1, -1};
        long selectIndexQuartet = 0x0000000000000000l;

        if (tableView.isSelect()) {
            Bounds cellBoundsBgn = tableViewColumnHeader.getHeaderBounds(selColumnIndex);
            Bounds cellBoundsEnd = tableViewColumnHeader.getHeaderBounds(curColumnIndex);

            selectRange[1] = 0;
            selectRange[3] = cellBoundsBgn.height - 1;

            if (curColumnIndex > selColumnIndex) {
                selectIndexQuartet |= ((long) selColumnIndex << 48) | ((long) curColumnIndex << 16);

                selectRange[0] = cellBoundsBgn.x;
                selectRange[2] = cellBoundsEnd.x + cellBoundsEnd.width - cellBoundsBgn.x;
            } else {
                selectIndexQuartet |= ((long) curColumnIndex << 48) | ((long) selColumnIndex << 16);

                selectRange[0] = cellBoundsEnd.x;
                selectRange[2] = cellBoundsBgn.x + cellBoundsBgn.width - cellBoundsEnd.x;
            }
        }

        //log.info("selectIndex:{}", selectIndex);
        tableViewColumnHeader.setSelectIndex(selectIndexQuartet);
        tableViewColumnHeader.setSelectRange(selectRange);
        tableViewColumnHeader.repaint();
    }

    public static void mouseMoveOnTableViewColumnHeader(SSTableViewHeader tableViewColumnHeader, final int columnIndex) {
        SSScrollPane scrollPane = (SSScrollPane) tableViewColumnHeader.getParent();
        SSTableView tableView = (SSTableView) scrollPane.getView();

        long selectCurrentIndexQuartet = tableView.getSelectCurrentIndexQuartet();
        final int selColIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 48));
        final int selRowIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 32));
        final int curColIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 16));
        final int curRowIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet));

        if (curColIndex == columnIndex) {
            return;
        } else if (tableView.isSelect()) {
            SSTableView tableViewRowHeader = (SSTableView) scrollPane.getRowHeader();
            if (!tableViewRowHeader.isHeaderPressed()) {
                selectCurrentIndexQuartet = ((long) selColIndex << 48) | ((long) selRowIndex << 32) | ((long) columnIndex << 16) | (long) curRowIndex;
                tableView.setSelectCurrentIndexQuartet(selectCurrentIndexQuartet);
            }

            SSTableViewRowHeaderFactory.refreshDecorator(tableViewRowHeader);
            SSTableViewFactory.refreshDecorator(tableView);
            refreshDecorator(tableViewColumnHeader);
        }
    }

}
