/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.factory.tableview;

import jp.pegasus.config.Config;
import jp.pegasus.config.Location;
import jp.pegasus.log.Logger;
import jp.pegasus.pivot.content.SSTableViewTextAreaCellRenderer;
import jp.pegasus.pivot.editor.SSTableViewRowEditor;
import jp.pegasus.pivot.factory.SSCornerFactory;
import jp.pegasus.pivot.listener.tableview.SSTableViewColumnListener;
import jp.pegasus.pivot.model.SSData;
import jp.pegasus.pivot.model.SSElementalData;
import jp.pegasus.pivot.model.SSFileData;
import jp.pegasus.pivot.model.SSGridData;
import jp.pegasus.pivot.ss.SSCorner;
import jp.pegasus.pivot.ss.SSScrollPane;
import jp.pegasus.pivot.ss.tableview.SSTableView;
import jp.pegasus.pivot.ss.tableview.SSTableViewHeader;
import jp.pegasus.pivot.util.SSTableViewUtil;
import jp.pegasus.util.JsonUtil;
import org.apache.pivot.collections.ArrayList;
import org.apache.pivot.collections.HashMap;
import org.apache.pivot.collections.List;
import org.apache.pivot.collections.Sequence;
import org.apache.pivot.wtk.*;

import java.io.File;

public class SSTableViewFactory {

//    private static SSTableViewRowEditor tableViewRowEditor = new SSTableViewRowEditor();

    public static SSTableView createTableViewOfSpreadSheet() {
        SSTableView tableView = new SSTableView(SSTableView.Type.SpreadSheet);
        JsonUtil.setStyles(tableView, Location.TABLE_VIEW);
        tableView.setSelectMode(SSTableView.SelectMode.MULTI);
//        SSTableViewRowEditor tableViewRowEditor = new SSTableViewRowEditor();
//        tableView.setRowEditor(tableViewRowEditor);

        for (int i = 1, colNum = Config.MAX_COL_NUMBER; i < colNum; i++) {
            SSTableView.Column col = createColumn(SSTableViewUtil.fromNumber2Alphabet(i), Config.COLUMN_WIDTH);
            tableView.getColumns().add(col);
        }

        tableView.getComponentKeyListeners().add(new ComponentKeyListener() {
            @Override
            public boolean keyTyped(Component component, char character) {
                SSTableView tableView = (SSTableView) component;
                if (Character.isLetterOrDigit(character)) {
                    Logger.log.debug("character: {}", character);

                    startEditColumn(tableView, character);
                }

                return false;
            }

            @Override
            @SuppressWarnings("unchecked")
            public boolean keyPressed(Component component, final int keyCode, Keyboard.KeyLocation keyLocation) {
                Logger.log.info("keyCode: {}", keyCode);
                SSTableView tableView = (SSTableView) component;

                long selectCurrentIndexQuartet = tableView.getSelectCurrentIndexQuartet();
                int selColIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 48));
                int selRowIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 32));
                int curColIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 16));
                int curRowIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet));

                if (tableView.isSelect()) {
                    if (false) {
                    } else if (keyCode == Keyboard.KeyCode.BACKSPACE) {
                        List<?> tableData = tableView.getTableData();
                        HashMap<String, Object> rowData = (HashMap<String, Object>) tableData.get(selRowIndex);
                        rowData.put(SSTableViewUtil.fromNumber2Alphabet(selColIndex + 1), "");
                        return false;
                    }
                    /**
                     * Copy
                     */
                    else if (keyCode == Keyboard.KeyCode.C && Keyboard.isPressed(Keyboard.Modifier.CTRL)) {
                        SSTableViewUtil.copy(tableView);
                        return false;
                    }
                    /**
                     * Cut
                     */
                    else if (keyCode == Keyboard.KeyCode.X && Keyboard.isPressed(Keyboard.Modifier.CTRL)) {
                        SSTableViewUtil.cut(tableView);
                        return false;
                    }
                    /**
                     * Exit(Close Window)
                     */
                    else if (keyCode == Keyboard.KeyCode.D && Keyboard.isPressed(Keyboard.Modifier.CTRL)) {
                        /**
                         * Save before Exit
                         */
                        final Window window = tableView.getWindow();
                        SSData data = (SSData) window.getAttribute(SSData.DEFAULT_TYPE);

                        final int selectFileIndex = data.getSelectFileIndex();
                        SSFileData fileData = data.getFileDataList().get(selectFileIndex);

                        final int selectSheetIndex = fileData.getSelectSheetIndex();

                        Object sheetData = fileData.getSheetDataList().get(selectSheetIndex);
                        SSGridData gridData = null;
                        if (sheetData instanceof HashMap) {
                            gridData = SSGridData.convertMapToObject((HashMap<String, ?>) sheetData);
                        } else if (sheetData instanceof SSGridData) {
                            gridData = (SSGridData) sheetData;
                        } else {
                            throw new RuntimeException();
                        }

                        ArrayList<HashMap<String, SSElementalData>> cellDataList = (ArrayList<HashMap<String, SSElementalData>>) tableView.getTableData();
                        String digest = JsonUtil.getDigest(cellDataList, Config.DIGEST_ARGORITHM);
                        StringBuilder cellDataFilePath = new StringBuilder().append(Config.APP_CELL_DATA_DIR).append(File.separator).append(digest);
                        JsonUtil.writeSheetData(cellDataList, cellDataFilePath.toString());

                        if (sheetData instanceof HashMap) {
                            ((HashMap<String, String>) sheetData).put("cellDataDigest", digest);
                        } else if (sheetData instanceof SSGridData) {
                            ((SSGridData) sheetData).setCellDataDigest(digest);
                        }
                        JsonUtil.writeGrandData(data, Config.APP_DATA_FILE_PATH);

                        DesktopApplicationContext.exit();

                        return false;
                    }
                    /**
                     * Delete
                     */
                    else if (keyCode == Keyboard.KeyCode.DELETE) {
                        SSTableViewUtil.delete(tableView);
                        return false;
                    } else if (keyCode == Keyboard.KeyCode.DOWN) {
                        final int maxRowNumber = Config.MAX_ROW_NUMBER - 2;
                        if (curRowIndex != maxRowNumber && selRowIndex != maxRowNumber) {
                            if (Keyboard.isPressed(Keyboard.Modifier.SHIFT)) {
                                curRowIndex++;
                            } else if (Keyboard.isPressed(Keyboard.Modifier.CTRL)) {
                                selRowIndex = maxRowNumber;
                            } else {
                                selRowIndex++;
                            }
                        }
                    } else if (keyCode == Keyboard.KeyCode.ENTER) {
                        final int maxRowNumber = Config.MAX_ROW_NUMBER - 2;
                        SSTableView.RowEditor rowEditor = tableView.getRowEditor();
                        if (rowEditor != null && rowEditor.isEditing()) {
                            rowEditor.endEdit(true);
                        }
                        tableView.setRowEditor(null);

                        if (selRowIndex != maxRowNumber) {
                            if (Keyboard.isPressed(Keyboard.Modifier.SHIFT)) {
                                curRowIndex++;
                            } else {
                                selRowIndex++;
                            }
                        }
                    } else if (keyCode == Keyboard.KeyCode.F2) {
                        startEditColumn(tableView);
                        return false;
                    } else if (keyCode == Keyboard.KeyCode.LEFT) {
                        if (curColIndex != 0 && selColIndex != 0) {
                            if (Keyboard.isPressed(Keyboard.Modifier.SHIFT)) {
                                curColIndex--;
                            } else if (Keyboard.isPressed(Keyboard.Modifier.CTRL)) {
                                selColIndex = 0;
                            } else {
                                selColIndex--;
                            }
                        }
                    } else if (keyCode == Keyboard.KeyCode.RIGHT) {
                        final int maxColNumber = Config.MAX_COL_NUMBER - 2;
                        if (curColIndex != maxColNumber && selColIndex != maxColNumber) {
                            //log.debug("{}", Keyboard.isPressed(Keyboard.Modifier.SHIFT));
                            if (Keyboard.isPressed(Keyboard.Modifier.SHIFT)) {
                                curColIndex++;
                            } else if (Keyboard.isPressed(Keyboard.Modifier.CTRL)) {
                                selColIndex = maxColNumber;
                            } else {
                                selColIndex++;
                            }
                        }
                    }
                    /**
                     * Save
                     */
                    else if (keyCode == Keyboard.KeyCode.S && Keyboard.isPressed(Keyboard.Modifier.CTRL)) {
                        final Window window = tableView.getWindow();
                        SSData data = (SSData) window.getAttribute(SSData.DEFAULT_TYPE);

                        final int selectFileIndex = data.getSelectFileIndex();
                        SSFileData fileData = data.getFileDataList().get(selectFileIndex);

                        final int selectSheetIndex = fileData.getSelectSheetIndex();
                        Object sheetData = fileData.getSheetDataList().get(selectSheetIndex);
                        SSGridData gridData = null;
                        if (sheetData instanceof HashMap) {
                            gridData = SSGridData.convertMapToObject((HashMap<String, ?>) sheetData);
                        } else if (sheetData instanceof SSGridData) {
                            gridData = (SSGridData) sheetData;
                        }

                        ArrayList<HashMap<String, SSElementalData>> cellDataList = (ArrayList<HashMap<String, SSElementalData>>) tableView.getTableData();
                        String digest = JsonUtil.getDigest(cellDataList, Config.DIGEST_ARGORITHM);
                        StringBuilder cellDataFilePath = new StringBuilder().append(Config.APP_CELL_DATA_DIR).append(File.separator).append(digest);
                        JsonUtil.writeSheetData(cellDataList, cellDataFilePath.toString());

                        if (sheetData instanceof HashMap) {
                            ((HashMap<String, String>) sheetData).put("cellDataDigest", digest);
                        } else if (sheetData instanceof SSGridData) {
                            ((SSGridData) sheetData).setCellDataDigest(digest);
                        }
                        JsonUtil.writeGrandData(data, Config.APP_DATA_FILE_PATH);

                        return false;
                    } else if (keyCode == Keyboard.KeyCode.TAB) {
                        if (Keyboard.isPressed(Keyboard.Modifier.SHIFT)) {
                            if (curColIndex != 0 && selColIndex != 0) {
                                selColIndex--;
                            }
                        } else {
                            final int maxColNumber = Config.MAX_COL_NUMBER - 2;

                            if (curColIndex != maxColNumber &&
                            selColIndex != maxColNumber) {
                                selColIndex++;
                            }
                        }
                        tableView.requestFocus();
                    } else if (keyCode == Keyboard.KeyCode.UP) {
                        if (curRowIndex != 0 && selRowIndex != 0) {
                            if (Keyboard.isPressed(Keyboard.Modifier.SHIFT)) {
                                curRowIndex--;
                            } else if (Keyboard.isPressed(Keyboard.Modifier.CTRL)) {
                                selRowIndex = 0;
                            } else {
                                selRowIndex--;
                            }
                        }
                    }
                    /**
                     * Paste
                     */
                    else if (keyCode == Keyboard.KeyCode.V &&
                    Keyboard.isPressed(Keyboard.Modifier.CTRL)) {
                        SSTableViewUtil.paste(tableView);
                        return false;
                    } else {
                        return false;
                    }
                }

                /////////////////////////////////////////////////////////////////////////////////////////////////////
                final SSScrollPane scrollPane = (SSScrollPane) tableView.getParent();
                final Bounds visibleArea = tableView.getVisibleArea();
                final long visibleRangeQuartet = SSTableViewUtil.getVisibleRange(tableView, visibleArea.x, visibleArea.y, visibleArea.width, visibleArea.height);
                final int colBgn = (int) (0x000000000000FFFFl & (visibleRangeQuartet >> 48));
                final int rowBgn = (int) (0x000000000000FFFFl & (visibleRangeQuartet >> 32));
                final int colEnd = (int) (0x000000000000FFFFl & (visibleRangeQuartet >> 16));
                final int rowEnd = (int) (0x000000000000FFFFl & (visibleRangeQuartet));
//                Logger.log.debug("visibleRange:{}", visibleRange);

                // horizontal scrolling
                if (colEnd < selColIndex + 1) {
                    final Bounds columnBounds = tableView.getColumnBounds(selColIndex);
                    final int newX = columnBounds.x + columnBounds.width - visibleArea.width;
                    final int newColumnRange = SSTableViewUtil.getVisibleColumnRange(tableView, newX, visibleArea.width);
                    final int newColumnStart = 0x0000FFFF & (newColumnRange >> 16);
                    final Bounds newColumnBounds = tableView.getColumnBounds(newColumnStart + 1);
                    scrollPane.setScrollLeft(newColumnBounds.x);
                } else if (selColIndex < colBgn) {
                    final Bounds newColumnBounds = tableView.getColumnBounds(selColIndex);
                    scrollPane.setScrollLeft(newColumnBounds.x);
                }

                // vertical scrolling
                if (rowEnd < selRowIndex + 1) {
                    final Bounds rowBounds = tableView.getRowBounds(selRowIndex);
                    final int newY = rowBounds.y + rowBounds.height - visibleArea.height;
                    final int newRowRange = SSTableViewUtil.getVisibleRowRange(tableView, newY, visibleArea.height);
                    final int newRowStart = 0x0000FFFF & (newRowRange >> 16);
                    final Bounds newRowBounds = tableView.getRowBounds(newRowStart + 1);
                    scrollPane.setScrollTop(newRowBounds.y);
                } else if (selRowIndex < rowBgn) {
                    final Bounds newRowBounds = tableView.getRowBounds(selRowIndex);
                    scrollPane.setScrollTop(newRowBounds.y);
                }
                /////////////////////////////////////////////////////////////////////////////////////////////////////

                if ((keyCode == Keyboard.KeyCode.TAB && Keyboard.isPressed(Keyboard.Modifier.SHIFT)) ||
                !Keyboard.isPressed(Keyboard.Modifier.SHIFT)) {
                    selectCurrentIndexQuartet = ((long) selColIndex << 48) | ((long) selRowIndex << 32) | ((long) selColIndex << 16) | (long) selRowIndex;
                } else {
                    selectCurrentIndexQuartet = ((long) selColIndex << 48) | ((long) selRowIndex << 32) | ((long) curColIndex << 16) | (long) curRowIndex;
                }
                tableView.setSelectCurrentIndexQuartet(selectCurrentIndexQuartet);

                SSCorner corner = (SSCorner) scrollPane.getCorner();
                SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) scrollPane.getColumnHeader();
                SSTableView tableViewRowHeader = (SSTableView) scrollPane.getRowHeader();

                SSCornerFactory.setDefaultSkin(corner);
                SSTableViewColumnHeaderFactory.refreshDecorator(tableViewColumnHeader);
                SSTableViewRowHeaderFactory.refreshDecorator(tableViewRowHeader);
                refreshDecorator(tableView);

                tableView.setMouseDown(false);

                return false;
            }

            @Override
            public boolean keyReleased(Component component, int keyCode, Keyboard.KeyLocation keyLocation) {
                Logger.log.info("keyCode: {}", keyCode);
                return false;
            }
        });

        tableView.getTableViewColumnListeners().add(new SSTableViewColumnListener() {

            @Override
            public void columnInserted(SSTableView tableView, int index) {
                //log.debug("");
            }

            @Override
            public void columnsRemoved(SSTableView tableView, int index,
                                       Sequence<SSTableView.Column> columns) {
                //log.debug("");
            }

            @Override
            public void columnNameChanged(SSTableView.Column column, String previousName) {
                //log.debug("");
            }

            @Override
            public void columnHeaderDataChanged(SSTableView.Column column,
                                                Object previousHeaderData) {
                //log.debug("");
            }

            @Override
            public void columnHeaderDataRendererChanged(SSTableView.Column column,
                                                        SSTableView.HeaderDataRenderer previousHeaderDataRenderer) {
                //log.debug("");
            }

            @Override
            public void columnWidthChanged(SSTableView.Column column, int previousWidth,
                                           boolean previousRelative) {
//                Logger.log.debug("column:{}, previous:{}", column.getWidth(), previousWidth);
//                Logger.log.debug("getX:{}", tableView.getX());
//                Logger.log.debug("getHeaderAt:{}", tableViewColumnHeader.getHeaderAt(tableView.getX()));
//
//                colmnHeaderBorderDecorator.setSelectX(tableViewColumnHeader.getHeaderAt(tableView.getX()));
//                colmnHeaderBorderDecorator.setSelectY(0);
//                colmnHeaderBorderDecorator.setSelectWidth(column.getWidth());
//                colmnHeaderBorderDecorator.setSelectHeight(tableViewColumnHeader.getHeight() - 1);
//                tableViewColumnHeader.getDecorators().update(0, colmnHeaderBorderDecorator);
            }

            @Override
            public void columnWidthLimitsChanged(SSTableView.Column column,
                                                 int previousMinimumWidth, int previousMaximumWidth) {
                //log.debug("");
            }

            @Override
            public void columnFilterChanged(SSTableView.Column column, Object previousFilter) {
                //log.debug("");
            }

            @Override
            public void columnCellRendererChanged(
            SSTableView.Column column,
            SSTableView.CellRenderer previousCellRenderer) {
                //log.debug("column:{}, previousCellRenderer:{}", column.toString(), previousCellRenderer.toString());
                //previousCellRenderer.getStyles().put("backgroundColor", Color.WHITE);
            }

        });

        tableView.getComponentMouseButtonListeners().add(new ComponentMouseButtonListener() {
            @Override
            public boolean mouseDown(Component component, Mouse.Button button, final int x, final int y) {
                Logger.log.info("button:{}", button);
                SSTableView tableView = (SSTableView) component;

                if (!tableView.isSelect() && button == Mouse.Button.RIGHT) {
                    return false;
                }

//                if (button == Mouse.Button.RIGHT) {
//                    tableView.getLocation();
//                    Frame frame = new Frame("Menu");
//                    frame.mapPointToAncestor(tableView.getParent(), x, y);
//                    frame.setLocation(x, y);
//                    frame.open(tableView.getWindow());
//                }

                if (!tableView.isMouseDown()) {
                    long selectCurrentIndexQuartet = tableView.getSelectCurrentIndexQuartet();
                    final int selColIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 48));
                    final int selRowIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 32));
                    final int curColIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 16));
                    final int curRowIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet));

                    if (button == Mouse.Button.LEFT) {
                        final int columnIndex = tableView.getColumnAt(x);
                        final int rowIndex = tableView.getRowAt(y);
                        if (Keyboard.isPressed(Keyboard.Modifier.SHIFT)) {
                            selectCurrentIndexQuartet = ((long) selColIndex << 48) | ((long) selRowIndex << 32) | ((long) columnIndex << 16) | (long) rowIndex;
                        } else {
                            selectCurrentIndexQuartet = ((long) columnIndex << 48) | ((long) rowIndex << 32) | ((long) columnIndex << 16) | (long) rowIndex;
                        }
                        tableView.setSelectCurrentIndexQuartet(selectCurrentIndexQuartet);
                        tableView.setPreCellIndexPair((curColIndex << 16) | curRowIndex);
                        tableView.setMouseDown(true);
                    }
                    tableView.setSelect(true);

//                tableView.setCopyIndexQuartet(0x0000000000000000l);
//                tableView.setCopyRange(new int[]{-1, -1, -1, -1});
//                tableView.setCopied(false);
//                tableView.setPasteIndexQuartet(0x0000000000000000l);
//                tableView.setPasteRange(new int[]{-1, -1, -1, -1});
//                tableView.setPasted(false);
                    refreshDecorator(tableView);

                    SSScrollPane scrollPane = (SSScrollPane) tableView.getParent();

                    SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) scrollPane.getColumnHeader();
                    tableViewColumnHeader.setSelect(true);
                    tableViewColumnHeader.setHeaderPressed(false);
                    SSTableViewColumnHeaderFactory.refreshDecorator(tableViewColumnHeader);

                    SSTableView tableViewRowHeader = (SSTableView) scrollPane.getRowHeader();
                    tableViewRowHeader.setSelect(true);
                    tableViewRowHeader.setHeaderPressed(false);
                    SSTableViewRowHeaderFactory.refreshDecorator(tableViewRowHeader);

                    SSCorner corner = (SSCorner) scrollPane.getCorner();
                    SSCornerFactory.setDefaultSkin(corner);
                }

                return false;
            }

            @Override
            public boolean mouseUp(Component component, Mouse.Button button, int x, int y) {
                Logger.log.debug("x:{}, y:{}", x, y);
                SSTableView tableView = (SSTableView) component;

                if (tableView.isSelect()) {
                    SSScrollPane scrollPane = (SSScrollPane) tableView.getParent();
                    SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) scrollPane.getColumnHeader();
                    SSTableView tableViewRowHeader = (SSTableView) scrollPane.getRowHeader();

                    SSTableViewColumnHeaderFactory.refreshDecorator(tableViewColumnHeader);
                    SSTableViewRowHeaderFactory.refreshDecorator(tableViewRowHeader);
                    refreshDecorator(tableView);

                    tableViewRowHeader.setHeaderPressed(false);
                    tableViewColumnHeader.setHeaderPressed(false);
                    tableViewColumnHeader.getStyles().put("columnsResizable", true);
                    tableView.setMouseDown(false);
                }

                return false;
            }

            @Override
            public boolean mouseClick(Component component, Mouse.Button button, final int x, final int y, final int count) {
                Logger.log.info("button:{}", button.toString());
                SSTableView tableView = (SSTableView) component;

                if (!tableView.isMouseDown() && button == Mouse.Button.LEFT) {
                    long selectCurrentIndexQuartet = tableView.getSelectCurrentIndexQuartet();
//                    final int selColIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 48));
//                    final int selRowIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 32));
                    final int curColIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 16));
                    final int curRowIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet));

                    if (!Keyboard.isPressed(Keyboard.Modifier.SHIFT)) {
//                        tableView.setRowEditor(null);
//                        selectCurrentIndexQuartet = ((long) selColIndex << 48) | ((long) selRowIndex << 32) | ((long) tableView.getColumnAt(x) << 16) | (long) tableView.getRowAt(y);
//                        tableView.setSelectCurrentIndexQuartet(selectCurrentIndexQuartet);
                    }
                    if (tableView.isSelect() &&
                    (count == 2 && ((curColIndex << 16) | curRowIndex) == tableView.getPreCellIndexPair())) {
                        startEditColumn(tableView);
                    }
                }

                return false;
            }
        });

        tableView.getComponentMouseListeners().add(new ComponentMouseListener() {

            @Override
            public boolean mouseMove(Component component, int x, int y) {
                SSTableView tableView = (SSTableView) component;

                if (tableView.isMouseDown()) {
                    SSTableView.RowEditor tableViewRowEditor = tableView.getRowEditor();
                    if (tableViewRowEditor != null && tableViewRowEditor.isEditing()) {
                        tableViewRowEditor.endEdit(true);
                    }
                    tableView.setRowEditor(null);

                    final int colIndex = tableView.getColumnAt(x);
                    final int rowIndex = tableView.getRowAt(y);
                    mouseMoveOnTableView(tableView, colIndex, rowIndex);
                }

                if (!tableView.isMouseDown() && tableView.isSelect()) {
                    SSScrollPane scrollPane = (SSScrollPane) tableView.getParent();
                    SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) scrollPane.getColumnHeader();
                    SSTableView tableViewRowHeader = (SSTableView) scrollPane.getRowHeader();

                    SSTableViewColumnHeaderFactory.refreshDecorator(tableViewColumnHeader);
                    SSTableViewRowHeaderFactory.refreshDecorator(tableViewRowHeader);
                    refreshDecorator(tableView);
                }

                return false;
            }

            @Override
            public void mouseOver(Component component) {
//                SSTableView tableView = (SSTableView)component;
//                SSTableViewRowEditor tableViewRowEditor = tableView.getRowEditor();
//                tableViewRowEditor.beginEdit();
            }

            @Override
            public void mouseOut(Component component) {
//                Logger.log.debug("");
//                SSTableView tableView = (SSTableView) component;
//
//                if (tableView.isSelect()) {
//                    SSScrollPane scrollPane = (SSScrollPane) tableView.getParent();
//                    SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) scrollPane.getColumnHeader();
//                    SSTableView tableViewRowHeader = (SSTableView) scrollPane.getRowHeader();
//
//                    SSTableViewColumnHeaderFactory.refreshDecorator(tableViewColumnHeader);
//                    SSTableViewRowHeaderFactory.refreshDecorator(tableViewRowHeader);
//                    refreshDecorator(tableView);
//
//                    tableViewRowHeader.setHeaderPressed(false);
//                    tableViewColumnHeader.setHeaderPressed(false);
//                    tableViewColumnHeader.getStyles().put("columnsResizable", true);
//                    tableView.setMouseDown(false);
//                }
            }

        });

        tableView.getComponentMouseWheelListeners().add(new ComponentMouseWheelListener() {
            @Override
            public boolean mouseWheel(Component component, Mouse.ScrollType scrollType, int scrollAmount, int wheelRotation, int x, int y) {
                return false;
            }
        });

        return tableView;
    }

    public static Boolean startEditColumn(SSTableView tableView) {
        startEditColumn(tableView, null);

        return false;
    }

    public static Boolean startEditColumn(SSTableView tableView, Character charactor) {
        SSTableViewRowEditor tableViewRowEditor = new SSTableViewRowEditor();
        tableViewRowEditor.setEnabled(true);

        if (tableViewRowEditor.isEditing()) {
            tableViewRowEditor.endEdit(true);
            return false;
        }
        final long selectCurrentIndexQuartet = tableView.getSelectCurrentIndexQuartet();
        final int selectColIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 48));
        final int selectRowIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 32));

        tableView.setRowEditor(tableViewRowEditor);
        tableViewRowEditor.beginEdit(tableView, selectColIndex, selectRowIndex, charactor);

        return false;
    }

    public static SSTableView.Column createColumn(String name, final int width) {
        SSTableView.Column column = new SSTableView.Column();
        column.setName(name);
        column.setHeaderData(name);
        column.setWidth(width);
        column.setMaximumWidth(Config.COLUMN_MAX_WIDTH);
        column.setMinimumWidth(Config.COLUMN_MIN_WIDTH);
        column.setCellRenderer(new SSTableViewTextAreaCellRenderer());
//        column.setCellRenderer(new SSTableViewTextInputCellRenderer());

        return column;
    }

    public static void mouseMoveOnTableView(SSTableView tableView, final int columnIndex, final int rowIndex) {
        long selectCurrentIndexQuartet = tableView.getSelectCurrentIndexQuartet();
        final int selectColIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 48));
        final int selectRowIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 32));
        int currentColIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 16));
        int currentRowIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet));

        if (!tableView.isSelect() || (currentColIndex == columnIndex && currentRowIndex == rowIndex)) {
            return;
        } else {
            SSScrollPane scrollPane = (SSScrollPane) tableView.getParent();
            SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) scrollPane.getColumnHeader();
            SSTableView tableViewRowHeader = (SSTableView) scrollPane.getRowHeader();

            if (!tableViewColumnHeader.isHeaderPressed() || !tableViewRowHeader.isHeaderPressed()) {
                if (!tableViewColumnHeader.isHeaderPressed()) {
                    currentRowIndex = rowIndex;
                }
                if (!tableViewRowHeader.isHeaderPressed()) {
                    currentColIndex = columnIndex;
                }
                selectCurrentIndexQuartet = ((long) selectColIndex << 48) | ((long) selectRowIndex << 32) | ((long) currentColIndex << 16) | (long) currentRowIndex;
                tableView.setSelectCurrentIndexQuartet(selectCurrentIndexQuartet);
            }

            SSTableViewColumnHeaderFactory.refreshDecorator(tableViewColumnHeader);
            SSTableViewRowHeaderFactory.refreshDecorator(tableViewRowHeader);
            refreshDecorator(tableView);
        }
    }

    /**
     * Refresh Decorator
     *
     * @param tableView
     */
    public static void refreshDecorator(SSTableView tableView) {
        final long selectCurrentIndexQuartet = tableView.getSelectCurrentIndexQuartet();
        final int selColIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 48));
        final int selRowIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 32));
        final int curColIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 16));
        final int curRowIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet));

        if (tableView.isCopied()) {
            final long copyIndexQuartet = tableView.getCopyIndexQuartet();
            final int copyColIndexFr = (int) (0x000000000000FFFFl & (copyIndexQuartet >> 48));
            final int copyRowIndexFr = (int) (0x000000000000FFFFl & (copyIndexQuartet >> 32));
            final int copyColIndexTo = (int) (0x000000000000FFFFl & (copyIndexQuartet >> 16));
            final int copyRowIndexTo = (int) (0x000000000000FFFFl & (copyIndexQuartet));
            int[] copyRange = {-1, -1, -1, -1};

            final Bounds copyCellBoundsBgn = tableView.getCellBounds(copyRowIndexFr, copyColIndexFr);
            final Bounds copyCellBoundsEnd = tableView.getCellBounds(copyRowIndexTo, copyColIndexTo);

            copyRange[0] = copyCellBoundsBgn.x;
            copyRange[1] = copyCellBoundsBgn.y;
            copyRange[2] = copyCellBoundsEnd.x + copyCellBoundsEnd.width - copyCellBoundsBgn.x;
            copyRange[3] = copyCellBoundsEnd.y + copyCellBoundsEnd.height - copyCellBoundsBgn.y;
            tableView.setCopyRange(copyRange);
        }

        if (tableView.isCutOut()) {
            final long cutIndexQuartet = tableView.getCutIndexQuartet();
            final int cutColIndexFr = (int) (0x000000000000FFFFl & (cutIndexQuartet >> 48));
            final int cutRowIndexFr = (int) (0x000000000000FFFFl & (cutIndexQuartet >> 32));
            final int cutColIndexTo = (int) (0x000000000000FFFFl & (cutIndexQuartet >> 16));
            final int cutRowIndexTo = (int) (0x000000000000FFFFl & (cutIndexQuartet));
            int[] cutRange = {-1, -1, -1, -1};

            final Bounds cutCellBoundsBgn = tableView.getCellBounds(cutRowIndexFr, cutColIndexFr);
            final Bounds cutCellBoundsEnd = tableView.getCellBounds(cutRowIndexTo, cutColIndexTo);

            cutRange[0] = cutCellBoundsBgn.x;
            cutRange[1] = cutCellBoundsBgn.y;
            cutRange[2] = cutCellBoundsEnd.x + cutCellBoundsEnd.width - cutCellBoundsBgn.x;
            cutRange[3] = cutCellBoundsEnd.y + cutCellBoundsEnd.height - cutCellBoundsBgn.y;
            tableView.setCutRange(cutRange);
        }

        if (tableView.isPasted()) {
            final long pasteIndexQuartet = tableView.getPasteIndexQuartet();
            final int pasteColIndexFr = (int) (0x000000000000FFFFl & (pasteIndexQuartet >> 48));
            final int pasteRowIndexFr = (int) (0x000000000000FFFFl & (pasteIndexQuartet >> 32));
            final int pasteColIndexTo = (int) (0x000000000000FFFFl & (pasteIndexQuartet >> 16));
            final int pasteRowIndexTo = (int) (0x000000000000FFFFl & (pasteIndexQuartet));
            int[] pasteRange = {-1, -1, -1, -1};

            final Bounds pasteCellBoundsBegin = tableView.getCellBounds(pasteRowIndexFr, pasteColIndexFr);
            final Bounds pasteCellBoundsEnd = tableView.getCellBounds(pasteRowIndexTo, pasteColIndexTo);

            pasteRange[0] = pasteCellBoundsBegin.x;
            pasteRange[1] = pasteCellBoundsBegin.y;
            pasteRange[2] = pasteCellBoundsEnd.x + pasteCellBoundsEnd.width - pasteCellBoundsBegin.x;
            pasteRange[3] = pasteCellBoundsEnd.y + pasteCellBoundsEnd.height - pasteCellBoundsBegin.y;
            tableView.setPasteRange(pasteRange);
        }


        int[] selectRange = {-1, -1, -1, -1};
        long selectIndexQuartet = 0x0000000000000000l;
        int selectCellIndexPair = 0x00000000;

        if (tableView.isSelect()) {
            final Bounds cellBoundsBegin = tableView.getCellBounds(selRowIndex, selColIndex);
            final Bounds cellBoundsEnd = tableView.getCellBounds(curRowIndex, curColIndex);

            selectCellIndexPair = (selColIndex << 16) | selRowIndex;

            if (curColIndex > selColIndex) {
                selectIndexQuartet |= ((long) selColIndex << 48) | ((long) curColIndex << 16);

                selectRange[0] = cellBoundsBegin.x;
                selectRange[2] = cellBoundsEnd.x + cellBoundsEnd.width - cellBoundsBegin.x;
            } else {
                selectIndexQuartet |= ((long) curColIndex << 48) | ((long) selColIndex << 16);

                selectRange[0] = cellBoundsEnd.x;
                selectRange[2] = cellBoundsBegin.x + cellBoundsBegin.width - cellBoundsEnd.x;
            }
            if (curRowIndex > selRowIndex) {
                selectIndexQuartet |= ((long) selRowIndex << 32) | ((long) curRowIndex);

                selectRange[1] = cellBoundsBegin.y;
                selectRange[3] = cellBoundsEnd.y + cellBoundsEnd.height - cellBoundsBegin.y;
            } else {
                selectIndexQuartet |= ((long) curRowIndex << 32) | ((long) selRowIndex);

                selectRange[1] = cellBoundsEnd.y;
                selectRange[3] = cellBoundsBegin.y + cellBoundsBegin.height - cellBoundsEnd.y;
            }
        }
        //log.debug("selectIndex:{}", selectIndex);

        tableView.setSelectCellIndexPair(selectCellIndexPair);
        tableView.setSelectIndexQuartet(selectIndexQuartet);
        tableView.setSelectRange(selectRange);
        tableView.repaint();
    }


}
