/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.factory.tableview;

import jp.pegasus.config.Config;
import jp.pegasus.config.Location;
import jp.pegasus.log.Logger;
import jp.pegasus.pivot.content.SSTableViewCellRenderer;
import jp.pegasus.pivot.factory.SSCornerFactory;
import jp.pegasus.pivot.model.SSData;
import jp.pegasus.pivot.ss.SSCorner;
import jp.pegasus.pivot.ss.SSScrollPane;
import jp.pegasus.pivot.ss.tableview.SSTableView;
import jp.pegasus.pivot.ss.tableview.SSTableViewHeader;
import jp.pegasus.pivot.util.SSTableViewUtil;
import jp.pegasus.util.JsonUtil;
import org.apache.pivot.collections.HashMap;
import org.apache.pivot.collections.List;
import org.apache.pivot.wtk.*;

public class SSTableViewRowHeaderFactory {

    @SuppressWarnings("unchecked")
    public static SSTableView createTableViewOfRowHeader() {
        SSTableView tableViewRowHeader = new SSTableView(SSTableView.Type.RowHeader);
        JsonUtil.setStyles(tableViewRowHeader, Location.TABLE_VIEW_ROW_HEADER);
        tableViewRowHeader.getColumns().add(createColumn("_", Config.HEADER_COL_WIDTH));

        List<Object> tableRowHeaderData = (List<Object>) tableViewRowHeader.getTableData();

        for (int i = 1, rowNumber = Config.MAX_ROW_NUMBER; i < rowNumber; i++) {
            HashMap<String, Object> headerRow = new HashMap<String, Object>();
            headerRow.put("_", i);
            tableRowHeaderData.add(headerRow);
        }

        tableViewRowHeader.getComponentKeyListeners().add(new ComponentKeyListener() {

            @Override
            public boolean keyTyped(Component component, char character) {
//                Logger.log.info("");
                return false;
            }

            @Override
            @SuppressWarnings("unchecked")
            public boolean keyPressed(Component component, int keyCode, Keyboard.KeyLocation keyLocation) {
                Logger.log.info("keyCode: {}", keyCode);
                SSTableView tableViewRowHeader = (SSTableView) component;
                SSScrollPane scrollPane = (SSScrollPane) tableViewRowHeader.getParent();
                SSTableView tableView = (SSTableView) scrollPane.getView();

                long selectCurrentIndexQuartet = tableView.getSelectCurrentIndexQuartet();
                int selColIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 48));
                int selRowIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 32));
                int curColIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 16));
                int curRowIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet));

                if (tableView.isSelect()) {
                    if (false) {
                    } else if (keyCode == Keyboard.KeyCode.DELETE) {
                        SSTableViewUtil.delete(tableView);
                        return false;
                    } else if (keyCode == Keyboard.KeyCode.BACKSPACE) {
                        return false;
                    } else if (keyCode == Keyboard.KeyCode.DOWN) {
                        final int maxRowNumber = Config.MAX_ROW_NUMBER - 2;
                        if (curRowIndex != maxRowNumber &&
                        selRowIndex != maxRowNumber) {
                            if (Keyboard.isPressed(Keyboard.Modifier.SHIFT)) {
                                curRowIndex++;
                            } else if (Keyboard.isPressed(Keyboard.Modifier.CTRL)) {
                                selRowIndex = maxRowNumber;
                            } else {
                                selRowIndex++;
                            }
                        }
                    } else if (keyCode == Keyboard.KeyCode.ENTER) {
                        final int maxRowNumber = Config.MAX_ROW_NUMBER - 2;
                        if (selRowIndex != maxRowNumber) {
                            if (Keyboard.isPressed(Keyboard.Modifier.SHIFT)) {
                                curRowIndex++;
                            } else {
                                selRowIndex++;
                            }
                        }
                    } else if (keyCode == Keyboard.KeyCode.UP) {
                        if (curRowIndex != 0 && selRowIndex != 0) {
                            if (Keyboard.isPressed(Keyboard.Modifier.SHIFT)) {
                                curRowIndex--;
                            } else if (Keyboard.isPressed(Keyboard.Modifier.CTRL)) {
                                selRowIndex = 0;
                            } else {
                                selRowIndex--;
                            }
                        }
                    }
                    /**
                     * Copy
                     */
                    else if (keyCode == Keyboard.KeyCode.C &&
                    Keyboard.isPressed(Keyboard.Modifier.CTRL)) {
                        SSTableViewUtil.copy(tableView);
                        return false;
                    }
                    /**
                     * Cut
                     */
                    else if (keyCode == Keyboard.KeyCode.X &&
                    Keyboard.isPressed(Keyboard.Modifier.CTRL)) {
                        SSTableViewUtil.cut(tableView);
                        return false;
                    }
                    /**
                     * Save
                     */
                    else if (keyCode == Keyboard.KeyCode.S &&
                    Keyboard.isPressed(Keyboard.Modifier.CTRL)) {
                        final long start = System.currentTimeMillis();
//                        JsonUtil.writeTableData(tableView, Config.APP_DATA_FILE_PATH);
                        Window window = tableView.getWindow();
                        SSData data = (SSData) window.getAttribute(SSData.DEFAULT_TYPE);
                        JsonUtil.writeGrandData(data, Config.APP_DATA_FILE_PATH);
                        final long end = System.currentTimeMillis();
                        Logger.log.info("It took {}ms to save.", end - start);
                    }
                    /**
                     * Paste
                     */
                    else if (keyCode == Keyboard.KeyCode.V &&
                    Keyboard.isPressed(Keyboard.Modifier.CTRL)) {
                        SSTableViewUtil.paste(tableView);
                        refreshDecorator(tableView);
                        return false;
                    } else {
                        return false;
                    }
                }

                final Bounds visibleArea = tableView.getVisibleArea();
                final long visibleRangeQuartet = SSTableViewUtil.getVisibleRange(tableView, visibleArea.x, visibleArea.y, visibleArea.width, visibleArea.height);
                final int rowBgn = (int) (0x000000000000FFFFl & (visibleRangeQuartet >> 32));
                final int rowEnd = (int) (0x000000000000FFFFl & (visibleRangeQuartet));

                // vertical scrolling
                if (rowEnd < selRowIndex + 1) {
                    final Bounds rowBounds = tableView.getRowBounds(selRowIndex);
                    final int newY = rowBounds.y + rowBounds.height - visibleArea.height;
                    final int newRowRange = SSTableViewUtil.getVisibleRowRange(tableView, newY, visibleArea.height);
                    final int newRowStart = 0x0000FFFF & (newRowRange >> 16);
                    final Bounds newRowBounds = tableView.getRowBounds(newRowStart + 1);
                    scrollPane.setScrollTop(newRowBounds.y);
                } else if (selRowIndex < rowBgn) {
                    final Bounds newRowBounds = tableView.getRowBounds(selRowIndex);
                    scrollPane.setScrollTop(newRowBounds.y);
                }
                ///////////////////////////////////////////////////////////////////////////////////

                if ((keyCode == Keyboard.KeyCode.TAB && Keyboard.isPressed(Keyboard.Modifier.SHIFT)) ||
                !Keyboard.isPressed(Keyboard.Modifier.SHIFT)) {
                    selectCurrentIndexQuartet = ((long) selColIndex << 48) | ((long) selRowIndex << 32) | ((long) selColIndex << 16) | (long) selRowIndex;
                } else {
                    selectCurrentIndexQuartet = ((long) selColIndex << 48) | ((long) selRowIndex << 32) | ((long) curColIndex << 16) | (long) curRowIndex;
                }
                tableView.setSelectCurrentIndexQuartet(selectCurrentIndexQuartet);

                SSCorner corner = (SSCorner) scrollPane.getCorner();
                SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) scrollPane.getColumnHeader();

                SSCornerFactory.setDefaultSkin(corner);
                SSTableViewColumnHeaderFactory.refreshDecorator(tableViewColumnHeader);
                SSTableViewRowHeaderFactory.refreshDecorator(tableViewRowHeader);
                refreshDecorator(tableView);

                tableView.setMouseDown(false);

                return false;
            }

            @Override
            public boolean keyReleased(Component component, int keyCode, Keyboard.KeyLocation keyLocation) {
//                Logger.log.info("");
                return false;
            }
        });


        tableViewRowHeader.getComponentMouseListeners().add(new ComponentMouseListener() {

            @Override
            public boolean mouseMove(Component component, int x, int y) {
                SSTableView tableViewRowHeader = (SSTableView) component;
                SSScrollPane scrollPane = (SSScrollPane) tableViewRowHeader.getParent();
                SSTableView tableView = (SSTableView) scrollPane.getView();

                if (tableView.isMouseDown()) {
                    final int rowIndex = tableViewRowHeader.getRowAt(y);
                    mouseMoveOnTableViewRowHeader(tableViewRowHeader, rowIndex);
                }

                if (!tableView.isMouseDown() && tableView.isSelect()) {
                    SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) scrollPane.getColumnHeader();
                    SSTableViewColumnHeaderFactory.refreshDecorator(tableViewColumnHeader);
                    SSTableViewFactory.refreshDecorator(tableView);
                    refreshDecorator(tableViewRowHeader);
                }

                return false;
            }

            @Override
            public void mouseOver(Component component) {
                Logger.log.debug("");
            }

            @Override
            public void mouseOut(Component component) {
                Logger.log.debug("");
            }
        });

        tableViewRowHeader.getComponentMouseButtonListeners().add(new ComponentMouseButtonListener() {

            @Override
            public boolean mouseDown(Component component, Mouse.Button button, int x, int y) {
                SSTableView tableViewRowHeader = (SSTableView) component;
                SSScrollPane scrollPane = (SSScrollPane) tableViewRowHeader.getParent();
                SSTableView tableView = (SSTableView) scrollPane.getView();

                if (button == Mouse.Button.LEFT) {
                    long selectCurIndexQuartet = tableView.getSelectCurrentIndexQuartet();
                    int selectRowIndex = (int) (0x000000000000FFFFl & (selectCurIndexQuartet >> 32));

                    final int rowIndex = tableViewRowHeader.getRowAt(y);
                    if (!Keyboard.isPressed(Keyboard.Modifier.SHIFT)) {
                        selectRowIndex = rowIndex;
                    }
                    selectCurIndexQuartet = (0l << 48) | ((long) selectRowIndex << 32) | ((long) (Config.MAX_COL_NUMBER - 2) << 16) | (long) rowIndex;
                    tableView.setSelectCurrentIndexQuartet(selectCurIndexQuartet);

                    SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) scrollPane.getColumnHeader();

                    tableViewColumnHeader.setSelect(true);
                    tableViewRowHeader.setHeaderPressed(true);
                    tableViewRowHeader.setSelect(true);
                    tableView.setMouseDown(true);
                    tableView.setSelect(true);

                    SSTableViewColumnHeaderFactory.refreshDecorator(tableViewColumnHeader);
                    SSTableViewFactory.refreshDecorator(tableView);
                    refreshDecorator(tableViewRowHeader);

                    SSCorner corner = (SSCorner) scrollPane.getCorner();
                    SSCornerFactory.setDefaultSkin(corner);
                }

                return false;
            }

            @Override
            public boolean mouseUp(Component component, Mouse.Button button, int x, int y) {
                SSTableView tableViewRowHeader = (SSTableView) component;
                SSScrollPane scrollPane = (SSScrollPane) tableViewRowHeader.getParent();
                SSTableView tableView = (SSTableView) scrollPane.getView();

                if (tableView.isSelect()) {
                    SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) scrollPane.getColumnHeader();
                    SSTableViewColumnHeaderFactory.refreshDecorator(tableViewColumnHeader);
                    SSTableViewFactory.refreshDecorator(tableView);
                    refreshDecorator(tableViewRowHeader);

                    tableViewColumnHeader.setHeaderPressed(false);
                    tableViewColumnHeader.getStyles().put("columnsResizable", true);
                    tableView.setMouseDown(false);
                }

                return false;
            }

            @Override
            public boolean mouseClick(Component component, Mouse.Button button, int x, int y, int count) {
                return false;
            }
        });

        return tableViewRowHeader;
    }

    public static SSTableView.Column createColumn(String name, final int width) {
        SSTableView.Column column = new SSTableView.Column();
        column.setName(name);

        column.setHeaderData(name);
        column.setWidth(width);
        column.setMaximumWidth(Config.COLUMN_MAX_WIDTH);
        column.setMinimumWidth(Config.COLUMN_MIN_WIDTH);
        column.setCellRenderer(new SSTableViewCellRenderer());
        column.getHeaderDataRenderer().getStyles().put("horizontalAlignment", HorizontalAlignment.CENTER);
        column.getCellRenderer().getStyles().put("horizontalAlignment", HorizontalAlignment.CENTER);

        return column;
    }

    public static void mouseMoveOnTableViewRowHeader(SSTableView tableViewRowHeader, final int rowIndex) {
        SSScrollPane scrollPane = (SSScrollPane) tableViewRowHeader.getParent();
        SSTableView tableView = (SSTableView) scrollPane.getView();

        long selectCurIndexQuartet = tableView.getSelectCurrentIndexQuartet();
        final int selColIndex = (int) (0x000000000000FFFFl & (selectCurIndexQuartet >> 48));
        final int selRowIndex = (int) (0x000000000000FFFFl & (selectCurIndexQuartet >> 32));
        final int curColIndex = (int) (0x000000000000FFFFl & (selectCurIndexQuartet >> 16));
        final int curRowIndex = (int) (0x000000000000FFFFl & (selectCurIndexQuartet));

        if (curRowIndex == rowIndex) {
            return;
        } else if (tableView.isSelect()) {
            SSTableViewHeader tableViewColumnHeader = (SSTableViewHeader) scrollPane.getColumnHeader();
            if (!tableViewColumnHeader.isHeaderPressed()) {
                selectCurIndexQuartet = ((long) selColIndex << 48) | ((long) selRowIndex << 32) | ((long) curColIndex << 16) | (long) rowIndex;
                tableView.setSelectCurrentIndexQuartet(selectCurIndexQuartet);
            }

            SSTableViewColumnHeaderFactory.refreshDecorator(tableViewColumnHeader);
            SSTableViewFactory.refreshDecorator(tableView);
            refreshDecorator(tableViewRowHeader);
        }
    }

    public static void refreshDecorator(SSTableView tableViewRowHeader) {
        SSScrollPane scrollPane = (SSScrollPane) tableViewRowHeader.getParent();
        SSTableView tableView = (SSTableView) scrollPane.getView();

        final long selectCurIndexQuartet = tableView.getSelectCurrentIndexQuartet();
        final int selRowIndex = (int) (0x000000000000FFFFl & (selectCurIndexQuartet >> 32));
        final int curRowIndex = (int) (0x000000000000FFFFl & (selectCurIndexQuartet));

        int[] selectRange = {-1, -1, -1, -1};
        long selectIndexQuartet = 0x0000000000000000l;

        if (!(selRowIndex == -1 && curRowIndex == -1)) {
            final Bounds cellBoundsBgn = tableViewRowHeader.getCellBounds(selRowIndex, 0);
            final Bounds cellBoundsEnd = tableViewRowHeader.getCellBounds(curRowIndex, 0);

            selectRange[0] = 0;
            selectRange[2] = Config.HEADER_COL_WIDTH;

            if (curRowIndex > selRowIndex) {
                selectIndexQuartet |= ((long) selRowIndex << 32) | ((long) curRowIndex);

                selectRange[1] = cellBoundsBgn.y;
                selectRange[3] = cellBoundsEnd.y + cellBoundsEnd.height - cellBoundsBgn.y;
            } else {
                selectIndexQuartet |= ((long) curRowIndex << 32) | ((long) selRowIndex);

                selectRange[1] = cellBoundsEnd.y;
                selectRange[3] = cellBoundsBgn.y + cellBoundsBgn.height - cellBoundsEnd.y;
            }
        }

        tableViewRowHeader.setSelectIndexQuartet(selectIndexQuartet);
        tableViewRowHeader.setSelectRange(selectRange);
        tableViewRowHeader.repaint();
    }

}
