/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.factory.tabpane;

import jp.pegasus.config.Config;
import jp.pegasus.config.Location;
import jp.pegasus.log.Logger;
import jp.pegasus.pivot.factory.ScrollPaneFactory;
import jp.pegasus.pivot.listener.tabpane.SSTabPaneSelectionListener;
import jp.pegasus.pivot.model.*;
import jp.pegasus.pivot.skin.SSTabPaneSkin;
import jp.pegasus.pivot.ss.SSBorder;
import jp.pegasus.pivot.ss.SSScrollPane;
import jp.pegasus.pivot.ss.SSTabPane;
import jp.pegasus.pivot.ss.SSTextArea;
import jp.pegasus.pivot.ss.tableview.SSTableView;
import jp.pegasus.util.JsonUtil;
import org.apache.pivot.collections.ArrayList;
import org.apache.pivot.collections.HashMap;
import org.apache.pivot.collections.List;
import org.apache.pivot.util.Vote;
import org.apache.pivot.wtk.*;
import org.apache.pivot.wtk.content.ButtonData;

import java.io.File;


public class SSTabPaneContentFactory {

    private static int previousClickIndex;

    public static SSTabPane createTabPane(List<SSScrollPane> scrollPaneList) {
        SSTabPane tabPane = new SSTabPane();
        JsonUtil.setStyles(tabPane, Location.TAB_PANE);

        tabPane.getComponentMouseButtonListeners().add(new ComponentMouseButtonListener() {
            @Override
            public boolean mouseDown(Component component, Mouse.Button button, int x, int y) {
                return false;
            }

            @Override
            public boolean mouseUp(Component component, Mouse.Button button, int x, int y) {
                return false;
            }

            @Override
            public boolean mouseClick(Component component, Mouse.Button button, int x, int y, int count) {
                SSTabPane tabPane = (SSTabPane) component;

                if (button == Mouse.Button.LEFT) {
                    final int clickIndex = tabPane.getSelectedIndex();
                    final int tabNumber = tabPane.getTabs().getLength();
                    Logger.log.info("tabPane.getSelectedIndex:{}, tabNumber:{}", tabPane.getSelectedIndex(), tabNumber);

                    /**
                     * Create New Tab
                     */
                    if (tabPane.getSelectedIndex() == tabNumber - 1) {
                        Window window = tabPane.getWindow();
                        if (window != null) {
                            SSData data = (SSData) window.getAttribute(SSData.DEFAULT_TYPE);
                            final int selectFileIndex = data.getSelectFileIndex();
                            SSFileData selectFileData = data.getFileDataList().get(selectFileIndex);
                            addSheet(tabPane, selectFileData);
                        }
                        return false;
                    }

                    /**
                     * Switch Tab
                     */
                    if (previousClickIndex != clickIndex) {
                        Window window = tabPane.getWindow();
                        if (window != null) {
                            /**
                             * Open clicked Tab
                             */
                            SSData data = (SSData) window.getAttribute(SSData.DEFAULT_TYPE);
                            final int selectFileIndex = data.getSelectFileIndex();
                            SSFileData selectFileData = data.getFileDataList().get(selectFileIndex);
                            selectFileData.setSelectSheetIndex(clickIndex);
                            Object sheetData = selectFileData.getSheetDataList().get(clickIndex);

                            final int selectSheetType = selectFileData.getSheetType();
                            if (selectSheetType == 0) {
                                SSGridData gridData = null;
                                if (sheetData instanceof HashMap) {
                                    gridData = SSGridData.convertMapToObject((HashMap<String, ?>) sheetData);
                                } else if (sheetData instanceof SSGridData) {
                                    gridData = (SSGridData) sheetData;
                                }

                                String cellDataDigest = gridData.getCellDataDigest();
                                ArrayList<HashMap<String, SSElementalData>> cellDataList = null;
                                if (cellDataDigest == null) {
                                    cellDataList = SSElementalData.createCellDataList();

                                    String digest = JsonUtil.getDigest(cellDataList, Config.DIGEST_ARGORITHM);
                                    StringBuilder cellDataFilePath = new StringBuilder().append(Config.APP_CELL_DATA_DIR).append(File.separator).append(digest);
                                    JsonUtil.writeSheetData(cellDataList, cellDataFilePath.toString());
                                    gridData.setCellDataDigest(digest);
                                } else {
                                    String cellDataFilePath = Config.APP_CELL_DATA_DIR + File.separator + cellDataDigest;
                                    cellDataList = JsonUtil.readCellDataList(cellDataFilePath);
                                }
                            } else if (selectSheetType == 1) {
                                SSTextData textData = null;
                                if (sheetData instanceof HashMap) {
                                    textData = SSTextData.convertMapToObject((HashMap<String, ?>) sheetData);
                                } else if (sheetData instanceof SSGridData) {
                                    textData = (SSTextData) sheetData;
                                }

                            } else {
                                throw new RuntimeException();
                            }


                            previousClickIndex = clickIndex;
                        }

                    } else {
                        Component pointComponent = tabPane.getComponentAt(x, y);
                        if (pointComponent != null) {
                            Logger.log.debug("component:" + pointComponent.toString());
                            /**
                             * Rename Tab
                             */
                            if (pointComponent instanceof Panorama && count == 2) {
                                SSTabPane.TabEditor tabEditor = tabPane.getTabEditor();
                                tabEditor.beginEdit(tabPane);
                            } else if (pointComponent instanceof jp.pegasus.pivot.ss.SSBorder) {
                                Component content = ((SSBorder) pointComponent).getContent();
                                Logger.log.debug("content:" + content.toString());
                                if (content instanceof SSScrollPane) {
                                    SSScrollPane scrollPane = (SSScrollPane) content;
                                    Component view = scrollPane.getView();
                                    if (view instanceof SSTextArea) {
                                        SSTextArea textArea = (SSTextArea) view;
                                        textArea.requestFocus();
                                    }
                                }
                            }
                        }
                    }
                } else if (button == Mouse.Button.RIGHT) {
                    Component componentWork = tabPane.getComponentAt(x, y);
                    Logger.log.info("componentWork: {}", componentWork);
                }
                return false;
            }
        });

        tabPane.getTabPaneSelectionListeners().add(new SSTabPaneSelectionListener() {
            @Override
            public Vote previewSelectedIndexChange(SSTabPane tabPane, int selectedIndex) {
                Logger.log.info("selectedIndex:{}", selectedIndex);

                /**
                 * Save previous Sheet
                 */
                Window window = tabPane.getWindow();
                if (window != null) {
                    SSData data = (SSData) window.getAttribute(SSData.DEFAULT_TYPE);

                    final int selectFileIndex = data.getSelectFileIndex();
                    SSFileData selectFileData = data.getFileDataList().get(selectFileIndex);

                    final int selectSheetIndex = selectFileData.getSelectSheetIndex();
                    final int selectSheetType = selectFileData.getSheetType();
                    Object sheetData = selectFileData.getSheetDataList().get(selectSheetIndex);
                    Object object = tabPane.getSelectedTab();
                    if (object instanceof SSBorder) {
                        object = ((SSBorder) object).getContent();
                    }
                    if (object instanceof SSScrollPane) {
                        SSScrollPane scrollPane = (SSScrollPane) object;
                        if (sheetData instanceof SSGridData) {
                            SSTableView tableView = (SSTableView) scrollPane.getView();
                            ArrayList<HashMap<String, SSElementalData>> cellDataList = (ArrayList<HashMap<String, SSElementalData>>) tableView.getTableData();

                            String digest = JsonUtil.getDigest(cellDataList, Config.DIGEST_ARGORITHM);
                            StringBuilder cellDataFilePath = new StringBuilder(Config.APP_CELL_DATA_DIR).append(File.separator).append(digest);
                            JsonUtil.writeSheetData(cellDataList, cellDataFilePath.toString());
                            ((SSGridData) sheetData).setCellDataDigest(digest);
                        } else if (sheetData instanceof SSTextData) {
                            final SSTextArea textArea = (SSTextArea) scrollPane.getView();
                            SSElementalData elementalData = new SSElementalData() {{
                                this.setText(textArea.getText());
                            }};

                            String digest = JsonUtil.getDigest(elementalData, Config.DIGEST_ARGORITHM);
                            StringBuilder textDataFilePath = new StringBuilder(Config.APP_TEXT_DATA_DIR).append(File.separator).append(digest);
                            JsonUtil.writeSheetData(elementalData, textDataFilePath.toString());
                            ((SSTextData) sheetData).setTextDataDigest(digest);
                        } else if (sheetData instanceof HashMap) {
                            if (selectSheetType == 0) {
                                SSTableView tableView = (SSTableView) scrollPane.getView();
                                ArrayList<HashMap<String, SSElementalData>> cellDataList = (ArrayList<HashMap<String, SSElementalData>>) tableView.getTableData();

                                String digest = JsonUtil.getDigest(cellDataList, Config.DIGEST_ARGORITHM);
                                StringBuilder cellDataFilePath = new StringBuilder(Config.APP_CELL_DATA_DIR).append(File.separator).append(digest);
                                JsonUtil.writeSheetData(cellDataList, cellDataFilePath.toString());
                                ((HashMap<String, String>) sheetData).put("cellDataDigest", digest);
                            } else if (selectSheetType == 1) {
                                final SSTextArea textArea = (SSTextArea) scrollPane.getView();
                                SSElementalData elementalData = new SSElementalData() {{
                                    this.setText(textArea.getText());
                                }};

                                String digest = JsonUtil.getDigest(elementalData, Config.DIGEST_ARGORITHM);
                                StringBuilder textDataFilePath = new StringBuilder(Config.APP_TEXT_DATA_DIR).append(File.separator).append(digest);
                                JsonUtil.writeSheetData(elementalData, textDataFilePath.toString());
                                ((HashMap<String, String>) sheetData).put("textDataDigest", digest);
                            } else {
                                throw new RuntimeException();
                            }
                        } else {
                            throw new RuntimeException();
                        }
                        JsonUtil.writeGrandData(data, Config.APP_DATA_FILE_PATH);
                    }
                }
                return Vote.APPROVE;
            }

            @Override
            public void selectedIndexChangeVetoed(SSTabPane tabPane, Vote reason) {
                Logger.log.info("reason:{}", reason.toString());
            }

            @Override
            public void selectedIndexChanged(SSTabPane tabPane, int previousSelectedIndex) {
                Logger.log.info("previousSelectedIndex:{}", previousSelectedIndex);
            }
        });

        return tabPane;
    }

    /**
     * Construct TabPane Content
     *
     * @param fileData
     * @return
     */
    public static SSTabPane constructTabPaneContent(SSFileData fileData) {
        /**
         * Sheet Data
         */
        ArrayList<SSScrollPane> scrollPaneList = new ArrayList<SSScrollPane>();
        SSTabPane tabPane = SSTabPaneContentFactory.createTabPane(scrollPaneList);

        final int sheetType = fileData.getSheetType();
        ArrayList<Object> sheetDataList = fileData.getSheetDataList();
        for (Object sheetData : sheetDataList) {
            /**
             * Table Data
             */
            if (sheetType == 0) {
                SSGridData gridData = null;
                if (sheetData instanceof HashMap) {
                    gridData = SSGridData.convertMapToObject((HashMap<String, ?>) sheetData);
                } else {
                    gridData = (SSGridData) sheetData;
                }
                ScrollPaneFactory.constructGridScrollPane(tabPane, gridData);
            } else if (sheetType == 1) {
                SSTextData textData = null;
                if (sheetData instanceof HashMap) {
                    textData = SSTextData.convertMapToObject((HashMap<String, ?>) sheetData);
                } else {
                    textData = (SSTextData) sheetData;
                }
                ScrollPaneFactory.constructTextScrollPane(tabPane, textData);
            } else {
                throw new RuntimeException();
            }
        }

        final int selectSheetIndex = fileData.getSelectSheetIndex();
        tabPane.setSelectedIndex(selectSheetIndex);

        return tabPane;
    }

    /**
     * Add Sheet
     *
     * @param tabPane
     * @param selectFileData
     */
    private static void addSheet(SSTabPane tabPane, SSFileData selectFileData) {
        Window window = tabPane.getWindow();
        if (window != null) {
            final int selectedTabIndex = tabPane.getSelectedIndex();
            Logger.log.debug("selectedTabIndex:{}", selectedTabIndex);
            Object sheetData = selectFileData.getSheetDataList().get(selectedTabIndex);

            final String sheetName = createSheetName(selectFileData, 1);
//            Logger.log.debug("tabCount:{}", tabCount);
            Component selectedTab = tabPane.getSelectedTab();
            SSTabPane.setTabData(selectedTab, new ButtonData(sheetName) {{
                setUserData(SSTabPaneSkin.Type.Sheet);
            }});

            final int tabCount = tabPane.getTabs().getLength();
            tabPane.setSelectedIndex(tabCount - 1);

            final int selectSheetType = selectFileData.getSheetType();
            if (sheetData instanceof HashMap) {
                ((HashMap) sheetData).put("sheetName", sheetName);
                ((HashMap) sheetData).put("createTime", System.currentTimeMillis());
                if (selectSheetType == 0) {
                    final SSGridData newGridData = SSData.createGridSheet(SSData.DEF_SHT_ADD);
                    ScrollPaneFactory.constructGridScrollPane(tabPane, newGridData);
                    selectFileData.getSheetDataList().add(newGridData);
                } else if (selectSheetType == 1) {
                    final SSTextData newTextData = SSData.createTextSheet(SSData.DEF_SHT_ADD);
                    ScrollPaneFactory.constructTextScrollPane(tabPane, newTextData);
                    selectFileData.getSheetDataList().add(newTextData);
                } else {
                    throw new RuntimeException();
                }

            } else if (sheetData instanceof SSGridData) {
                ((SSGridData) sheetData).setSheetName(sheetName);
                ((SSGridData) sheetData).setCreateTime(System.currentTimeMillis());
                SSGridData newGridData = SSData.createGridSheet(SSData.DEF_SHT_ADD);
                ScrollPaneFactory.constructGridScrollPane(tabPane, newGridData);
                selectFileData.getSheetDataList().add(newGridData);
            } else if (sheetData instanceof SSTextData) {
                ((SSTextData) sheetData).setSheetName(sheetName);
                ((SSTextData) sheetData).setCreateTime(System.currentTimeMillis());
                SSTextData newTextData = SSData.createTextSheet(SSData.DEF_SHT_ADD);
                ScrollPaneFactory.constructTextScrollPane(tabPane, newTextData);
                selectFileData.getSheetDataList().add(newTextData);
            } else {
                throw new RuntimeException();
            }

//            SSWindowFactory.save(window);
        }
    }

    private static String createSheetName(SSFileData selectFileData, int number) {
        ArrayList<Object> sheetDataList = selectFileData.getSheetDataList();
        for (Object sheetData : sheetDataList) {

            String sheetName = null;
            if (sheetData instanceof SSGridData) {
                sheetName = ((SSGridData) sheetData).getSheetName();
            } else if (sheetData instanceof SSTextData) {
                sheetName = ((SSTextData) sheetData).getSheetName();
            } else if (sheetData instanceof HashMap) {
                sheetName = (String) ((HashMap<String, ?>) sheetData).get("sheetName");
            } else {
                throw new RuntimeException();
            }

            if (sheetName != null && sheetName.equalsIgnoreCase(String.format(SSData.DEF_SHT_NAME, number))) {
                return createSheetName(selectFileData, ++number);
            }
        }

        return String.format(SSData.DEF_SHT_NAME, number);
    }


}
