/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.factory.tabpane;

import jp.pegasus.config.Location;
import jp.pegasus.pivot.factory.ScrollPaneFactory;
import jp.pegasus.pivot.factory.tableview.SSTableViewFactory;
import jp.pegasus.pivot.model.SSElementalData;
import jp.pegasus.pivot.skin.SSTabPaneSkin;
import jp.pegasus.pivot.ss.SSBorder;
import jp.pegasus.pivot.ss.SSScrollPane;
import jp.pegasus.pivot.ss.SSTabPane;
import jp.pegasus.pivot.ss.tableview.SSTableView;
import jp.pegasus.pivot.ss.tableview.SSTableViewHeader;
import jp.pegasus.util.JsonUtil;
import org.apache.pivot.collections.ArrayList;
import org.apache.pivot.collections.HashMap;
import org.apache.pivot.wtk.ApplicationContext;
import org.apache.pivot.wtk.ScrollPane;
import org.apache.pivot.wtk.content.ButtonData;

import java.lang.management.ManagementFactory;
import java.lang.management.RuntimeMXBean;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

public class SSTabPanePropertyFactory {

    /**
     * Construct TabPane Property
     *
     * @return
     */
    public static SSTabPane constructTabPaneProperty() {
        SSTabPane tabPane = new SSTabPane();
        JsonUtil.setStyles(tabPane, Location.TAB_PANE_PP);

        /**
         * Get Utility Pane
         */
        SSBorder utilityPane = getUtilityPane();
        SSTabPane.setTabData(utilityPane, new ButtonData(String.format("%-8s", "Utility")) {{
            setUserData(SSTabPaneSkin.Type.Property);
        }});
        tabPane.getTabs().add(utilityPane);

        /**
         * Get Metadata Pane
         */
        SSBorder scrollPaneMD = getMetadataPane();
        SSTabPane.setTabData(scrollPaneMD, new ButtonData(String.format("%-8s", "Metadata")) {{
            setUserData(SSTabPaneSkin.Type.Property);
        }});
        tabPane.getTabs().add(scrollPaneMD);

        /**
         * Get Change History Pane
         */
        SSBorder scrollPaneCH = getChangeHistoryPane();
        SSTabPane.setTabData(scrollPaneCH, new ButtonData(String.format("%-8s", "History")) {{
            setUserData(SSTabPaneSkin.Type.Property);
        }});
        tabPane.getTabs().add(scrollPaneCH);

        return tabPane;
    }

    private static SSBorder getUtilityPane() {
        final SSTableView tableView = new SSTableView(SSTableView.Type.Property);
        JsonUtil.setStyles(tableView, Location.TABLE_VIEW_PP);

        SSTableView.Column colName = SSTableViewFactory.createColumn("Name", 100);
        SSTableView.Column colValue = SSTableViewFactory.createColumn("Value", 150);
        tableView.getColumns().add(colName);
        tableView.getColumns().add(colValue);

        final long startUpTime = System.currentTimeMillis();

        ApplicationContext.scheduleRecurringCallback(new Runnable() {
            @Override
            public void run() {
                final NumberFormat form = NumberFormat.getNumberInstance();
                final double ttalMem = Runtime.getRuntime().totalMemory() / 1000000.0;
                final double freeMem = Runtime.getRuntime().freeMemory() / 1000000.0;
                final double usedMem = ttalMem - freeMem;
                final Date date = new Date();

                tableView.setTableData(
                new ArrayList<HashMap<String, SSElementalData>>() {{
                    add(new HashMap<String, SSElementalData>() {{
                        put("Name", new SSElementalData() {{
                            setText("Elapsed Seconds");
                        }});
                        put("Value", new SSElementalData() {
                            long elapsedSeconds = (System.currentTimeMillis() - startUpTime) / 1000;

                            {
                                setText(String.format("%d ms", elapsedSeconds));
                            }
                        });
                    }});
                    add(new HashMap<String, SSElementalData>() {{
                        put("Name", new SSElementalData() {{
                            setText("Current Date");
                        }});
                        put("Value", new SSElementalData() {{
                            setText(new SimpleDateFormat("yyyy/MM/dd").format(date));
                        }});
                    }});
                    add(new HashMap<String, SSElementalData>() {{
                        put("Name", new SSElementalData() {{
                            setText("Current Time");
                        }});
                        put("Value", new SSElementalData() {{
                            setText(new SimpleDateFormat("HH:mm:ss z").format(date));
                        }});
                    }});
                    add(new HashMap<String, SSElementalData>() {{
                        put("Name", new SSElementalData() {{
                            setText("Memory(Total)");
                        }});
                        put("Value", new SSElementalData() {{
                            setText(String.format("%-8s MB", form.format(ttalMem)));
                        }});
                    }});
                    add(new HashMap<String, SSElementalData>() {{
                        put("Name", new SSElementalData() {{
                            setText("Memory(Free)");
                        }});
                        put("Value", new SSElementalData() {{
                            setText(String.format("%-8s MB", form.format(freeMem)));
                        }});
                    }});
                    add(new HashMap<String, SSElementalData>() {{
                        put("Name", new SSElementalData() {{
                            setText("Memory(Used)");
                        }});
                        put("Value", new SSElementalData() {{
                            setText(String.format("%-8s MB", form.format(usedMem)));
                        }});
                    }});
                }}
                );
            }
        }, 1000, 1000);

        SSTableViewHeader tableViewColumnHeader = new SSTableViewHeader();
        JsonUtil.setStyles(tableViewColumnHeader, Location.TABLE_VIEW_COL_HEADER_PP);
        tableViewColumnHeader.setTableView(tableView);

        SSScrollPane scrollPane = ScrollPaneFactory.createScrollPane(ScrollPane.ScrollBarPolicy.AUTO, ScrollPane.ScrollBarPolicy.AUTO);
        scrollPane.setView(tableView);
        scrollPane.setColumnHeader(tableViewColumnHeader);

        SSBorder border = new SSBorder();
        JsonUtil.setStyles(border, Location.BORDER_PP);
        border.setContent(scrollPane);

        return border;
    }

    private static SSBorder getMetadataPane() {
        SSTableView tableView = new SSTableView(SSTableView.Type.SpreadSheet);
        JsonUtil.setStyles(tableView, Location.TABLE_VIEW_PP);

        SSTableView.Column colName = SSTableViewFactory.createColumn("Name", 100);
        SSTableView.Column colValue = SSTableViewFactory.createColumn("Value", 200);
        tableView.getColumns().add(colName);
        tableView.getColumns().add(colValue);

        tableView.setTableData(new ArrayList<HashMap<String, SSElementalData>>() {{
            RuntimeMXBean runtimemxBean = ManagementFactory.getRuntimeMXBean();
            java.util.List<String> arguments = runtimemxBean.getInputArguments();

            int counter = 1;
            for (final String argument : arguments) {
                final int number = counter;
                add(new HashMap<String, SSElementalData>() {{
                    put("Name", new SSElementalData() {{
                        setText(String.format("JVM Argument %2d", number));
                    }});
                    put("Value", new SSElementalData() {{
                        setText(argument);
                    }});
                }});
                counter++;
            }
        }});

        SSTableViewHeader tableViewColumnHeader = new SSTableViewHeader();
        JsonUtil.setStyles(tableViewColumnHeader, Location.TABLE_VIEW_COL_HEADER_PP);
        tableViewColumnHeader.setTableView(tableView);

        SSScrollPane scrollPane = ScrollPaneFactory.createScrollPane(ScrollPane.ScrollBarPolicy.AUTO, ScrollPane.ScrollBarPolicy.AUTO);
        scrollPane.setView(tableView);
        scrollPane.setColumnHeader(tableViewColumnHeader);

        SSBorder border = new SSBorder();
        JsonUtil.setStyles(border, Location.BORDER_PP);
        border.setContent(scrollPane);

        return border;
    }

    private static SSBorder getChangeHistoryPane() {
        SSTableView tableView = new SSTableView(SSTableView.Type.SpreadSheet);
        JsonUtil.setStyles(tableView, Location.TABLE_VIEW_PP);

        SSTableViewHeader tableViewColumnHeader = new SSTableViewHeader();
        JsonUtil.setStyles(tableViewColumnHeader, Location.TABLE_VIEW_COL_HEADER_PP);
        tableViewColumnHeader.setTableView(tableView);

        SSScrollPane scrollPane = ScrollPaneFactory.createScrollPane(ScrollPane.ScrollBarPolicy.AUTO, ScrollPane.ScrollBarPolicy.AUTO);
        scrollPane.setView(tableView);
        scrollPane.setColumnHeader(tableViewColumnHeader);

        SSBorder border = new SSBorder();
        JsonUtil.setStyles(border, Location.BORDER_PP);
        border.setContent(scrollPane);

        return border;
    }


}
