/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.factory.treeview;

import jp.pegasus.config.Location;
import jp.pegasus.log.Logger;
import jp.pegasus.pivot.factory.SSTreeNodeFactory;
import jp.pegasus.pivot.skin.SSTabPaneSkin;
import jp.pegasus.pivot.ss.SSPalette;
import jp.pegasus.pivot.ss.SSTreeNode;
import jp.pegasus.pivot.ss.SSTreeView;
import jp.pegasus.pivot.util.SSTabPaneUtil;
import jp.pegasus.util.JsonUtil;
import org.apache.pivot.collections.ArrayList;
import org.apache.pivot.wtk.Component;
import org.apache.pivot.wtk.ComponentMouseButtonListener;
import org.apache.pivot.wtk.Mouse;

public class SSTreeViewOfPopupTabPaneFactory {

    public static SSTreeView createTreeViewOfPopupTabPane(SSTabPaneSkin.TabButton tabButton) {
        SSTreeView treeView = new SSTreeView(SSTreeView.Type.PopupTabPane);
        JsonUtil.setStyles(treeView, Location.TREE_VIEW);
        treeView.setTabButton(tabButton);
        treeView.setTreeData(
        new ArrayList<SSTreeNode>() {{
            add(SSTreeNodeFactory.createTreeNode(SSTreeNode.NodeType.TabDel));
        }});

        treeView.getComponentMouseButtonListeners().add(new ComponentMouseButtonListener() {
            @Override
            public boolean mouseDown(Component component, Mouse.Button button, int x, int y) {
                Logger.log.info("");
                SSTreeView treeView = (SSTreeView) component;
                SSTabPaneSkin.TabButton tabButton = treeView.getTabButton();

                SSTreeNode treeNode = (SSTreeNode) treeView.getSelectedNode();
                if (treeNode != null) {
                    switch (treeNode.getNodeType()) {
                        case TabDel:
                            SSTabPaneUtil.delete(tabButton);
                            break;
                        default:
                            throw new RuntimeException();
                    }
                    final String text = treeNode.getText();
                    Logger.log.debug(text);

                    treeView.clearSelect();
                    SSPalette palette = (SSPalette) treeView.getParent();
                    palette.close();
                }
                return false;
            }

            @Override
            public boolean mouseUp(Component component, Mouse.Button button, int x, int y) {
                Logger.log.info("");
                return false;
            }

            @Override
            public boolean mouseClick(Component component, Mouse.Button button, int x, int y, int count) {
                return false;
            }
        });

        return treeView;
    }
}
