/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.factory.treeview;

import jp.pegasus.config.Location;
import jp.pegasus.log.Logger;
import jp.pegasus.pivot.editor.SSTreeViewNodeEditor;
import jp.pegasus.pivot.listener.treeview.SSTreeViewNodeListener;
import jp.pegasus.pivot.listener.treeview.SSTreeViewSelectListener;
import jp.pegasus.pivot.model.SSData;
import jp.pegasus.pivot.ss.SSTabPane;
import jp.pegasus.pivot.ss.SSTreeBranch;
import jp.pegasus.pivot.ss.SSTreeNode;
import jp.pegasus.pivot.ss.SSTreeView;
import jp.pegasus.util.JsonUtil;
import org.apache.pivot.collections.Sequence;
import org.apache.pivot.wtk.*;

public class SSTreeViewOfWindowMenuFactory {

    private static int previousClickIndex;

    public static SSTreeView createTreeViewOfWindowMenu() {
        SSTreeView treeView = new SSTreeView(SSTreeView.Type.WindowMenu);
        JsonUtil.setStyles(treeView, Location.TREE_VIEW_MENU);

        treeView.getTreeViewSelectListeners().add(new SSTreeViewSelectListener() {
            @Override
            public void selectedPathAdded(SSTreeView treeView, Sequence.Tree.Path path) {
                Logger.log.info("");
            }

            @Override
            public void selectedPathRemoved(SSTreeView treeView, Sequence.Tree.Path path) {
                Logger.log.info("");
            }

            @Override
            public void selectedPathsChanged(SSTreeView treeView, Sequence<Sequence.Tree.Path> previousSelectedPaths) {
//                Logger.log.info("previousSelectedPaths: {}", previousSelectedPaths.toString());
//                Logger.log.info("selectedPaths: {}", treeView.getSelectedPath().toString());
            }

            @Override
            public void selectedNodeChanged(SSTreeView treeView, Object previousSelectedNode) {
                SSTreeNode treeNode = (SSTreeNode) treeView.getSelectedNode();
                final int selectIndex = treeView.getTreeData().indexOf((SSTreeBranch) treeNode);

                Window window = treeView.getWindow();
                if (window != null) {
                    SSData data = (SSData) window.getAttribute(SSData.DEFAULT_TYPE);
                    data.setSelectFileIndex(selectIndex);

                    SSTabPane tabPane = treeNode.getTabPane();
                    SplitPane splitPaneMenu = (SplitPane) treeView.getParent().getParent().getParent().getParent();
                    splitPaneMenu.setRight(tabPane);
                }
            }
        });

        treeView.getComponentMouseButtonListeners().add(new ComponentMouseButtonListener() {
            @Override
            public boolean mouseDown(Component component, Mouse.Button button, int x, int y) {
                Logger.log.info("");
//                SSTreeView treeView = (SSTreeView) component;
//                SSTreeView menuTreeView = treeView.getMenuTreeView();
//                SSTreeNode treeNode = (SSTreeNode) treeView.getSelectedNode();
                return false;
            }

            @Override
            public boolean mouseUp(Component component, Mouse.Button button, int x, int y) {
                Logger.log.info("");
                return false;
            }

            @Override
            public boolean mouseClick(Component component, Mouse.Button button, int x, int y, int count) {
                SSTreeView treeView = (SSTreeView) component;

                int clickIndex = treeView.getTreeData().indexOf((SSTreeBranch) treeView.getSelectedNode());
                if (previousClickIndex != clickIndex) {
                    previousClickIndex = clickIndex;
                } else {
                    if (SSTreeView.Type.WindowMenu.equals(treeView.getTreeViewType()) && button == Mouse.Button.LEFT && count == 2) {
                        startEditMenu(treeView);
                    }
                }
                return false;
            }
        });

        treeView.getTreeViewNodeListeners().add(new SSTreeViewNodeListener() {
            @Override
            public void nodeInserted(SSTreeView treeView, Sequence.Tree.Path path, int index) {
            }

            @Override
            public void nodesRemoved(SSTreeView treeView, Sequence.Tree.Path path, int index, int count) {
            }

            @Override
            public void nodeUpdated(SSTreeView treeView, Sequence.Tree.Path path, int index) {
            }

            @Override
            public void nodesCleared(SSTreeView treeView, Sequence.Tree.Path path) {
            }

            @Override
            public void nodesSorted(SSTreeView treeView, Sequence.Tree.Path path) {
            }
        });

        return treeView;
    }

    public static Boolean startEditMenu(SSTreeView treeView) {
        treeView.setNodeEditor(new SSTreeViewNodeEditor());
        SSTreeView.NodeEditor nodeEditor = treeView.getNodeEditor();
        nodeEditor.beginEdit(treeView, treeView.getSelectedPath());

        return false;
    }

}
