/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener;

import jp.pegasus.pivot.ss.SSCardPane;
import org.apache.pivot.util.Vote;

public interface SSCardPaneListener {
    /**
     * Card pane listener adapter.
     */
    public static class Adapter implements SSCardPaneListener {
        @Override
        public Vote previewSelectIndexChange(SSCardPane cardPane, int selectIndex) {
            return Vote.APPROVE;
        }

        @Override
        public void selectIndexChangeVetoed(SSCardPane cardPane, Vote reason) {
        }

        @Override
        public void selectIndexChanged(SSCardPane cardPane, int previousSelectIndex) {
        }
    }

    /**
     * Called to preview a select index change.
     *
     * @param cardPane    The source of the event.
     * @param selectIndex The index that will be select.
     */
    public Vote previewSelectIndexChange(SSCardPane cardPane, int selectIndex);

    /**
     * Called when a select index change has been vetoed.
     *
     * @param cardPane The source of the event.
     * @param reason   The reason the event was vetoed.
     */
    public void selectIndexChangeVetoed(SSCardPane cardPane, Vote reason);

    /**
     * Called when a card pane's select index has changed.
     *
     * @param cardPane            The source of the event.
     * @param previousSelectIndex If the selection changed directly, contains the index that was previously
     *                            select. Otherwise, contains the current selection.
     */
    public void selectIndexChanged(SSCardPane cardPane, int previousSelectIndex);
}
