/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */
package jp.pegasus.pivot.listener.drawing;

import java.awt.*;

/**
 * Shape listener interface.
 */
public interface ShapeListener {
    /**
     * Called when a shape's origin has changed.
     *
     * @param shape
     * @param previousX
     * @param previousY
     */
    public void originChanged(jp.pegasus.pivot.media.drawing.Shape shape, int previousX, int previousY);

    /**
     * Called when a shape's stroke has changed.
     *
     * @param shape
     * @param previousStroke
     */
    public void strokeChanged(jp.pegasus.pivot.media.drawing.Shape shape, Paint previousStroke);

    /**
     * Called when a shape's stroke thickness has changed.
     *
     * @param shape
     * @param previousStrokeThickness
     */
    public void strokeThicknessChanged(jp.pegasus.pivot.media.drawing.Shape shape, int previousStrokeThickness);

    /**
     * Called when a shape's fill has changed.
     *
     * @param shape
     * @param previousFill
     */
    public void fillChanged(jp.pegasus.pivot.media.drawing.Shape shape, Paint previousFill);

    /**
     * Called when a shape's visible flag has changed.
     *
     * @param shape
     */
    public void visibleChanged(jp.pegasus.pivot.media.drawing.Shape shape);
}
