/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.listview;

import jp.pegasus.pivot.ss.SSListView;
import org.apache.pivot.wtk.BindType;

public interface SSListViewBindingListener {
    /**
     * List view binding listener adapter.
     */
    public static class Adapter implements SSListViewBindingListener {
        @Override
        public void listDataKeyChanged(SSListView listView, String previousListDataKey) {
        }

        @Override
        public void listDataBindTypeChanged(SSListView listView, BindType previousListDataBindType) {
        }

        @Override
        public void listDataBindMappingChanged(SSListView listView,
                                               SSListView.ListDataBindMapping previousListDataBindMapping) {
        }

        @Override
        public void selectedItemKeyChanged(SSListView listView, String previousSelectedItemKey) {
        }

        @Override
        public void selectedItemBindTypeChanged(SSListView listView, BindType previousSelectedItemBindType) {
        }

        @Override
        public void selectedItemBindMappingChanged(SSListView listView,
                                                   SSListView.ItemBindMapping previousSelectedItemBindMapping) {
        }

        @Override
        public void selectedItemsKeyChanged(SSListView listView, String previousSelectedItemsKey) {
        }

        @Override
        public void selectedItemsBindTypeChanged(SSListView listView, BindType previousSelectedItemsBindType) {
        }

        @Override
        public void selectedItemsBindMappingChanged(SSListView listView,
                                                    SSListView.ItemBindMapping previousSelectedItemsBindMapping) {
        }

        @Override
        public void checkedItemsKeyChanged(SSListView listView, String previousCheckedItemsKey) {
        }

        @Override
        public void checkedItemsBindTypeChanged(SSListView listView, BindType previousCheckedItemsBindType) {
        }

        @Override
        public void checkedItemsBindMappingChanged(SSListView listView,
                                                   SSListView.ItemBindMapping previousCheckedItemsBindMapping) {
        }
    }

    /**
     * Called when a list view's list data key has changed.
     *
     * @param listView
     * @param previousListDataKey
     */
    public void listDataKeyChanged(SSListView listView, String previousListDataKey);

    /**
     * Called when a list view's list data bind type has changed.
     *
     * @param listView
     * @param previousListDataBindType
     */
    public void listDataBindTypeChanged(SSListView listView, BindType previousListDataBindType);

    /**
     * Called when a list view's list data bind mapping has changed.
     *
     * @param listView
     * @param previousListDataBindMapping
     */
    public void listDataBindMappingChanged(SSListView listView, SSListView.ListDataBindMapping previousListDataBindMapping);

    /**
     * Called when a list view's selected item key has changed.
     *
     * @param listView
     * @param previousSelectedItemKey
     */
    public void selectedItemKeyChanged(SSListView listView, String previousSelectedItemKey);

    /**
     * Called when a list view's selected item bind type has changed.
     *
     * @param listView
     * @param previousSelectedItemBindType
     */
    public void selectedItemBindTypeChanged(SSListView listView, BindType previousSelectedItemBindType);

    /**
     * Called when a list view's selected item bind mapping has changed.
     *
     * @param listView
     * @param previousSelectedItemBindMapping
     *
     */
    public void selectedItemBindMappingChanged(SSListView listView,
                                               SSListView.ItemBindMapping previousSelectedItemBindMapping);

    /**
     * Called when a list view's selected items key has changed.
     *
     * @param listView
     * @param previousSelectedItemsKey
     */
    public void selectedItemsKeyChanged(SSListView listView, String previousSelectedItemsKey);

    /**
     * Called when a list view's selected items bind type has changed.
     *
     * @param listView
     * @param previousSelectedItemsBindType
     */
    public void selectedItemsBindTypeChanged(SSListView listView, BindType previousSelectedItemsBindType);

    /**
     * Called when a list view's selected items bind mapping has changed.
     *
     * @param listView
     * @param previousSelectedItemsBindMapping
     *
     */
    public void selectedItemsBindMappingChanged(SSListView listView,
                                                SSListView.ItemBindMapping previousSelectedItemsBindMapping);

    /**
     * Called when a list view's checked items key has changed.
     *
     * @param listView
     * @param previousCheckedItemsKey
     */
    public void checkedItemsKeyChanged(SSListView listView, String previousCheckedItemsKey);

    /**
     * Called when a list view's checked items bind type has changed.
     *
     * @param listView
     * @param previousCheckedItemsBindType
     */
    public void checkedItemsBindTypeChanged(SSListView listView, BindType previousCheckedItemsBindType);

    /**
     * Called when a list view's checked items bind mapping has changed.
     *
     * @param listView
     * @param previousCheckedItemsBindMapping
     *
     */
    public void checkedItemsBindMappingChanged(SSListView listView,
                                               SSListView.ItemBindMapping previousCheckedItemsBindMapping);
}
