/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.listview;

import jp.pegasus.pivot.ss.SSListView;

public interface SSListViewItemListener {
    /**
     * List view item listener adapter.
     */
    public static class Adapter implements SSListViewItemListener {
        @Override
        public void itemInserted(SSListView listView, int index) {
        }

        @Override
        public void itemsRemoved(SSListView listView, int index, int count) {
        }

        @Override
        public void itemUpdated(SSListView listView, int index) {
        }

        @Override
        public void itemsCleared(SSListView listView) {
        }

        @Override
        public void itemsSorted(SSListView listView) {
        }
    }

    /**
     * Called when an item has been inserted into the list view.
     *
     * @param listView The source of the event.
     * @param index    The index of the item that was inserted.
     */
    public void itemInserted(SSListView listView, int index);

    /**
     * Called when items have been removed from the list view.
     *
     * @param listView The source of the event.
     * @param index    The first index affected by the event.
     * @param count    The number of items that were removed, or <tt>-1</tt> if all items
     *                 were removed.
     */
    public void itemsRemoved(SSListView listView, int index, int count);

    /**
     * Called when an item in the list view has been updated.
     *
     * @param listView The source of the event.
     * @param index    The first index affected by the event.
     */
    public void itemUpdated(SSListView listView, int index);

    /**
     * Called when the items in a list view have been cleared.
     *
     * @param listView
     */
    public void itemsCleared(SSListView listView);

    /**
     * Called when the items in a list view have been sorted.
     *
     * @param listView The source of the event.
     */
    public void itemsSorted(SSListView listView);
}
