/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.listview;

import jp.pegasus.pivot.ss.SSListView;
import org.apache.pivot.collections.List;
import org.apache.pivot.util.Filter;

public interface SSListViewListener {
    /**
     * List view listener adapter.
     */
    public static class Adapter implements SSListViewListener {
        @Override
        public void listDataChanged(SSListView listView, List<?> previousListData) {
        }

        @Override
        public void itemRendererChanged(SSListView listView, SSListView.ItemRenderer previousItemRenderer) {
        }

        @Override
        public void itemEditorChanged(SSListView listView, SSListView.ItemEditor previousItemEditor) {
        }

        @Override
        public void selectModeChanged(SSListView listView, SSListView.SelectMode previousSelectMode) {
        }

        @Override
        public void checkmarksEnabledChanged(SSListView listView) {
        }

        @Override
        public void disabledItemFilterChanged(SSListView listView,
                                              Filter<?> previousDisabledItemFilter) {
        }

        @Override
        public void disabledCheckmarkFilterChanged(SSListView listView,
                                                   Filter<?> previousDisabledCheckmarkFilter) {
        }
    }

    /**
     * Called when a list view's list data has changed.
     *
     * @param listView
     * @param previousListData
     */
    public void listDataChanged(SSListView listView, List<?> previousListData);

    /**
     * Called when a list view's item renderer has changed.
     *
     * @param listView
     * @param previousItemRenderer
     */
    public void itemRendererChanged(SSListView listView, SSListView.ItemRenderer previousItemRenderer);

    /**
     * Called when a list view's item editor has changed.
     *
     * @param listView
     * @param previousItemEditor
     */
    public void itemEditorChanged(SSListView listView, SSListView.ItemEditor previousItemEditor);

    /**
     * Called when a list view's select mode has changed.
     *
     * @param listView
     * @param previousSelectMode
     */
    public void selectModeChanged(SSListView listView, SSListView.SelectMode previousSelectMode);

    /**
     * Called when a list view's checkmarks enabled flag has changed.
     *
     * @param listView
     */
    public void checkmarksEnabledChanged(SSListView listView);

    /**
     * Called when a list view's disabled item filter has changed.
     *
     * @param listView
     * @param previousDisabledItemFilter
     */
    public void disabledItemFilterChanged(SSListView listView, Filter<?> previousDisabledItemFilter);

    /**
     * Called when a list view's disabled checkmark filter has changed.
     *
     * @param listView
     * @param previousDisabledCheckmarkFilter
     *
     */
    public void disabledCheckmarkFilterChanged(SSListView listView,
                                               Filter<?> previousDisabledCheckmarkFilter);
}
