/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.listview;

import jp.pegasus.pivot.ss.SSListView;
import org.apache.pivot.collections.Sequence;
import org.apache.pivot.wtk.Span;

public interface SSListViewSelectionListener {
    /**
     * List view selection listener adapter.
     */
    public static class Adapter implements SSListViewSelectionListener {
        @Override
        public void selectedRangeAdded(SSListView listView, int rangeStart, int rangeEnd) {
        }

        @Override
        public void selectedRangeRemoved(SSListView listView, int rangeStart, int rangeEnd) {
        }

        @Override
        public void selectedRangesChanged(SSListView listView, Sequence<Span> previousSelectedRanges) {
        }

        @Override
        public void selectedItemChanged(SSListView listView, Object previousSelectedItem) {
        }
    }

    /**
     * Called when a range has been added to a list view's selection.
     *
     * @param listView   The source of the event.
     * @param rangeStart The start index of the range that was added, inclusive.
     * @param rangeEnd   The end index of the range that was added, inclusive.
     */
    public void selectedRangeAdded(SSListView listView, int rangeStart, int rangeEnd);

    /**
     * Called when a range has been removed from a list view's selection.
     *
     * @param listView   The source of the event.
     * @param rangeStart The starting index of the range that was removed, inclusive.
     * @param rangeEnd   The starting index of the range that was removed, inclusive.
     */
    public void selectedRangeRemoved(SSListView listView, int rangeStart, int rangeEnd);

    /**
     * Called when a list view's selection state has changed.
     *
     * @param listView               The source of the event.
     * @param previousSelectedRanges If the selection changed directly, contains the ranges that were previously
     *                               selected. If the selection changed indirectly as a result of a model change,
     *                               contains the current selection. Otherwise, contains <tt>null</tt>.
     */
    public void selectedRangesChanged(SSListView listView, Sequence<Span> previousSelectedRanges);

    /**
     * Called when a list view's selected item has changed.
     *
     * @param listView             The source of the event.
     * @param previousSelectedItem The item that was previously selected.
     */
    public void selectedItemChanged(SSListView listView, Object previousSelectedItem);
}
