/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.tableview;

import jp.pegasus.pivot.ss.tableview.SSTableView;
import org.apache.pivot.wtk.BindType;

public interface SSTableViewBindingListener {
    /**
     * Table view binding listener adapter.
     */
    public static class Adapter implements SSTableViewBindingListener {
        @Override
        public void tableDataKeyChanged(SSTableView tableView, String previousTableDataKey) {
        }

        @Override
        public void tableDataBindTypeChanged(SSTableView tableView, BindType previousTableDataBindType) {
        }

        @Override
        public void tableDataBindMappingChanged(SSTableView tableView,
                                                SSTableView.TableDataBindMapping previousTableDataBindMapping) {
        }

        @Override
        public void selectRowKeyChanged(SSTableView tableView, String previousSelectRowKey) {
        }

        @Override
        public void selectRowBindTypeChanged(SSTableView tableView, BindType previousSelectRowBindType) {
        }

        @Override
        public void selectRowBindMappingChanged(SSTableView tableView,
                                                SSTableView.SelectRowBindMapping previousSelectRowBindMapping) {
        }

        @Override
        public void selectRowsKeyChanged(SSTableView tableView, String previousSelectRowsKey) {
        }

        @Override
        public void selectRowsBindTypeChanged(SSTableView tableView, BindType previousSelectRowsBindType) {
        }

        @Override
        public void selectRowsBindMappingChanged(SSTableView tableView,
                                                 SSTableView.SelectRowBindMapping previousSelectRowsBindMapping) {
        }
    }

    /**
     * Called when a table view's table data key has changed.
     *
     * @param tableView
     * @param previousTableDataKey
     */
    public void tableDataKeyChanged(SSTableView tableView, String previousTableDataKey);

    /**
     * Called when a table view's table data bind type has changed.
     *
     * @param tableView
     * @param previousTableDataBindType
     */
    public void tableDataBindTypeChanged(SSTableView tableView, BindType previousTableDataBindType);

    /**
     * Called when a table view's table data bind mapping has changed.
     *
     * @param tableView
     * @param previousTableDataBindMapping
     */
    public void tableDataBindMappingChanged(SSTableView tableView, SSTableView.TableDataBindMapping previousTableDataBindMapping);

    /**
     * Called when a table view's select row key has changed.
     *
     * @param tableView
     * @param previousSelectRowKey
     */
    public void selectRowKeyChanged(SSTableView tableView, String previousSelectRowKey);

    /**
     * Called when a table view's select row bind type has changed.
     *
     * @param tableView
     * @param previousSelectRowBindType
     */
    public void selectRowBindTypeChanged(SSTableView tableView, BindType previousSelectRowBindType);

    /**
     * Called when a table view's select row bind mapping has changed.
     *
     * @param tableView
     * @param previousSelectRowBindMapping
     */
    public void selectRowBindMappingChanged(SSTableView tableView,
                                            SSTableView.SelectRowBindMapping previousSelectRowBindMapping);

    /**
     * Called when a table view's select rows key has changed.
     *
     * @param tableView
     * @param previousSelectRowsKey
     */
    public void selectRowsKeyChanged(SSTableView tableView, String previousSelectRowsKey);

    /**
     * Called when a table view's select rows bind type has changed.
     *
     * @param tableView
     * @param previousSelectRowsBindType
     */
    public void selectRowsBindTypeChanged(SSTableView tableView, BindType previousSelectRowsBindType);

    /**
     * Called when a table view's select rows bind mapping has changed.
     *
     * @param tableView
     * @param previousSelectRowsBindMapping
     */
    public void selectRowsBindMappingChanged(SSTableView tableView,
                                             SSTableView.SelectRowBindMapping previousSelectRowsBindMapping);
}
