/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.tableview;

import jp.pegasus.pivot.ss.tableview.SSTableView;
import org.apache.pivot.collections.Sequence;

public interface SSTableViewColumnListener {
    /**
     * Table view column listener adapter.
     */
    public static class Adapter implements SSTableViewColumnListener {
        @Override
        public void columnInserted(SSTableView SSTableView, int index) {
        }

        @Override
        public void columnsRemoved(SSTableView SSTableView, int index,
                                   Sequence<SSTableView.Column> columns) {
        }

        @Override
        public void columnNameChanged(SSTableView.Column column, String previousName) {
        }

        @Override
        public void columnHeaderDataChanged(SSTableView.Column column, Object previousHeaderData) {
        }

        @Override
        public void columnHeaderDataRendererChanged(SSTableView.Column column,
                                                    SSTableView.HeaderDataRenderer previousColumnHeaderDataRenderer) {
        }

        @Override
        public void columnWidthChanged(SSTableView.Column column, int previousWidth,
                                       boolean previousRelative) {
        }

        @Override
        public void columnWidthLimitsChanged(SSTableView.Column column, int previousMinimumWidth,
                                             int previousMaximumWidth) {
        }

        @Override
        public void columnFilterChanged(SSTableView.Column column, Object previousFilter) {
        }

        @Override
        public void columnCellRendererChanged(SSTableView.Column column,
                                              SSTableView.CellRenderer previousCellRenderer) {
        }
    }

    /**
     * Called when a column is inserted into a table view's column sequence.
     *
     * @param SSTableView
     * @param index
     */
    public void columnInserted(SSTableView SSTableView, int index);

    /**
     * Called when columns are removed from a table view's column sequence.
     *
     * @param SSTableView
     * @param index
     * @param columns
     */
    public void columnsRemoved(SSTableView SSTableView, int index, Sequence<SSTableView.Column> columns);

    /**
     * Called when a column's name has changed.
     *
     * @param column
     * @param previousName
     */
    public void columnNameChanged(SSTableView.Column column, String previousName);

    /**
     * Called when a column's header data has changed.
     *
     * @param column
     * @param previousHeaderData
     */
    public void columnHeaderDataChanged(SSTableView.Column column, Object previousHeaderData);

    /**
     * Called when a column's header data renderer has changed.
     *
     * @param column
     * @param previousHeaderDataRenderer
     */
    public void columnHeaderDataRendererChanged(SSTableView.Column column,
                                                SSTableView.HeaderDataRenderer previousHeaderDataRenderer);

    /**
     * Called when a column's width has changed.
     *
     * @param column
     * @param previousWidth
     * @param previousRelative
     */
    public void columnWidthChanged(SSTableView.Column column, int previousWidth,
                                   boolean previousRelative);

    /**
     * Called when a column's width limits have changed.
     *
     * @param column
     * @param previousMinimumWidth
     * @param previousMaximumWidth
     */
    public void columnWidthLimitsChanged(SSTableView.Column column, int previousMinimumWidth,
                                         int previousMaximumWidth);

    /**
     * Called when a column's filter has changed.
     *
     * @param column
     * @param previousFilter
     */
    public void columnFilterChanged(SSTableView.Column column, Object previousFilter);

    /**
     * Called when a column's cell renderer has changed.
     *
     * @param column
     * @param previousCellRenderer
     */
    public void columnCellRendererChanged(SSTableView.Column column,
                                          SSTableView.CellRenderer previousCellRenderer);
}
