/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.tableview;

import jp.pegasus.pivot.ss.tableview.SSTableView;
import org.apache.pivot.collections.List;
import org.apache.pivot.util.Filter;

public interface SSTableViewListener {
    /**
     * Table view listener adapter.
     */
    public static class Adapter implements SSTableViewListener {
        @Override
        public void tableDataChanged(SSTableView tableView, List<?> previousTableData) {
        }

        @Override
        public void columnSourceChanged(SSTableView tableView, SSTableView previousColumnSource) {
        }

        @Override
        public void rowEditorChanged(SSTableView tableView, SSTableView.RowEditor previousRowEditor) {
        }

        @Override
        public void selectModeChanged(SSTableView tableView, SSTableView.SelectMode previousSelectMode) {
        }

        @Override
        public void disabledRowFilterChanged(SSTableView tableView, Filter<?> previousDisabledRowFilter) {
        }
    }

    /**
     * Called when a table view's table data has changed.
     *
     * @param tableView
     * @param previousTableData
     */
    public void tableDataChanged(SSTableView tableView, List<?> previousTableData);

    /**
     * Called when a table view's column source has changed.
     *
     * @param tableView
     * @param previousColumnSource
     */
    public void columnSourceChanged(SSTableView tableView, SSTableView previousColumnSource);

    /**
     * Called when a table view's row editor has changed.
     *
     * @param tableView
     * @param previousRowEditor
     */
    public void rowEditorChanged(SSTableView tableView, SSTableView.RowEditor previousRowEditor);

    /**
     * Called when a table view's select mode has changed.
     *
     * @param tableView
     * @param previousSelectMode
     */
    public void selectModeChanged(SSTableView tableView, SSTableView.SelectMode previousSelectMode);

    /**
     * Called when a table view's disabled row filter has changed.
     *
     * @param tableView
     * @param previousDisabledRowFilter
     */
    public void disabledRowFilterChanged(SSTableView tableView, Filter<?> previousDisabledRowFilter);
}
