/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.tableview;

import jp.pegasus.pivot.ss.tableview.SSTableView;

public interface SSTableViewRowListener {
    /**
     * Table row listener adapter.
     */
    public static class Adapter implements SSTableViewRowListener {
        @Override
        public void rowInserted(SSTableView tableView, int index) {
        }

        @Override
        public void rowsRemoved(SSTableView tableView, int index, int count) {
        }

        @Override
        public void rowUpdated(SSTableView tableView, int index) {
        }

        @Override
        public void rowsCleared(SSTableView tableView) {
        }

        @Override
        public void rowsSorted(SSTableView tableView) {
        }
    }

    /**
     * Called when a row has been inserted into the table view.
     *
     * @param tableView The source of the event.
     * @param index     The index of the row that was inserted.
     */
    public void rowInserted(SSTableView tableView, int index);

    /**
     * Called when rows have been removed from the table view.
     *
     * @param tableView The source of the event.
     * @param index     The first index affected by the event.
     * @param count     The number of rows that were removed, or <tt>-1</tt> if all rows
     *                  were removed.
     */
    public void rowsRemoved(SSTableView tableView, int index, int count);

    /**
     * Called when an row in the table view has been updated.
     *
     * @param tableView The source of the event.
     * @param index     The first index affected by the event.
     */
    public void rowUpdated(SSTableView tableView, int index);

    /**
     * Called when the rows in a table view have been cleared.
     *
     * @param tableView
     */
    public void rowsCleared(SSTableView tableView);

    /**
     * Called when the rows in a table have been sorted.
     *
     * @param tableView The source of the event.
     */
    public void rowsSorted(SSTableView tableView);
}
