/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.tableview;

import jp.pegasus.pivot.ss.tableview.SSTableView;
import org.apache.pivot.collections.Sequence;
import org.apache.pivot.wtk.Span;

public interface SSTableViewSelectListener {
    /**
     * Table view selection listener adapter.
     */
    public static class Adapter implements SSTableViewSelectListener {
        @Override
        public void selectRangeAdded(SSTableView tableView, int rangeStart, int rangeEnd) {
        }

        @Override
        public void selectRangeRemoved(SSTableView tableView, int rangeStart, int rangeEnd) {
        }

        @Override
        public void selectRangesChanged(SSTableView tableView, Sequence<Span> previousSelectRanges) {
        }

        @Override
        public void selectRowChanged(SSTableView tableView, Object previousSelectRow) {
        }
    }

    /**
     * Called when a range has been added to a table view's selection.
     *
     * @param tableView  The source of the event.
     * @param rangeStart The start index of the range that was added, inclusive.
     * @param rangeEnd   The end index of the range that was added, inclusive.
     */
    public void selectRangeAdded(SSTableView tableView, int rangeStart, int rangeEnd);

    /**
     * Called when a range has been removed from a table view's selection.
     *
     * @param tableView  The source of the event.
     * @param rangeStart The start index of the range that was removed, inclusive.
     * @param rangeEnd   The end index of the range that was removed, inclusive.
     */
    public void selectRangeRemoved(SSTableView tableView, int rangeStart, int rangeEnd);

    /**
     * Called when a table view's selection state has been reset.
     *
     * @param tableView            The source of the event.
     * @param previousSelectRanges If the selection changed directly, contains the ranges that were previously
     *                             select. If the selection changed indirectly as a result of a model change,
     *                             contains the current selection. Otherwise, contains <tt>null</tt>.
     */
    public void selectRangesChanged(SSTableView tableView, Sequence<Span> previousSelectRanges);

    /**
     * Called when a table view's select item has changed.
     *
     * @param tableView         The source of the event.
     * @param previousSelectRow The row that was previously select.
     */
    public void selectRowChanged(SSTableView tableView, Object previousSelectRow);
}
