/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.tableview;

import jp.pegasus.pivot.ss.tableview.SSTableView;
import org.apache.pivot.wtk.SortDirection;

public interface SSTableViewSortListener {
    /**
     * Table view sort listener adapter.
     */
    public static class Adapter implements SSTableViewSortListener {
        public void sortAdded(SSTableView tableView, String columnName) {
        }

        public void sortUpdated(SSTableView tableView, String columnName,
                                SortDirection previousSortDirection) {
        }

        public void sortRemoved(SSTableView tableView, String columnName,
                                SortDirection sortDirection) {
        }

        public void sortChanged(SSTableView tableView) {
        }
    }

    /**
     * Called when a sort has been added to a table view.
     *
     * @param tableView
     * @param columnName
     */
    public void sortAdded(SSTableView tableView, String columnName);

    /**
     * Called when a sort has been updated in a table view.
     *
     * @param tableView
     * @param columnName
     * @param previousSortDirection
     */
    public void sortUpdated(SSTableView tableView, String columnName,
                            SortDirection previousSortDirection);

    /**
     * Called when a sort has been removed from a table view.
     *
     * @param tableView
     * @param columnName
     * @param sortDirection
     */
    public void sortRemoved(SSTableView tableView, String columnName,
                            SortDirection sortDirection);

    /**
     * Called when a table view's sort has changed.
     *
     * @param tableView
     */
    public void sortChanged(SSTableView tableView);
}
