/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.tabpane;

import jp.pegasus.pivot.ss.SSTabPane;
import org.apache.pivot.collections.Sequence;
import org.apache.pivot.util.Vote;
import org.apache.pivot.wtk.Button;
import org.apache.pivot.wtk.Component;

public interface SSTabPaneListener {
    /**
     * Tab pane listener adapter.
     */
    public static class Adapter implements SSTabPaneListener {
        @Override
        public void tabInserted(SSTabPane tabPane, int index) {
        }

        @Override
        public Vote previewRemoveTabs(SSTabPane tabPane, int index, int count) {
            return Vote.APPROVE;
        }

        @Override
        public void removeTabsVetoed(SSTabPane tabPane, Vote vote) {
        }

        @Override
        public void tabsRemoved(SSTabPane tabPane, int index, Sequence<Component> tabs) {
        }

        @Override
        public void cornerChanged(SSTabPane tabPane, Component previousCorner) {
        }

        @Override
        public void tabDataRendererChanged(SSTabPane tabPane, Button.DataRenderer previousTabDataRenderer) {
        }

        @Override
        public void closeableChanged(SSTabPane tabPane) {
        }

        @Override
        public void collapsibleChanged(SSTabPane tabPane) {
        }
    }

    /**
     * Called when a tab has been inserted into a tab pane's tab sequence.
     *
     * @param tabPane
     * @param index
     */
    public void tabInserted(SSTabPane tabPane, int index);

    /**
     * Called to preview a tab removal.
     *
     * @param tabPane
     * @param index
     * @param count
     */
    public Vote previewRemoveTabs(SSTabPane tabPane, int index, int count);

    /**
     * Called when a tab removal has been vetoed.
     *
     * @param tabPane
     * @param reason
     */
    public void removeTabsVetoed(SSTabPane tabPane, Vote reason);

    /**
     * Called when a tab has been removed from a tab pane's tab sequence.
     *
     * @param tabPane
     * @param index
     * @param tabs
     */
    public void tabsRemoved(SSTabPane tabPane, int index, Sequence<Component> tabs);

    /**
     * Called when a tab pane's corner component has changed.
     *
     * @param tabPane
     * @param previousCorner
     */
    public void cornerChanged(SSTabPane tabPane, Component previousCorner);

    /**
     * Called when a tab pane's tab data renderer has changed.
     *
     * @param tabPane
     * @param previousTabDataRenderer
     */
    public void tabDataRendererChanged(SSTabPane tabPane, Button.DataRenderer previousTabDataRenderer);

    /**
     * Called when a tab pane's closeable property has changed.
     *
     * @param tabPane
     */
    public void closeableChanged(SSTabPane tabPane);

    /**
     * Called when a tab pane's collapsible property has changed.
     *
     * @param tabPane
     */
    public void collapsibleChanged(SSTabPane tabPane);
}
