/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.tabpane;

import jp.pegasus.pivot.ss.SSTabPane;
import org.apache.pivot.util.Vote;

public interface SSTabPaneSelectionListener {
    /**
     * Tab pane selection listener adapter.
     */
    public static class Adapter implements SSTabPaneSelectionListener {
        @Override
        public Vote previewSelectedIndexChange(SSTabPane tabPane, int selectedIndex) {
            return Vote.APPROVE;
        }

        @Override
        public void selectedIndexChangeVetoed(SSTabPane tabPane, Vote reason) {
        }

        @Override
        public void selectedIndexChanged(SSTabPane tabPane, int previousSelectedIndex) {
        }
    }

    /**
     * Called to preview a selected index change.
     *
     * @param tabPane       The source of the event.
     * @param selectedIndex The index that will be selected.
     */
    public Vote previewSelectedIndexChange(SSTabPane tabPane, int selectedIndex);

    /**
     * Called when a selected index change has been vetoed.
     *
     * @param tabPane The source of the event.
     * @param reason  The reason the event was vetoed.
     */
    public void selectedIndexChangeVetoed(SSTabPane tabPane, Vote reason);

    /**
     * Called when a tab pane's selected index has changed.
     *
     * @param tabPane               The source of the event.
     * @param previousSelectedIndex If the selection changed directly, contains the index that was previously
     *                              selected. Otherwise, contains the current selection.
     */
    public void selectedIndexChanged(SSTabPane tabPane, int previousSelectedIndex);
}
