/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.textarea;

import jp.pegasus.pivot.ss.SSTextArea;
import org.apache.pivot.collections.Sequence;

public interface SSTextAreaContentListener {
    /**
     * Text input text listener adapter.
     */
    public static class Adapter implements SSTextAreaContentListener {
        @Override
        public void paragraphInserted(SSTextArea ssTextArea, int index) {
        }

        @Override
        public void paragraphsRemoved(SSTextArea ssTextArea, int index, Sequence<SSTextArea.Paragraph> removed) {
        }

        @Override
        public void textChanged(SSTextArea ssTextArea) {
        }
    }

    /**
     * Called when a paragraph has been inserted into a text area's paragraph
     * sequence.
     *
     * @param ssTextArea The source of the event.
     * @param index      The index at which the paragraph was inserted.
     */
    public void paragraphInserted(SSTextArea ssTextArea, int index);

    /**
     * Called when paragraphs have been removed from a text area's paragraph
     * sequence.
     *
     * @param ssTextArea The source of the event.
     * @param index      The starting index from which the paragraphs were removed.
     * @param removed    The paragraphs that were removed.
     */
    public void paragraphsRemoved(SSTextArea ssTextArea, int index, Sequence<SSTextArea.Paragraph> removed);

    /**
     * Called when a text area's text has changed.
     *
     * @param ssTextArea The source of the event.
     */
    public void textChanged(SSTextArea ssTextArea);

}
