/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.textinput;

import jp.pegasus.pivot.ss.SSTextInput;
import org.apache.pivot.util.Vote;

public interface SSTextInputContentListener {
    /**
     * Text input text listener adapter.
     */
    public static class Adapter implements SSTextInputContentListener {
        @Override
        public Vote previewInsertText(SSTextInput textInput, CharSequence text, int index) {
            return Vote.APPROVE;
        }

        @Override
        public void insertTextVetoed(SSTextInput textInput, Vote reason) {
        }

        @Override
        public void textInserted(SSTextInput textInput, int index, int count) {
        }

        @Override
        public Vote previewRemoveText(SSTextInput textInput, int index, int count) {
            return Vote.APPROVE;
        }

        @Override
        public void removeTextVetoed(SSTextInput textInput, Vote reason) {
        }

        @Override
        public void textRemoved(SSTextInput textInput, int index, int count) {
        }

        @Override
        public void textChanged(SSTextInput textInput) {
        }
    }

    /**
     * Called to preview a text insertion.
     *
     * @param textInput The source of the event.
     * @param text      The text that will be inserted.
     * @param index     The index at which the text will be inserted.
     */
    public Vote previewInsertText(SSTextInput textInput, CharSequence text, int index);

    /**
     * Called when a text insertion has been vetoed.
     *
     * @param textInput The source of the event.
     * @param reason    The reason the event was vetoed.
     */
    public void insertTextVetoed(SSTextInput textInput, Vote reason);

    /**
     * Called when text has been inserted into a text input.
     *
     * @param textInput The source of the event.
     * @param index     The index at which the text was inserted.
     * @param count     The number of characters that were inserted.
     */
    public void textInserted(SSTextInput textInput, int index, int count);

    /**
     * Called to preview a text removal.
     *
     * @param textInput The source of the event.
     * @param index     The starting index from which the text will be removed.
     */
    public Vote previewRemoveText(SSTextInput textInput, int index, int count);

    /**
     * Called when a text removal has been vetoed.
     *
     * @param textInput The source of the event.
     * @param reason    The reason the event was vetoed.
     */
    public void removeTextVetoed(SSTextInput textInput, Vote reason);

    /**
     * Called when text has been removed from a text input.
     *
     * @param textInput The source of the event.
     * @param index     The index from which the text was removed.
     * @param count     The number of characters that were removed.
     */
    public void textRemoved(SSTextInput textInput, int index, int count);

    /**
     * Called when a text input's text has changed.
     *
     * @param textInput The source of the event.
     */
    public void textChanged(SSTextInput textInput);
}
