/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.textinput;

import jp.pegasus.pivot.ss.SSTextInput;
import org.apache.pivot.wtk.validation.Validator;

public interface SSTextInputListener {
    /**
     * Text input listener adapter.
     */
    public static class Adapter implements SSTextInputListener {
        @Override
        public void textSizeChanged(SSTextInput textInput, int previousTextSize) {
        }

        @Override
        public void maximumLengthChanged(SSTextInput textInput, int previousMaximumLength) {
        }

        @Override
        public void passwordChanged(SSTextInput textInput) {
        }

        @Override
        public void promptChanged(SSTextInput textInput, String previousPrompt) {
        }

        @Override
        public void textValidatorChanged(SSTextInput textInput, Validator previousValidator) {
        }

        @Override
        public void strictValidationChanged(SSTextInput textInput) {
        }

        @Override
        public void textValidChanged(SSTextInput textInput) {
        }
    }

    /**
     * Called when a text input's text size has changed.
     *
     * @param textInput
     * @param previousTextSize
     */
    public void textSizeChanged(SSTextInput textInput, int previousTextSize);

    /**
     * Called when a text input's maximum length has changed.
     *
     * @param textInput
     * @param previousMaximumLength
     */
    public void maximumLengthChanged(SSTextInput textInput, int previousMaximumLength);

    /**
     * Called when a text input's password flag has changed.
     *
     * @param textInput
     */
    public void passwordChanged(SSTextInput textInput);

    /**
     * Called when a text input's prompt has changed.
     *
     * @param textInput
     * @param previousPrompt
     */
    public void promptChanged(SSTextInput textInput, String previousPrompt);

    /**
     * Called when the validator changes.
     *
     * @param textInput
     * @param previousValidator
     */
    public void textValidatorChanged(SSTextInput textInput, Validator previousValidator);

    /**
     * Called when a text input's strict validation flag has changed.
     *
     * @param textInput
     */
    public void strictValidationChanged(SSTextInput textInput);

    /**
     * Called when the text changes validity.
     *
     * @param textInput
     */
    public void textValidChanged(SSTextInput textInput);
}
