/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.treeview;

import jp.pegasus.pivot.ss.SSTreeView;
import org.apache.pivot.collections.List;
import org.apache.pivot.util.Filter;

public interface SSTreeViewListener {
    /**
     * Tree view listener adapter.
     */
    public static class Adapter implements SSTreeViewListener {
        @Override
        public void treeDataChanged(SSTreeView treeView, List<?> previousTreeData) {
        }

        @Override
        public void nodeRendererChanged(SSTreeView treeView,
                                        SSTreeView.NodeRenderer previousNodeRenderer) {
        }

        @Override
        public void nodeEditorChanged(SSTreeView treeView, SSTreeView.NodeEditor previousNodeEditor) {
        }

        @Override
        public void selectModeChanged(SSTreeView treeView, SSTreeView.SelectMode previousSelectMode) {
        }

        @Override
        public void checkmarksEnabledChanged(SSTreeView treeView) {
        }

        @Override
        public void showMixedCheckmarkStateChanged(SSTreeView treeView) {
        }

        @Override
        public void disabledNodeFilterChanged(SSTreeView treeView,
                                              Filter<?> previousDisabledNodeFilter) {
        }

        @Override
        public void disabledCheckmarkFilterChanged(SSTreeView treeView,
                                                   Filter<?> previousDisabledCheckmarkFilter) {
        }
    }

    /**
     * Called when a tree view's data has changed.
     *
     * @param treeView
     * @param previousTreeData
     */
    public void treeDataChanged(SSTreeView treeView, List<?> previousTreeData);

    /**
     * Called when a tree view's node renderer has changed.
     *
     * @param treeView
     * @param previousNodeRenderer
     */
    public void nodeRendererChanged(SSTreeView treeView, SSTreeView.NodeRenderer previousNodeRenderer);

    /**
     * Called when a tree view's node editor has changed.
     *
     * @param treeView
     * @param previousNodeEditor
     */
    public void nodeEditorChanged(SSTreeView treeView, SSTreeView.NodeEditor previousNodeEditor);

    /**
     * Called when a tree view's select mode has changed.
     *
     * @param treeView
     * @param previousSelectMode
     */
    public void selectModeChanged(SSTreeView treeView, SSTreeView.SelectMode previousSelectMode);

    /**
     * Called when a tree view's checkmarks enabled flag has changed.
     *
     * @param treeView
     */
    public void checkmarksEnabledChanged(SSTreeView treeView);

    /**
     * Called when a tree view's "show mixed checkmark state" flag has changed.
     *
     * @param treeView
     */
    public void showMixedCheckmarkStateChanged(SSTreeView treeView);

    /**
     * Called when a tree view's disabled node filter has changed.
     *
     * @param treeView
     * @param previousDisabledNodeFilter
     */
    public void disabledNodeFilterChanged(SSTreeView treeView, Filter<?> previousDisabledNodeFilter);

    /**
     * Called when a tree view's disabled checkmark filter has changed.
     *
     * @param treeView
     * @param previousDisabledCheckmarkFilter
     *
     */
    public void disabledCheckmarkFilterChanged(SSTreeView treeView,
                                               Filter<?> previousDisabledCheckmarkFilter);
}
