/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.treeview;

import jp.pegasus.pivot.ss.SSTreeView;
import org.apache.pivot.collections.Sequence;

public interface SSTreeViewNodeListener {
    /**
     * Tree view node listener adapter.
     */
    public static class Adapter implements SSTreeViewNodeListener {
        @Override
        public void nodeInserted(SSTreeView treeView, Sequence.Tree.Path path, int index) {
        }

        @Override
        public void nodesRemoved(SSTreeView treeView, Sequence.Tree.Path path, int index, int count) {
        }

        @Override
        public void nodeUpdated(SSTreeView treeView, Sequence.Tree.Path path, int index) {
        }

        @Override
        public void nodesCleared(SSTreeView treeView, Sequence.Tree.Path path) {
        }

        @Override
        public void nodesSorted(SSTreeView treeView, Sequence.Tree.Path path) {
        }
    }

    /**
     * Called when a node has been inserted into the tree view.
     *
     * @param treeView
     * @param path
     * @param index
     */
    public void nodeInserted(SSTreeView treeView, Sequence.Tree.Path path, int index);

    /**
     * Called when nodes have been removed from the tree view.
     *
     * @param treeView
     * @param path
     * @param index
     * @param count    The number of nodes that were removed, or <tt>-1</tt> if all nodes
     *                 were removed.
     */
    public void nodesRemoved(SSTreeView treeView, Sequence.Tree.Path path, int index, int count);

    /**
     * Called when a node in the tree view has been updated.
     *
     * @param treeView
     * @param path
     * @param index
     */
    public void nodeUpdated(SSTreeView treeView, Sequence.Tree.Path path, int index);

    /**
     * Called when the nodes in a branch have been cleared.
     *
     * @param treeView
     */
    public void nodesCleared(SSTreeView treeView, Sequence.Tree.Path path);

    /**
     * Called when the nodes in a branch have been sorted.
     *
     * @param treeView
     * @param path
     */
    public void nodesSorted(SSTreeView treeView, Sequence.Tree.Path path);
}
