/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.treeview;

import jp.pegasus.pivot.ss.SSTreeView;
import org.apache.pivot.collections.Sequence;

public interface SSTreeViewSelectListener {
    /**
     * Tree view selection listener adapter.
     */
    public static class Adapter implements SSTreeViewSelectListener {
        @Override
        public void selectedPathAdded(SSTreeView treeView, Sequence.Tree.Path path) {
        }

        @Override
        public void selectedPathRemoved(SSTreeView treeView, Sequence.Tree.Path path) {
        }

        @Override
        public void selectedPathsChanged(SSTreeView treeView, Sequence<Sequence.Tree.Path> previousSelectedPaths) {
        }

        @Override
        public void selectedNodeChanged(SSTreeView treeView, Object previousSelectedNode) {
        }
    }

    /**
     * Called when a selected path has been added to a tree view.
     *
     * @param treeView
     * @param path
     */
    public void selectedPathAdded(SSTreeView treeView, Sequence.Tree.Path path);

    /**
     * Called when a selected path has been removed from a tree view.
     *
     * @param treeView
     * @param path
     */
    public void selectedPathRemoved(SSTreeView treeView, Sequence.Tree.Path path);

    /**
     * Called when a tree view's selection state has been reset.
     *
     * @param treeView
     * @param previousSelectedPaths
     */
    public void selectedPathsChanged(SSTreeView treeView, Sequence<Sequence.Tree.Path> previousSelectedPaths);

    /**
     * Called when a tree view's selected node has changed.
     *
     * @param treeView
     * @param previousSelectedNode
     */
    public void selectedNodeChanged(SSTreeView treeView, Object previousSelectedNode);
}
