/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.window;

import org.apache.pivot.collections.Sequence;
import org.apache.pivot.wtk.Action;
import org.apache.pivot.wtk.Keyboard;
import org.apache.pivot.wtk.Window;

public interface SSWindowActionMappingListener {
    /**
     * Called when an action mapping has been added to a window.
     *
     * @param window
     */
    public void actionMappingAdded(Window window);

    /**
     * Called when action mappings have been removed from a window.
     *
     * @param window
     * @param index
     * @param removed
     */
    public void actionMappingsRemoved(Window window, int index, Sequence<Window.ActionMapping> removed);

    /**
     * Called when an action mapping's keystroke has changed.
     *
     * @param actionMapping
     * @param previousKeyStroke
     */
    public void keyStrokeChanged(Window.ActionMapping actionMapping, Keyboard.KeyStroke previousKeyStroke);

    /**
     * Called when an action mapping's action has changed.
     *
     * @param actionMapping
     * @param previousAction
     */
    public void actionChanged(Window.ActionMapping actionMapping, Action previousAction);
}
