/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.window;

import jp.pegasus.pivot.ss.SSWindow;
import org.apache.pivot.collections.Sequence;
import org.apache.pivot.wtk.Component;
import org.apache.pivot.wtk.media.Image;

public interface SSWindowListener {
    /**
     * SSWindow listener adapter.
     */
    public static class Adapter implements SSWindowListener {
        @Override
        public void titleChanged(SSWindow window, String previousTitle) {
        }

        @Override
        public void iconAdded(SSWindow window, Image addedIcon) {
        }

        @Override
        public void iconInserted(SSWindow window, Image addedIcon, int index) {
        }

        @Override
        public void iconsRemoved(SSWindow window, int index, Sequence<Image> removed) {
        }

        @Override
        public void contentChanged(SSWindow window, Component previousContent) {
        }

        @Override
        public void activeChanged(SSWindow window, SSWindow obverseWindow) {
        }

        @Override
        public void maximizedChanged(SSWindow window) {
        }
    }

    /**
     * Called when a window's title has changed.
     *
     * @param window
     * @param previousTitle
     */
    public void titleChanged(SSWindow window, String previousTitle);

    /**
     * Called when a window's icon has changed.
     *
     * @param window
     * @param addedIcon
     */
    public void iconAdded(SSWindow window, Image addedIcon);

    /**
     * Called when a window's icon has changed.
     *
     * @param window
     * @param addedIcon
     */
    public void iconInserted(SSWindow window, Image addedIcon, int index);

    /**
     * Called when a window's icon has changed.
     *
     * @param window
     * @param index
     * @param removed
     */
    public void iconsRemoved(SSWindow window, int index, Sequence<Image> removed);

    /**
     * Called when a window's content component has changed.
     *
     * @param window
     * @param previousContent
     */
    public void contentChanged(SSWindow window, Component previousContent);

    /**
     * Called when a window's active state has changed.
     *
     * @param window
     * @param obverseWindow
     */
    public void activeChanged(SSWindow window, SSWindow obverseWindow);

    /**
     * Called when a window's maximized state has changed.
     *
     * @param window
     */
    public void maximizedChanged(SSWindow window);
}
