/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.listener.window;

import jp.pegasus.pivot.ss.SSWindow;
import org.apache.pivot.util.Vote;
import org.apache.pivot.wtk.Display;

public interface SSWindowStateListener {
    /**
     * SSWindow state listener adapter.
     */
    public static class Adapter implements SSWindowStateListener {
        @Override
        public void windowOpened(SSWindow window) {
        }

        @Override
        public Vote previewWindowClose(SSWindow window) {
            return Vote.APPROVE;
        }

        @Override
        public void windowCloseVetoed(SSWindow window, Vote reason) {
        }

        @Override
        public void windowClosed(SSWindow window, Display display, SSWindow owner) {
        }
    }

    /**
     * Called when a window has opened.
     *
     * @param window
     */
    public void windowOpened(SSWindow window);

    /**
     * Called to preview a window close event.
     *
     * @param window
     */
    public Vote previewWindowClose(SSWindow window);

    /**
     * Called when a window close event has been vetoed.
     *
     * @param window
     * @param reason
     */
    public void windowCloseVetoed(SSWindow window, Vote reason);

    /**
     * Called when a window has closed.
     *
     * @param window
     * @param display
     * @param owner
     */
    public void windowClosed(SSWindow window, Display display, SSWindow owner);
}
