/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */
package jp.pegasus.pivot.media.drawing;

import jp.pegasus.pivot.listener.drawing.CanvasListener;
import org.apache.pivot.util.ListenerList;

/**
 * Shape representing the root of a shape hierarchy.
 */
public class Canvas extends Group {
    private static class CanvasListenerList extends ListenerList<CanvasListener>
    implements CanvasListener {
        public void regionUpdated(Canvas canvas, int x, int y, int width, int height) {
            for (CanvasListener listener : this) {
                listener.regionUpdated(canvas, x, y, width, height);
            }
        }

        @Override
        public void canvasInvalidated(Canvas canvas) {
            for (CanvasListener listener : this) {
                listener.canvasInvalidated(canvas);
            }
        }
    }

    @Override
    protected void setParent(Group parent) {
        throw new UnsupportedOperationException();
    }

    @Override
    public void setOrigin(int x, int y) {
        throw new UnsupportedOperationException();
    }

    @Override
    public TransformSequence getTransforms() {
        throw new UnsupportedOperationException();
    }

    @Override
    public void invalidate() {
        if (isValid()) {
            super.invalidate();
            canvasListeners.canvasInvalidated(this);
        }
    }

    @Override
    protected void update(int x, int y, int width, int height) {
        super.update(x, y, width, height);
        canvasListeners.regionUpdated(this, x, y, width, height);
    }

    private CanvasListenerList canvasListeners = new CanvasListenerList();

    public ListenerList<CanvasListener> getCanvasListeners() {
        return canvasListeners;
    }
}
