/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */
package jp.pegasus.pivot.media.drawing;

import jp.pegasus.pivot.listener.drawing.RectangleListener;
import org.apache.pivot.util.ListenerList;

import java.awt.geom.Rectangle2D;
import java.awt.geom.RectangularShape;
import java.awt.geom.RoundRectangle2D;

/**
 * Shape representing a rectangle.
 */
public class Rectangle extends Shape2D {
    private static class RectangleListenerList extends ListenerList<RectangleListener>
    implements RectangleListener {
        public void sizeChanged(Rectangle rectangle, int previousWidth, int previousHeight) {
            for (RectangleListener listener : this) {
                listener.sizeChanged(rectangle, previousWidth, previousHeight);
            }
        }

        public void cornerRadiusChanged(Rectangle rectangle, int previousCornerRadius) {
            for (RectangleListener listener : this) {
                listener.cornerRadiusChanged(rectangle, previousCornerRadius);
            }
        }
    }

    // TODO Use only RoundRectangle2D.Float when Sun fixes rendering issues with zero-value
    // arc width and height
    private RectangularShape rectangularShape = new Rectangle2D.Float();

    private RectangleListenerList rectangleListeners = new RectangleListenerList();

    public int getWidth() {
        return (int) rectangularShape.getWidth();
    }

    public void setWidth(int width) {
        setSize(width, getHeight());
    }

    public int getHeight() {
        return (int) rectangularShape.getHeight();
    }

    public void setHeight(int height) {
        setSize(getWidth(), height);
    }

    public void setSize(int width, int height) {
        int previousWidth = (int) rectangularShape.getWidth();
        int previousHeight = (int) rectangularShape.getHeight();
        if (previousWidth != width
        || previousHeight != height) {
            if (rectangularShape instanceof Rectangle2D) {
                Rectangle2D.Float rectangle2D = (Rectangle2D.Float) rectangularShape;
                rectangle2D.width = width;
                rectangle2D.height = height;
            } else {
                RoundRectangle2D.Float roundRectangle2D = (RoundRectangle2D.Float) rectangularShape;
                roundRectangle2D.width = width;
                roundRectangle2D.height = height;
            }

            invalidate();
            rectangleListeners.sizeChanged(this, previousWidth, previousHeight);
        }
    }

    public int getCornerRadius() {
        int cornerRadius;
        if (rectangularShape instanceof Rectangle2D) {
            cornerRadius = 0;
        } else {
            RoundRectangle2D.Float roundRectangle2D = (RoundRectangle2D.Float) rectangularShape;
            cornerRadius = (int) roundRectangle2D.archeight;
        }

        return cornerRadius;
    }

    public void setCornerRadius(int cornerRadius) {
        int previousCornerRadius = getCornerRadius();

        int width = getWidth();
        int height = getHeight();

        if (cornerRadius == 0) {
            Rectangle2D.Float rectangle2D = new Rectangle2D.Float();
            rectangle2D.width = width;
            rectangle2D.height = height;

            rectangularShape = new Rectangle2D.Float();
        } else {
            RoundRectangle2D.Float roundRectangle2D = new RoundRectangle2D.Float();
            roundRectangle2D.width = width;
            roundRectangle2D.height = height;
            roundRectangle2D.arcwidth = cornerRadius;
            roundRectangle2D.archeight = cornerRadius;

            rectangularShape = roundRectangle2D;
        }

        invalidate();

        rectangleListeners.cornerRadiusChanged(this, previousCornerRadius);
    }

    @Override
    protected java.awt.Shape getShape2D() {
        return rectangularShape;
    }

    public ListenerList<RectangleListener> getRectangleListeners() {
        return rectangleListeners;
    }
}
