/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */
package jp.pegasus.pivot.media.drawing;

import java.awt.*;

/**
 * Abstract base class for shapes that encapsulate a Java2D shape.
 */
public abstract class Shape2D extends Shape {
    @Override
    public boolean contains(int x, int y) {
        boolean contains = false;

        java.awt.Shape shape2D = getShape2D();

        if (getFill() != null) {
            contains = shape2D.contains(x, y);
        }

        if (getStroke() != null) {
            BasicStroke basicStroke = new BasicStroke(getStrokeThickness());
            java.awt.Shape strokeShape = basicStroke.createStrokedShape(shape2D);
            contains |= strokeShape.contains(x, y);
        }

        return contains;
    }

    @Override
    public void draw(Graphics2D graphics) {
        java.awt.Shape shape2D = getShape2D();

        Paint fill = getFill();
        if (fill != null) {
            graphics.setPaint(fill);
            graphics.fill(shape2D);
        }

        Paint stroke = getStroke();
        if (stroke != null) {
            graphics.setPaint(stroke);
            BasicStroke basicStroke = new BasicStroke(getStrokeThickness());
            java.awt.Shape strokeShape = basicStroke.createStrokedShape(shape2D);
            graphics.fill(strokeShape);
        }
    }

    @Override
    protected void validate() {
        if (!isValid()) {
            java.awt.Shape shape2D = getShape2D();

            java.awt.Shape boundingShape;
            if (getStroke() == null) {
                boundingShape = shape2D;
            } else {
                int strokeThickness = getStrokeThickness();
                BasicStroke basicStroke = new BasicStroke(strokeThickness);
                java.awt.Shape strokeShape = basicStroke.createStrokedShape(shape2D);
                boundingShape = strokeShape;
            }

            java.awt.Rectangle bounds = boundingShape.getBounds();
            setBounds(bounds.x, bounds.y, bounds.width, bounds.height);
        }
    }

    protected abstract java.awt.Shape getShape2D();
}
