/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.model;

import org.apache.pivot.collections.ArrayList;
import org.apache.pivot.json.JSONSerializer;

import java.io.StringReader;
import java.io.StringWriter;

/**
 * SSData
 * <p/>
 * SSData*
 * | 1
 * |
 * | 0..*
 * SSFileData
 * | 1
 * |
 * | 0..*
 * SSGridData
 * | 1
 * |
 * | 0..*
 * SSElementalData
 */
public class SSData {

    public static final Type DEFAULT_TYPE = Type.Default;
    public static final String DEF_FIL_NAME = "New File";
    public static final String DEF_CPY_FIL_NAME = "Copy of %s";
    public static final String DEF_SHT_NAME = "New Sheet %d";
    public static final String DEF_SHT_ADD = "+";

    private enum Type {
        Default,
    }

    /**
     * Initial Data Creation
     */
    public static SSData initializeData() {
        return new SSData() {{
            this.setSelectFileIndex(0);
        }};
    }

    /**
     * Create Sheet
     */
    public static SSFileData createSheetFile(final String fileName, final String[] sheetNameArr, final int sheetType) {
        return new SSFileData() {{
            this.setSelectSheetIndex(0);
            this.setCreateTime(System.currentTimeMillis());
            this.setFileName(fileName);
            this.setSheetType(sheetType);
            this.setSheetDataList(
            new ArrayList<Object>() {{
                /**
                 * SheetData: Sheet Object
                 */
                if (sheetType == 0) {
                    for (String sheetName : sheetNameArr) {
                        this.add(SSData.createGridSheet(sheetName));
                    }
                } else if (sheetType == 1) {
                    for (String sheetName : sheetNameArr) {
                        this.add(SSData.createTextSheet(sheetName));
                    }
                } else {
                    throw new RuntimeException();
                }
            }});
        }};
    }

    /**
     * Create Grid Sheet
     */
    public static SSGridData createGridSheet(final String sheetName) {
        SSGridData gridData = new SSGridData() {{
            this.setSelectCurrentIndexQuartet(0l);
            this.setCreateTime(System.currentTimeMillis());
            this.setSheetName(sheetName);
            this.setCellDataDigest(null);
        }};

        return gridData;
    }

    /**
     * Create Text Sheet
     */
    public static SSTextData createTextSheet(final String sheetName) {
        SSTextData textData = new SSTextData() {{
            this.setCreateTime(System.currentTimeMillis());
            this.setSheetName(sheetName);
        }};

        return textData;
    }

    private ArrayList<SSFileData> fileDataList = new ArrayList<SSFileData>();
    private int selectFileIndex = 0;

    public ArrayList<SSFileData> getFileDataList() {
        return fileDataList;
    }

    public void setFileDataList(ArrayList<SSFileData> fileDataList) {
        this.fileDataList = fileDataList;
    }

    public int getSelectFileIndex() {
        return selectFileIndex;
    }

    public void setSelectFileIndex(int selectFileIndex) {
        this.selectFileIndex = selectFileIndex;
    }

    @SuppressWarnings("unchecked")
    public static void main(String[] args) throws Exception {
        SSData data = SSData.initializeData();
        JSONSerializer jsonSerializer = new JSONSerializer(SSData.class);

        try {
            StringWriter stringWriter = new StringWriter();
            jsonSerializer.writeObject(data, stringWriter);

            StringReader stringReader = new StringReader(stringWriter.toString());
            SSData readData = (SSData) jsonSerializer.readObject(stringReader);

            System.out.println(readData.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
