/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.model;

import jp.pegasus.config.Config;
import org.apache.pivot.collections.ArrayList;
import org.apache.pivot.collections.HashMap;

/**
 * SSElementalData
 * <p/>
 * SSData
 * | 1
 * |
 * | 0..*
 * SSFileData
 * | 1
 * |
 * | 0..*
 * SSSheetData
 * | 1
 * |
 * .------------------.
 * |                  |
 * | 0..1             | 0..1
 * SSGridData         SSTextData*
 * | 1                | 1
 * |                  |
 * | 0..*             | 0..1
 * SSElementalData*   SSElementalData*
 */
public class SSElementalData implements Cloneable {

    /**
     * Create Cell Data List
     */
    public static ArrayList<HashMap<String, SSElementalData>> createCellDataList() {

        return new ArrayList<HashMap<String, SSElementalData>>() {{
            for (int i = 1, rowNum = Config.MAX_ROW_NUMBER; i < rowNum; i++) {
                this.add(new HashMap<String, SSElementalData>());
            }
        }};
    }


    private String font;
    private String text;

    /**
     * Color
     * <p/>
     * 0x00000000 format
     * <p/>
     * value = value.substring(2);
     * if (value.length() != 8) {
     * throw new IllegalArgumentException();
     * }
     * int rgb = Integer.parseInt(value.substring(0, 6), 16);
     * float alpha = Integer.parseInt(value.substring(6, 8), 16) / 255f;
     */
    private int color;
    private int backgroundColor;

    public SSElementalData() {
    }

    public SSElementalData(String font, String text, int color, int backgroundColor) {
        this.font = font;
        this.text = text;
        this.color = color;
        this.backgroundColor = backgroundColor;
    }

    public String getFont() {
        return font;
    }

    public void setFont(String font) {
        this.font = font;
    }

    public String getText() {
        return text;
    }

    public void setText(String text) {
        this.text = text;
    }

    public int getColor() {
        return color;
    }

    public void setColor(int color) {
        this.color = color;
    }

    public int getBackgroundColor() {
        return backgroundColor;
    }

    public void setBackgroundColor(int backgroundColor) {
        this.backgroundColor = backgroundColor;
    }

    @Override
    public SSElementalData clone() throws CloneNotSupportedException {
        SSElementalData copiedElementalData = (SSElementalData) super.clone();
        copiedElementalData.setBackgroundColor(this.getBackgroundColor());
        copiedElementalData.setColor(this.getColor());
        copiedElementalData.setFont(this.getFont());
        copiedElementalData.setText(this.getText());

        return copiedElementalData;
    }
}