/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.model;

import org.apache.pivot.collections.ArrayList;
import org.apache.pivot.collections.HashMap;

/**
 * SSFileData
 * <p/>
 * SSData
 * | 1
 * |
 * | 0..*
 * SSFileData*
 * | 1
 * |
 * | 0..*
 * SSSheetData
 * | 1
 * |
 * .----------------.
 * |                |
 * | 0..1           | 0..1
 * SSGridData       SSTextData*
 * | 1
 * |
 * | 0..*
 * SSElementalData
 */
public class SSFileData implements Cloneable {
    private String fileName = null;
    private long createTime;
    private int selectSheetIndex = 0;

    /**
     * sheetType
     * 0: Grid
     * 1: Text
     */
    private int sheetType = 0;
    private ArrayList<Object> sheetDataList = new ArrayList<Object>();

    public SSFileData() {
    }

    public int getSheetType() {
        return sheetType;
    }

    public void setSheetType(int sheetType) {
        this.sheetType = sheetType;
    }

    public String getFileName() {
        return fileName;
    }

    public void setFileName(String fileName) {
        this.fileName = fileName;
    }

    public long getCreateTime() {
        return createTime;
    }

    public void setCreateTime(final long createTime) {
        this.createTime = createTime;
    }

    public ArrayList<Object> getSheetDataList() {
        return sheetDataList;
    }

    public void setSheetDataList(ArrayList<Object> sheetDataList) {
        this.sheetDataList = sheetDataList;
    }

    public int getSelectSheetIndex() {
        return selectSheetIndex;
    }

    public void setSelectSheetIndex(int selectSheetIndex) {
        this.selectSheetIndex = selectSheetIndex;
    }

    @Override
    public SSFileData clone() throws CloneNotSupportedException {
        SSFileData copiedFileData = (SSFileData) super.clone();
        copiedFileData.setCreateTime(System.currentTimeMillis());
        copiedFileData.setFileName(this.getFileName());
        copiedFileData.setSelectSheetIndex(this.getSelectSheetIndex());
        copiedFileData.setSheetType(this.getSheetType());

        ArrayList<Object> copiedSheetDataList = new ArrayList<Object>();
        for (int i = 0, fileTypeNum = this.getSheetDataList().getLength(); i < fileTypeNum; i++) {

            Object sheetData = this.getSheetDataList().get(i);
            if (sheetData instanceof SSGridData) {
                SSGridData gridData = ((SSGridData) sheetData).clone();
                copiedSheetDataList.add(gridData);
            } else if (sheetData instanceof SSTextData) {
                SSTextData textData = ((SSTextData) sheetData).clone();
                copiedSheetDataList.add(textData);
            } else if (sheetData instanceof HashMap) {
                final int sheetType = this.getSheetType();
                if (sheetType == 0) {
                    SSGridData gridData = SSGridData.convertMapToObject((HashMap<String, ?>) sheetData);
                    copiedSheetDataList.add(gridData);
                } else if (sheetType == 1) {
                    SSTextData textData = SSTextData.convertMapToObject((HashMap<String, ?>) sheetData);
                    copiedSheetDataList.add(textData);
                }
            }
        }
        copiedFileData.setSheetDataList(copiedSheetDataList);

        return copiedFileData;
    }
}
