/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.model;


/**
 * SSGridData
 * <p/>
 * SSData
 * | 1
 * |
 * | 0..*
 * SSFileData
 * | 1
 * |
 * | 0..*
 * SSSheetData
 * | 1
 * |
 * .----------------.
 * |                |
 * | 0..1           | 0..1
 * SSGridData*      SSTextData
 * | 1              | 1
 * |                |
 * | 0..*           | 0..1
 * SSElementalData  SSElementalData
 */
public class SSGridData implements Cloneable {

    public static SSGridData convertMapToObject(org.apache.pivot.collections.HashMap<String, ?> gridDataMap) {
        final String sheetName = (String) gridDataMap.get("sheetName");
        Object createTimeObj = gridDataMap.get("createTime");
        final long createTime = (createTimeObj == null) ? System.currentTimeMillis() : (Long) createTimeObj;
        Object selCurIdxQuartetObj = gridDataMap.get("selectCurrentIndexQuartet");
        final int selectCurrentIndexQuartet = (selCurIdxQuartetObj == null) ? 0 : (Integer) selCurIdxQuartetObj;
        final String cellDataDigest = (String) gridDataMap.get("cellDataDigest");

        SSGridData gridData = new SSGridData();
        gridData.setSheetName(sheetName);
        gridData.setCreateTime(createTime);
        gridData.setSelectCurrentIndexQuartet((long) selectCurrentIndexQuartet);
        gridData.setCellDataDigest(cellDataDigest);

        return gridData;
    }

    private String sheetName = null;
    private long createTime;
    private long selectCurrentIndexQuartet = 0x0000000000000000l;
    private String cellDataDigest = null;

    public SSGridData() {
    }

    public String getSheetName() {
        return sheetName;
    }

    public void setSheetName(String sheetName) {
        this.sheetName = sheetName;
    }

    public long getCreateTime() {
        return createTime;
    }

    public void setCreateTime(long createTime) {
        this.createTime = createTime;
    }

    public long getSelectCurrentIndexQuartet() {
        return selectCurrentIndexQuartet;
    }

    public void setSelectCurrentIndexQuartet(long selectCurrentIndexQuartet) {
        this.selectCurrentIndexQuartet = selectCurrentIndexQuartet;
    }

    public String getCellDataDigest() {
        return cellDataDigest;
    }

    public void setCellDataDigest(String cellDataDigest) {
        this.cellDataDigest = cellDataDigest;
    }

    @Override
    public SSGridData clone() throws CloneNotSupportedException {
        SSGridData copiedGridData = (SSGridData) super.clone();
        copiedGridData.setCreateTime(System.currentTimeMillis());
        copiedGridData.setSelectCurrentIndexQuartet(this.getSelectCurrentIndexQuartet());
        copiedGridData.setSheetName(this.getSheetName());
        copiedGridData.setCellDataDigest(this.getCellDataDigest());

        return copiedGridData;
    }
}