/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.model;

/**
 * SSTextData
 * <p/>
 * SSData
 * | 1
 * |
 * | 0..*
 * SSFileData
 * | 1
 * |
 * | 0..*
 * SSSheetData
 * | 1
 * |
 * .----------------.
 * |                |
 * | 0..1           | 0..1
 * SSGridData       SSTextData*
 * | 1
 * |
 * | 0..*
 * SSElementalData
 */
public class SSTextData implements Cloneable {

    public static SSTextData convertMapToObject(org.apache.pivot.collections.HashMap<String, ?> textDataMap) {
        final String sheetName = (String) textDataMap.get("sheetName");
        Object createTimeObj = textDataMap.get("createTime");
        final long createTime = (createTimeObj == null) ? System.currentTimeMillis() : (Long) createTimeObj;
        final String textDataDigest = (String) textDataMap.get("textDataDigest");

        SSTextData textData = new SSTextData();
        textData.setSheetName(sheetName);
        textData.setCreateTime(createTime);
        textData.setTextDataDigest(textDataDigest);

        return textData;
    }

    private String sheetName = null;
    private long createTime;
    private String textDataDigest = null;

    public String getSheetName() {
        return sheetName;
    }

    public void setSheetName(String sheetName) {
        this.sheetName = sheetName;
    }

    public long getCreateTime() {
        return createTime;
    }

    public void setCreateTime(long createTime) {
        this.createTime = createTime;
    }

    public String getTextDataDigest() {
        return textDataDigest;
    }

    public void setTextDataDigest(String textDataDigest) {
        this.textDataDigest = textDataDigest;
    }

    @Override
    public SSTextData clone() throws CloneNotSupportedException {
        SSTextData copiedTextData = (SSTextData) super.clone();

        return copiedTextData;
    }
}
