/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *     Pegasus is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     Pegasus is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.navigation;

import org.apache.pivot.wtk.Component;
import org.apache.pivot.wtk.Orientation;
import org.apache.pivot.wtk.skin.ComponentSkin;

import java.awt.*;

class RulerSkin extends ComponentSkin implements RulerListener {
    @Override
    public void install(Component component) {
        super.install(component);

        Ruler ruler = (Ruler)component;
        ruler.getRulerListeners().add(this);
    }

    @Override
    public void layout() {
        // No-op
    }

    @Override
    public int getPreferredHeight(int width) {
        Ruler ruler = (Ruler)getComponent();
        Orientation orientation = ruler.getOrientation();

        return (orientation == Orientation.HORIZONTAL) ? 20 : 0;
    }

    @Override
    public int getPreferredWidth(int height) {
        Ruler ruler = (Ruler)getComponent();
        Orientation orientation = ruler.getOrientation();

        return (orientation == Orientation.VERTICAL) ? 20 : 0;
    }

    @Override
    public void paint(Graphics2D graphics) {
        int width = getWidth();
        int height = getHeight();

        Ruler ruler = (Ruler)getComponent();

        graphics.setColor(new Color(0xFF, 0xFF, 0xE0));
        graphics.fillRect(0, 0, width, height);

        graphics.setColor(Color.BLACK);
        graphics.drawRect(0, 0, width - 1, height - 1);

        Orientation orientation = ruler.getOrientation();
        switch (orientation) {
            case HORIZONTAL: {
                for (int i = 0, n = width / 5 + 1; i < n; i++) {
                    int x = i * 5;

                    if (i % 4 == 0) {
                        graphics.drawLine(x, 0, x, height / 2);
                    } else {
                        graphics.drawLine(x, 0, x, height / 4);
                    }
                }

                break;
            }

            case VERTICAL: {
                for (int i = 0, n = height / 5 + 1; i < n; i++) {
                    int y = i * 5;

                    if (i % 4 == 0) {
                        graphics.drawLine(0, y, width / 2, y);
                    } else {
                        graphics.drawLine(0, y, width / 4, y);
                    }
                }

                break;
            }
        }
    }

    public void orientationChanged(Ruler ruler) {
        invalidateComponent();
    }
}
