/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.sample;

import jp.pegasus.main.Main;
import jp.pegasus.pivot.ss.*;
import org.apache.pivot.collections.ArrayList;
import org.apache.pivot.collections.List;
import org.apache.pivot.util.concurrent.TaskExecutionException;
import org.apache.pivot.wtk.media.Image;

import java.awt.*;
import java.util.Random;

public class SampleCreator {

    public static SSTabPane createSampleTabPane() {
        SSTabPane tabPane = new SSTabPane();

        for (int i = 0; i < 11; i++) {
            final int No = i;
            tabPane.getTabs().add(new SSTextArea() {
                {
                    setText(SampleCreator.echo("Label " + No, 100));
                    SSTabPane.setTabData(this, "Label" + No);
                    getStyles().put("Color", new Color(No * 25, No * 25, No * 25));
                    getStyles().put("BackgroundColor", new Color(255 - No * 25, 255 - No * 25, 255 - No * 25));
                }
            });
        }

        return tabPane;
    }

    public static SSTreeView getSampleTreeView() {
        SSTreeView treeView = new SSTreeView();

        try {
            List<? super SSTreeBranch> branches1 = new ArrayList<SSTreeBranch>();
            for (int i = 1; i < 10; i++) {
                branches1.add(SampleCreator.getRandomTreeBranch());
            }

            treeView.setTreeData(branches1);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return treeView;
    }

    public static Image getRandomImageBranch() {
        Image imgBranch = null;
        Random rnd = new Random();

        try {
            switch (rnd.nextInt(9) % 9) {
                case 0:
                    imgBranch = Image.load(Main.class.getResource("/image/folder.png"));
                    break;
                case 1:
                    imgBranch = Image.load(Main.class.getResource("/image/book_open.png"));
                    break;
                case 2:
                    imgBranch = Image.load(Main.class.getResource("/image/folder.png"));
                    break;
                case 3:
                    imgBranch = Image.load(Main.class.getResource("/image/flag_red.png"));
                    break;
                case 4:
                    imgBranch = Image.load(Main.class.getResource("/image/folder.png"));
                    break;
                case 5:
                    imgBranch = Image.load(Main.class.getResource("/image/star.png"));
                    break;
                case 6:
                    imgBranch = Image.load(Main.class.getResource("/image/folder.png"));
                    break;
                case 7:
                    imgBranch = Image.load(Main.class.getResource("/image/help.png"));
                    break;
                case 8:
                    imgBranch = Image.load(Main.class.getResource("/image/folder.png"));
                    break;
                default:
                    imgBranch = Image.load(Main.class.getResource("/image/folder.png"));
                    break;
            }
        } catch (TaskExecutionException e) {
            e.printStackTrace();
        }
        return imgBranch;
    }

    public static Image getRandomImageDocument() {
        Image imgDocument = null;
        Random rnd = new Random();

        try {
            switch (rnd.nextInt(9) % 9) {
                case 0:
                    imgDocument = Image.load(Main.class.getResource("/image/document-new.png"));
                    break;
                case 1:
                    imgDocument = Image.load(Main.class.getResource("/image/document-open.png"));
                    break;
                case 2:
                    imgDocument = Image.load(Main.class.getResource("/image/document-new.png"));
                    break;
                case 3:
                    imgDocument = Image.load(Main.class.getResource("/image/document-save-as.png"));
                    break;
                case 4:
                    imgDocument = Image.load(Main.class.getResource("/image/document-new.png"));
                    break;
                case 5:
                    imgDocument = Image.load(Main.class.getResource("/image/document-save.png"));
                    break;
                case 6:
                    imgDocument = Image.load(Main.class.getResource("/image/document-new.png"));
                    break;
                case 7:
                    imgDocument = Image.load(Main.class.getResource("/image/x-office-document.png"));
                    break;
                case 8:
                    imgDocument = Image.load(Main.class.getResource("/image/document-new.png"));
                    break;
                default:
                    imgDocument = Image.load(Main.class.getResource("/image/document-new.png"));
                    break;
            }
        } catch (TaskExecutionException e) {
            e.printStackTrace();
        }
        return imgDocument;
    }


    public static SSTreeNode getRandomTreeNode() {
        Random rnd = new Random();
        int randI = rnd.nextInt(1000);
        return new SSTreeNode(SampleCreator.getRandomImageDocument(), "leaf " + randI, null);
    }

    public static SSTreeBranch getRandomTreeBranch() {
        Random rnd = new Random();
        int randI = rnd.nextInt(100);

        SSTreeBranch branch = new SSTreeBranch(SampleCreator.getRandomImageBranch(), "Branch " + randI, null);

        if (randI % 3 == 0 || randI % 3 == 1) {
            branch.add(getRandomTreeBranch());
            if (rnd.nextInt(100) % 10 == 0) {
                branch.add(getRandomTreeBranch());
            }
            if (rnd.nextInt(100) % 10 == 1) {
                branch.add(getRandomTreeNode());
            }
            if (rnd.nextInt(100) % 10 == 2) {
                branch.add(getRandomTreeNode());
            }
            if (rnd.nextInt(100) % 10 == 3) {
                branch.add(getRandomTreeNode());
            }
            if (rnd.nextInt(100) % 10 == 4) {
                branch.add(getRandomTreeNode());
            }
            if (rnd.nextInt(100) % 10 == 5) {
                branch.add(getRandomTreeNode());
            }
            if (rnd.nextInt(100) % 10 == 6) {
                branch.add(getRandomTreeNode());
            }
            if (rnd.nextInt(100) % 10 == 7) {
                branch.add(getRandomTreeNode());
            }
        } else if (randI % 3 == 2) {
            branch.add(getRandomTreeNode());
            if (rnd.nextInt(100) % 10 == 0) {
                branch.add(getRandomTreeBranch());
            }
            if (rnd.nextInt(100) % 10 == 1) {
                branch.add(getRandomTreeNode());
            }
            if (rnd.nextInt(100) % 10 == 2) {
                branch.add(getRandomTreeNode());
            }
            if (rnd.nextInt(100) % 10 == 3) {
                branch.add(getRandomTreeNode());
            }
            if (rnd.nextInt(100) % 10 == 4) {
                branch.add(getRandomTreeNode());
            }
            if (rnd.nextInt(100) % 10 == 5) {
                branch.add(getRandomTreeNode());
            }
            if (rnd.nextInt(100) % 10 == 6) {
                branch.add(getRandomTreeNode());
            }
            if (rnd.nextInt(100) % 10 == 7) {
                branch.add(getRandomTreeNode());
            }
        }

        return branch;
    }

    public static String echo(String word, int time) {
        StringBuffer echo = new StringBuffer();
        for (int i = 0; i < time; i++) {
            echo.append(word);
            echo.append(" ");
            if (i % 20 == 0) {
                //echo.append("\r\n");
            }
        }
        return echo.toString();
    }

}
