/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.skin;

import jp.pegasus.config.Config;
import jp.pegasus.pivot.listener.tableview.SSTableViewColumnListener;
import jp.pegasus.pivot.listener.tableview.SSTableViewHeaderListener;
import jp.pegasus.pivot.listener.tableview.SSTableViewSortListener;
import jp.pegasus.pivot.ss.tableview.SSTableView;
import jp.pegasus.pivot.ss.tableview.SSTableViewHeader;
import org.apache.pivot.collections.ArrayList;
import org.apache.pivot.collections.Dictionary;
import org.apache.pivot.collections.Sequence;
import org.apache.pivot.wtk.*;
import org.apache.pivot.wtk.Component;
import org.apache.pivot.wtk.Cursor;
import org.apache.pivot.wtk.skin.ComponentSkin;
import org.apache.pivot.wtk.skin.terra.TerraTheme;

import java.awt.*;

public class TableViewHeaderSkin extends ComponentSkin
implements SSTableViewHeader.Skin, SSTableViewHeaderListener, SSTableViewColumnListener,
SSTableViewSortListener, Config {
    private Font font;
    private Color color;
    private Color disabledColor;
    private Color backgroundColor;
    private Color disabledBackgroundColor;
    private Color borderColor;
    private Color disabledBorderColor;
    private boolean headersPressable;
    private boolean columnsResizable;
    private boolean includeTrailingVerticalGridLine;

    private Color bevelColor;
    private Color pressedBevelColor;
    private Color disabledBevelColor;

    private ArrayList<Integer> headerWidths = null;

    private int pressedHeaderIndex = -1;
    private int resizeHeaderIndex = -1;

    private Color selectBackgroundColor;
    private Color verticalGridColor;
    private Color selectVerticalGridColor;
    private Color selectRangeGridColor;

    private static final int SORT_INDICATOR_PADDING = 2;
    private static final int RESIZE_HANDLE_SIZE = 6;
    public static final int MINIMUM_COLUMN_WIDTH = 2;

    public TableViewHeaderSkin() {
        TerraTheme theme = (TerraTheme) Theme.getTheme();

        font = theme.getFont();
        color = theme.getColor(1);
        disabledColor = theme.getColor(7);
        backgroundColor = theme.getColor(10);
        disabledBackgroundColor = theme.getColor(10);
        borderColor = theme.getColor(7);
        disabledBorderColor = theme.getColor(7);
        headersPressable = true;
        columnsResizable = true;
        includeTrailingVerticalGridLine = false;

        // Set the derived colors
        bevelColor = TerraTheme.brighten(backgroundColor);
        pressedBevelColor = TerraTheme.darken(backgroundColor);
        disabledBevelColor = disabledBackgroundColor;
    }

    @Override
    public void paint(Graphics2D graphics) {
        int width = getWidth();
        int height = getHeight();

        SSTableViewHeader tableViewHeader = (SSTableViewHeader) getComponent();

        Color backgroundColor = null;
        Color borderColor = null;

        if (tableViewHeader.isEnabled()) {
            backgroundColor = this.backgroundColor;
            borderColor = this.borderColor;
        } else {
            backgroundColor = this.disabledBackgroundColor;
            borderColor = this.disabledBorderColor;
        }

        // Paint the background
        graphics.setPaint(backgroundColor);
        graphics.fillRect(0, 0, width, height);

        // Paint the border
        graphics.setPaint(borderColor);
        GraphicsUtilities.drawLine(graphics, 0, height - 1, width, Orientation.HORIZONTAL);

        // Paint the content
        SSTableView tableView = tableViewHeader.getTableView();

        if (tableView != null) {
            SSTableView.ColumnSequence columns = tableView.getColumns();

            // Ensure that we only paint items that are visible
            int columnCount = columns.getLength();
            int columnStart = 0;
            int columnEnd = columnCount - 1;

            Rectangle clipBounds = graphics.getClipBounds();
//            log.debug("clipBounds:{}", clipBounds.toString());
            if (clipBounds != null) {
                columnStart = getHeaderAt(clipBounds.x + 1);
                if (columnStart == -1) {
                    columnStart = columns.getLength();
                }

                if (columnEnd != -1) {
                    int clipRight = clipBounds.x + clipBounds.width - 1;
                    clipRight = Math.min(clipRight, getHeaderBoundaries(columnEnd) - 1);
                    columnEnd = getHeaderAt(clipRight);
                }
            }

            int[] selectRange = tableViewHeader.getSelectRange();
            final long selectIndexQuartet = tableView.getSelectIndexQuartet();
            final int selectColumnIndexFr = (int) (0x000000000000FFFFl & (selectIndexQuartet >> 48));
            final int selectColumnIndexTo = (int) (0x000000000000FFFFl & (selectIndexQuartet >> 16));

            if (tableViewHeader.isSelect()) {
                graphics.setPaint(selectBackgroundColor);
                graphics.fillRect(selectRange[0], selectRange[1], selectRange[2], selectRange[3] + 2);
            }

            int headerX = getHeaderX(columnStart);
            for (int columnIndex = columnStart; columnIndex <= columnEnd; columnIndex++) {
                SSTableView.Column column = columns.get(columnIndex);
                int headerWidth = headerWidths.get(columnIndex);
//                log.debug("headerX: {}", headerX);

                // Paint the header data
                Object headerData = column.getHeaderData();
                SSTableView.HeaderDataRenderer headerDataRenderer = column.getHeaderDataRenderer();
                if (tableViewHeader.isSelect() && selectColumnIndexFr <= columnIndex && columnIndex <= selectColumnIndexTo) {
                    headerDataRenderer.render(headerData, columnIndex, tableViewHeader, column.getName(), true);
                } else {
                    headerDataRenderer.render(headerData, columnIndex, tableViewHeader, column.getName(), false);
                }
                headerDataRenderer.setSize(headerWidth, height);

                Graphics2D rendererGraphics =
                (Graphics2D) graphics.create(headerX, 0, headerWidth, height);
                headerDataRenderer.paint(rendererGraphics);
//                rendererGraphics.dispose();

                // Draw the divider
                headerX += headerWidth;
                if (columnIndex <= columnEnd || includeTrailingVerticalGridLine) {
                    if (tableViewHeader.isSelect() && selectColumnIndexFr <= columnIndex && columnIndex <= selectColumnIndexTo) {
                        graphics.setPaint(getSelectVerticalGridColor());
                    } else {
                        graphics.setPaint(getVerticalGridColor());
                    }
                    GraphicsUtilities.drawLine(graphics, headerX, clipBounds.y, clipBounds.height, Orientation.VERTICAL);
                }
                headerX++;
            }

            if (tableViewHeader.isSelect()) {
                graphics.setPaint(getSelectRangeGridColor());
                graphics.drawRect(selectRange[0] - 1, selectRange[1], selectRange[2] + 1, selectRange[3] + 1);
            }

        }
    }

    @Override
    public int getBaseline(int width, int height) {
        int baseline = -1;

        SSTableViewHeader tableViewHeader = (SSTableViewHeader) getComponent();
        SSTableView tableView = tableViewHeader.getTableView();

        if (tableView != null) {
//            ArrayList<Integer> headerWidths = TerraTableViewSkin.getColumnWidths(tableView, width);
            ArrayList<Integer> headerWidths = SSTableViewSkin.getColumnWidths(tableView, width);
            int rowHeight = getPreferredHeight(width) - 1;

            SSTableView.ColumnSequence columns = tableView.getColumns();
            for (int i = 0, n = columns.getLength(); i < n; i++) {
                SSTableView.Column column = columns.get(i);
                SSTableView.HeaderDataRenderer headerDataRenderer = column.getHeaderDataRenderer();
                headerDataRenderer.render(column.getHeaderData(), i, tableViewHeader, column.getName(), false);
                baseline = Math.max(baseline, headerDataRenderer.getBaseline(headerWidths.get(i), rowHeight));
            }
        }

        return baseline;
    }

    @Override
    public void layout() {
        SSTableViewHeader tableViewHeader = (SSTableViewHeader) getComponent();
        SSTableView tableView = tableViewHeader.getTableView();

        if (tableView != null) {
//            headerWidths = TerraTableViewSkin.getColumnWidths(tableView, getWidth());
            headerWidths = SSTableViewSkin.getColumnWidths(tableView, getWidth());
        } else {
            headerWidths = null;
        }
    }

    @Override
    public int getHeaderAt(int x) {
        if (x < 0) {
            throw new IllegalArgumentException("x is negative");
        }

        int headerIndex = -1;

        SSTableViewHeader tableViewHeader = (SSTableViewHeader) getComponent();
        SSTableView tableView = tableViewHeader.getTableView();
        if (tableView != null) {
            int i = 0;
            int n = tableView.getColumns().getLength();
            int headerX = 0;
            while (i < n && x > headerX) {
                headerX += (headerWidths.get(i) + 1);
                i++;
            }

            if (x <= headerX) {
                headerIndex = i - 1;
            }
        }

        return headerIndex;
    }

    @Override
    public Bounds getHeaderBounds(int headerIndex) {
        Bounds headerBounds = null;

        SSTableViewHeader tableViewHeader = (SSTableViewHeader) getComponent();
        SSTableView tableView = tableViewHeader.getTableView();
        if (tableView != null) {
            if (headerIndex < 0 || headerIndex >= headerWidths.getLength()) {
                throw new IndexOutOfBoundsException();
            }

            int cellX = 0;
            for (int i = 0; i < headerIndex; i++) {
                cellX += (headerWidths.get(i) + 1);
            }

            headerBounds = new Bounds(cellX, 0, headerWidths.get(headerIndex), getHeight() - 1);
        }

        return headerBounds;
    }

    public void setHeadersPressable(boolean headersPressable) {
        this.headersPressable = headersPressable;

        pressedHeaderIndex = -1;
        repaintComponent();
    }

    @Override
    public void enabledChanged(org.apache.pivot.wtk.Component component) {
        super.enabledChanged(component);

        pressedHeaderIndex = -1;
        resizeHeaderIndex = -1;
        repaintComponent();
    }

    @Override
    public void mouseOut(Component component) {
        super.mouseOut(component);

        if (pressedHeaderIndex != -1) {
            repaintComponent(getHeaderBounds(pressedHeaderIndex));
        }

        pressedHeaderIndex = -1;
    }

    @Override
    public boolean mouseDown(Component component, Mouse.Button button, int x, int y) {
        boolean consumed = super.mouseDown(component, button, x, y);

        if (button == Mouse.Button.LEFT) {
            SSTableViewHeader tableViewHeader = (SSTableViewHeader) component;
            SSTableView tableView = tableViewHeader.getTableView();

            if (tableView != null) {
                int headerIndex = getHeaderAt(x);

                if (headerIndex != -1) {
                    Bounds headerBounds = getHeaderBounds(headerIndex);
                    SSTableView.Column column = tableView.getColumns().get(headerIndex);

                    if (columnsResizable
                    && !column.isRelative()
                    && column.getWidth() != -1
                    && x > headerBounds.x + headerBounds.width - RESIZE_HANDLE_SIZE) {
                        resizeHeaderIndex = headerIndex;
                    } else if (headersPressable) {
                        pressedHeaderIndex = headerIndex;
                        repaintComponent(headerBounds);
                    }
                }
            }
        }

        return consumed;
    }

    @Override
    public boolean mouseUp(Component component, Mouse.Button button, int x, int y) {
        //boolean consumed = super.mouseUp(component, button, x, y);

        if (button == Mouse.Button.LEFT) {
            if (resizeHeaderIndex != -1) {
                if (Mouse.getCapturer() == component) {
                    Mouse.release();
                    resizeHeaderIndex = -1;
                }
            } else if (pressedHeaderIndex != -1) {
                repaintComponent(getHeaderBounds(pressedHeaderIndex));
            }
        }

        return false;
        //return consumed;
    }

    @Override
    public boolean mouseClick(Component component, Mouse.Button button, int x, int y, int count) {
        boolean consumed = super.mouseClick(component, button, x, y, count);

        if (button == Mouse.Button.LEFT) {
            SSTableViewHeader tableViewHeader = (SSTableViewHeader) component;
            SSTableView tableView = tableViewHeader.getTableView();

            if (resizeHeaderIndex != -1) {
                SSTableView.Column column = tableView.getColumns().get(resizeHeaderIndex);

                if (count == 2
                && !column.isRelative()
                && column.getWidth() != -1) {
                    // Size the column to fit its contents
                    int columnWidth = 0;

                    SSTableView.CellRenderer cellRenderer = column.getCellRenderer();
                    org.apache.pivot.collections.List<?> tableData = tableView.getTableData();

                    int rowIndex = 0;
                    for (Object rowData : tableData) {
                        cellRenderer.render(rowData, rowIndex++, resizeHeaderIndex, tableView,
                        column.getName(), false, false, false, false, false, false);
                        columnWidth = Math.max(cellRenderer.getPreferredWidth(-1), columnWidth);
                    }

                    column.setWidth(columnWidth);
                }
            } else if (pressedHeaderIndex != -1) {
                // Press the header
                tableViewHeader.pressHeader(pressedHeaderIndex);

                // Update the sort
                SSTableViewHeader.SortMode sortMode = tableViewHeader.getSortMode();

                if (sortMode != SSTableViewHeader.SortMode.NONE) {
                    SSTableView.Column column = tableView.getColumns().get(pressedHeaderIndex);
                    String columnName = column.getName();

                    SortDirection sortDirection = tableView.getSort().get(columnName);
                    if (sortDirection == null) {
                        sortDirection = SortDirection.ASCENDING;
                    } else if (sortDirection == SortDirection.ASCENDING) {
                        sortDirection = SortDirection.DESCENDING;
                    } else {
                        sortDirection = SortDirection.ASCENDING;
                    }

                    if (sortMode == SSTableViewHeader.SortMode.MULTI_COLUMN
                    && Keyboard.isPressed(Keyboard.Modifier.SHIFT)) {
                        tableView.getSort().put(columnName, sortDirection);
                    } else {
                        tableView.setSort(columnName, sortDirection);
                    }

                    consumed = true;
                }
            }

            resizeHeaderIndex = -1;
            pressedHeaderIndex = -1;
        }

        return consumed;
    }

    @Override
    public boolean mouseMove(Component component, int x, int y) {
        SSTableViewHeader tableViewHeader = (SSTableViewHeader) component;
        SSTableView tableView = tableViewHeader.getTableView();

        if (tableView != null) {
            if (resizeHeaderIndex != -1
            && Mouse.getCapturer() != tableViewHeader) {
                Mouse.capture(tableViewHeader);
            }

            if (Mouse.getCapturer() == tableViewHeader) {
                SSTableView.Column column = tableView.getColumns().get(resizeHeaderIndex);
                Bounds headerBounds = getHeaderBounds(resizeHeaderIndex);
                int columnWidth = Math.max(x - headerBounds.x, MINIMUM_COLUMN_WIDTH);
                column.setWidth(columnWidth, false);
            } else {
                int headerIndex = getHeaderAt(x);

                if (headerIndex != -1 && columnsResizable) {
                    Bounds headerBounds = getHeaderBounds(headerIndex);
                    SSTableView.Column column = tableView.getColumns().get(headerIndex);

                    if (!column.isRelative()
                    && column.getWidth() != -1
                    && x > headerBounds.x + headerBounds.width - RESIZE_HANDLE_SIZE) {
                        tableViewHeader.setCursor(Cursor.RESIZE_EAST);
                    } else {
                        tableViewHeader.setCursor(null);
                    }
                } else {
                    tableViewHeader.setCursor(null);
                }
            }
        }

//        return consumed;
        return false;
    }

    private int getHeaderX(int headerIndex) {
        int cellX = 0;

        for (int i = 0; i < headerIndex; i++) {
            cellX += (headerWidths.get(i) + 1);
        }

        return cellX;
    }

    private int getHeaderBoundaries(int headerIndex) {
        int cellX = -1;
        for (int i = 0; i <= headerIndex; i++) {
            cellX += (headerWidths.get(i) + 1);
        }
        return cellX;
    }

    public Color getSelectBackgroundColor() {
        return selectBackgroundColor;
    }

    public void setSelectBackgroundColor(Color selectBackgroundColor) {
        this.selectBackgroundColor = selectBackgroundColor;
        repaintComponent();
    }

    public void setSelectBackgroundColor(String selectBackgroundColor) {
        if (selectBackgroundColor == null) {
            throw new IllegalArgumentException("selectBackgroundColor is null.");
        }
        setSelectBackgroundColor(GraphicsUtilities.decodeColor(selectBackgroundColor));
    }

    public Color getSelectVerticalGridColor() {
        return selectVerticalGridColor;
    }

    public void setSelectVerticalGridColor(Color selectVerticalGridColor) {
        this.selectVerticalGridColor = selectVerticalGridColor;
    }

    public void setSelectVerticalGridColor(String selectVerticalGridColor) {
        if (selectVerticalGridColor == null) {
            throw new IllegalArgumentException("selectVerticalGridColor is null.");
        }
        setSelectVerticalGridColor(GraphicsUtilities.decodeColor(selectVerticalGridColor));
    }

    public Color getVerticalGridColor() {
        return verticalGridColor;
    }

    public void setVerticalGridColor(Color verticalGridColor) {
        this.verticalGridColor = verticalGridColor;
    }

    public void setVerticalGridColor(String verticalGridColor) {
        if (verticalGridColor == null) {
            throw new IllegalArgumentException("verticalGridColor is null.");
        }
        setVerticalGridColor(GraphicsUtilities.decodeColor(verticalGridColor));
    }

    public Color getSelectRangeGridColor() {
        return selectRangeGridColor;
    }

    public void setSelectRangeGridColor(Color selectRangeGridColor) {
        this.selectRangeGridColor = selectRangeGridColor;
    }

    public void setSelectRangeGridColor(String selectRangeGridColor) {
        if (selectRangeGridColor == null) {
            throw new IllegalArgumentException("selectRangeGridColor is null.");
        }
        setSelectRangeGridColor(GraphicsUtilities.decodeColor(selectRangeGridColor));
    }

    public boolean isColumnsResizable() {
        return columnsResizable;
    }

    public void setColumnsResizable(boolean columnsResizable) {
        this.columnsResizable = columnsResizable;
    }

    @Override
    public void install(org.apache.pivot.wtk.Component component) {
        super.install(component);

        SSTableViewHeader tableViewHeader = (SSTableViewHeader) component;
        tableViewHeader.getTableViewHeaderListeners().add(this);

        SSTableView tableView = tableViewHeader.getTableView();
        if (tableView != null) {
            tableView.getTableViewColumnListeners().add(this);
            tableView.getTableViewSortListeners().add(this);
        }
    }

    //-------------------------------------------------------------------------------------------

    @Override
    public int getPreferredWidth(int height) {
        int preferredWidth = 0;

        SSTableViewHeader tableViewHeader = (SSTableViewHeader) getComponent();
        SSTableView tableView = tableViewHeader.getTableView();

        if (tableView != null) {
            preferredWidth = SSTableViewSkin.getPreferredWidth(tableView,
            includeTrailingVerticalGridLine);
        }

        return preferredWidth;
    }

    @Override
    public int getPreferredHeight(int width) {
        int preferredHeight = 0;

        SSTableViewHeader tableViewHeader = (SSTableViewHeader) getComponent();
        SSTableView tableView = tableViewHeader.getTableView();

        if (tableView != null) {
            SSTableView.ColumnSequence columns = tableView.getColumns();

            for (int i = 0, n = columns.getLength(); i < n; i++) {
                SSTableView.Column column = columns.get(i);
                SSTableView.HeaderDataRenderer headerDataRenderer = column.getHeaderDataRenderer();
                headerDataRenderer.render(column.getHeaderData(), i, tableViewHeader, column.getName(), false);
                preferredHeight = Math.max(preferredHeight, headerDataRenderer.getPreferredHeight(-1));
            }

            // Include the bottom border
            preferredHeight++;
        }

        return preferredHeight;
    }

    @Override
    public Dimensions getPreferredSize() {
        return new Dimensions(getPreferredWidth(-1), getPreferredHeight(-1));
    }

    @Override
    public boolean isFocusable() {
        return false;
    }

    public Font getFont() {
        return font;
    }

    public void setFont(Font font) {
        if (font == null) {
            throw new IllegalArgumentException("font is null.");
        }

        this.font = font;
        invalidateComponent();
    }

    public final void setFont(String font) {
        if (font == null) {
            throw new IllegalArgumentException("font is null.");
        }

        setFont(decodeFont(font));
    }

    public final void setFont(Dictionary<String, ?> font) {
        if (font == null) {
            throw new IllegalArgumentException("font is null.");
        }

        setFont(Theme.deriveFont(font));
    }

    public Color getColor() {
        return color;
    }

    public void setColor(Color color) {
        if (color == null) {
            throw new IllegalArgumentException("color is null.");
        }

        this.color = color;
        repaintComponent();
    }

    public final void setColor(String color) {
        if (color == null) {
            throw new IllegalArgumentException("color is null.");
        }

        setColor(GraphicsUtilities.decodeColor(color));
    }

    public final void setColor(int color) {
        TerraTheme theme = (TerraTheme) Theme.getTheme();
        setColor(theme.getColor(color));
    }

    public Color getDisabledColor() {
        return disabledColor;
    }

    public void setDisabledColor(Color disabledColor) {
        if (disabledColor == null) {
            throw new IllegalArgumentException("disabledColor is null.");
        }

        this.disabledColor = disabledColor;
        repaintComponent();
    }

    public final void setDisabledColor(String disabledColor) {
        if (disabledColor == null) {
            throw new IllegalArgumentException("disabledColor is null.");
        }

        setDisabledColor(GraphicsUtilities.decodeColor(disabledColor));
    }

    public final void setDisabledColor(int disabledColor) {
        TerraTheme theme = (TerraTheme) Theme.getTheme();
        setDisabledColor(theme.getColor(disabledColor));
    }

    public Color getBackgroundColor() {
        return backgroundColor;
    }

    public void setBackgroundColor(Color backgroundColor) {
        if (backgroundColor == null) {
            throw new IllegalArgumentException("backgroundColor is null.");
        }

        this.backgroundColor = backgroundColor;
        bevelColor = TerraTheme.brighten(backgroundColor);
        pressedBevelColor = TerraTheme.darken(backgroundColor);
        repaintComponent();
    }

    public final void setBackgroundColor(String backgroundColor) {
        if (backgroundColor == null) {
            throw new IllegalArgumentException("backgroundColor is null.");
        }

        setBackgroundColor(GraphicsUtilities.decodeColor(backgroundColor));
    }

    public final void setBackgroundColor(int backgroundColor) {
        TerraTheme theme = (TerraTheme) Theme.getTheme();
        setBackgroundColor(theme.getColor(backgroundColor));
    }

    public Color getDisabledBackgroundColor() {
        return disabledBackgroundColor;
    }

    public void setDisabledBackgroundColor(Color disabledBackgroundColor) {
        if (disabledBackgroundColor == null) {
            throw new IllegalArgumentException("disabledBackgroundColor is null.");
        }

        this.disabledBackgroundColor = disabledBackgroundColor;
        disabledBevelColor = disabledBackgroundColor;
        repaintComponent();
    }

    public final void setDisabledBackgroundColor(String disabledBackgroundColor) {
        if (disabledBackgroundColor == null) {
            throw new IllegalArgumentException("disabledBackgroundColor is null.");
        }

        setDisabledBackgroundColor(GraphicsUtilities.decodeColor(disabledBackgroundColor));
    }

    public final void setDisabledBackgroundColor(int disabledBackgroundColor) {
        TerraTheme theme = (TerraTheme) Theme.getTheme();
        setDisabledBackgroundColor(theme.getColor(disabledBackgroundColor));
    }

    public Color getBorderColor() {
        return borderColor;
    }

    public void setBorderColor(Color borderColor) {
        if (borderColor == null) {
            throw new IllegalArgumentException("borderColor is null.");
        }

        this.borderColor = borderColor;
        repaintComponent();
    }

    public final void setBorderColor(String borderColor) {
        if (borderColor == null) {
            throw new IllegalArgumentException("borderColor is null.");
        }

        setBorderColor(GraphicsUtilities.decodeColor(borderColor));
    }

    public final void setBorderColor(int borderColor) {
        TerraTheme theme = (TerraTheme) Theme.getTheme();
        setBorderColor(theme.getColor(borderColor));
    }

    public Color getDisabledBorderColor() {
        return disabledBorderColor;
    }

    public void setDisabledBorderColor(Color disabledBorderColor) {
        if (disabledBorderColor == null) {
            throw new IllegalArgumentException("disabledBorderColor is null.");
        }

        this.disabledBorderColor = disabledBorderColor;
        repaintComponent();
    }

    public final void setDisabledBorderColor(String disabledBorderColor) {
        if (disabledBorderColor == null) {
            throw new IllegalArgumentException("disabledBorderColor is null.");
        }

        setDisabledBorderColor(GraphicsUtilities.decodeColor(disabledBorderColor));
    }

    public final void setDisabledBorderColor(int disabledBorderColor) {
        TerraTheme theme = (TerraTheme) Theme.getTheme();
        setDisabledBorderColor(theme.getColor(disabledBorderColor));
    }

    public boolean getHeadersPressable() {
        return headersPressable;
    }

    public boolean getColumnsResizable() {
        return columnsResizable;
    }

    public boolean getIncludeTrailingVerticalGridLine() {
        return includeTrailingVerticalGridLine;
    }

    public void setIncludeTrailingVerticalGridLine(boolean includeTrailingVerticalGridLine) {
        this.includeTrailingVerticalGridLine = includeTrailingVerticalGridLine;
        invalidateComponent();
    }

    // Table view header events
    @Override
    public void tableViewChanged(SSTableViewHeader tableViewHeader,
                                 SSTableView previousTableView) {
        if (previousTableView != null) {
            previousTableView.getTableViewColumnListeners().remove(this);
            previousTableView.getTableViewSortListeners().remove(this);
        }

        SSTableView tableView = tableViewHeader.getTableView();
        if (tableView != null) {
            tableView.getTableViewColumnListeners().add(this);
            tableView.getTableViewSortListeners().add(this);
        }

        invalidateComponent();
    }

    @Override
    public void sortModeChanged(SSTableViewHeader tableViewHeader, SSTableViewHeader.SortMode previousSortMode) {
        // No-op
    }

    // Table view column events
    @Override
    public void columnInserted(SSTableView tableView, int index) {
        invalidateComponent();
    }

    @Override
    public void columnsRemoved(SSTableView tableView, int index, Sequence<SSTableView.Column> columns) {
        invalidateComponent();
    }

    @Override
    public void columnNameChanged(SSTableView.Column column, String previousName) {
        // No-op
    }

    @Override
    public void columnHeaderDataChanged(SSTableView.Column column, Object previousHeaderData) {
        invalidateComponent();
    }

    @Override
    public void columnHeaderDataRendererChanged(SSTableView.Column column,
                                                SSTableView.HeaderDataRenderer previousHeaderDataRenderer) {
        invalidateComponent();
    }

    @Override
    public void columnWidthChanged(SSTableView.Column column, int previousWidth, boolean previousRelative) {
        invalidateComponent();
    }

    @Override
    public void columnWidthLimitsChanged(SSTableView.Column column, int previousMinimumWidth, int previousMaximumWidth) {
        invalidateComponent();
    }

    @Override
    public void columnFilterChanged(SSTableView.Column column, Object previousFilter) {
        // No-op
    }

    @Override
    public void columnCellRendererChanged(SSTableView.Column column, SSTableView.CellRenderer previousCellRenderer) {
        // No-op
    }

    // Table view sort events
    public void sortAdded(SSTableView tableView, String columnName) {
        repaintComponent();
    }

    public void sortUpdated(SSTableView tableView, String columnName,
                            SortDirection previousSortDirection) {
        repaintComponent();
    }

    public void sortRemoved(SSTableView tableView, String columnName,
                            SortDirection sortDirection) {
        repaintComponent();
    }

    public void sortChanged(SSTableView tableView) {
        repaintComponent();
    }

}
