/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.ss;

import jp.pegasus.pivot.skin.SSDisplaySkin;
import org.apache.pivot.wtk.ApplicationContext;
import org.apache.pivot.wtk.Skin;
import org.apache.pivot.wtk.Window;

import java.awt.*;

public class SSDisplay extends org.apache.pivot.wtk.Container {
    private ApplicationContext.DisplayHost displayHost;

    public SSDisplay(ApplicationContext.DisplayHost displayHost) {
        this.displayHost = displayHost;
        super.setSkin(new SSDisplaySkin());
    }

    public ApplicationContext.DisplayHost getDisplayHost() {
        return displayHost;
    }

    public java.awt.Window getHostWindow() {
        java.awt.Container parent = displayHost.getParent();
        while (parent != null
        && !(parent instanceof java.awt.Window)) {
            parent = parent.getParent();
        }

        if (parent == null) {
            throw new IllegalArgumentException("SSWindow does not have a native host.");
        }

        return (java.awt.Window) parent;
    }

    @Override
    protected void setSkin(Skin skin) {
        throw new UnsupportedOperationException("Can't replace SSDisplay skin.");
    }

    @Override
    protected void setParent(org.apache.pivot.wtk.Container parent) {
        throw new UnsupportedOperationException("SSDisplay can't have a parent.");
    }

    @Override
    public void setLocation(int x, int y) {
        throw new UnsupportedOperationException("Can't change the location of the display.");
    }

    @Override
    public void setVisible(boolean visible) {
        throw new UnsupportedOperationException("Can't change the visibility of the display.");
    }

    @Override
    public void repaint(int x, int y, int width, int height, boolean immediate) {
        if (immediate) {
            Graphics2D graphics = (Graphics2D) displayHost.getGraphics();

            // If the display host has been made non-displayable (as will
            // happen when the native peer closes), graphics will be null
            if (graphics != null) {
                double scale = displayHost.getScale();
                if (scale == 1) {
                    graphics.clipRect(x, y, width, height);
                } else {
                    graphics.clipRect((int) Math.floor(x * scale), (int) Math.floor(y * scale),
                    (int) Math.ceil(width * scale) + 1, (int) Math.ceil(height * scale) + 1);
                }

                displayHost.paint(graphics);
                graphics.dispose();
            }
        } else {
            displayHost.repaint(x, y, width, height);
        }
    }

    @Override
    public void insert(org.apache.pivot.wtk.Component component, int index) {
        if (!(component instanceof Window)) {
            throw new IllegalArgumentException("component must be an instance "
            + "of " + Window.class);
        }

        super.insert(component, index);
    }

    @Override
    protected void descendantAdded(org.apache.pivot.wtk.Component descendant) {
        super.descendantAdded(descendant);

        String automationID = descendant.getAutomationID();

        if (automationID != null) {
            SSAutomation.add(automationID, descendant);
        }
    }

    @Override
    protected void descendantRemoved(org.apache.pivot.wtk.Component descendant) {
        super.descendantRemoved(descendant);

        String automationID = descendant.getAutomationID();

        if (automationID != null) {
            SSAutomation.remove(automationID);
        }
    }

    @Override
    public org.apache.pivot.wtk.FocusTraversalPolicy getFocusTraversalPolicy() {
        return null;
    }
}
