/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.ss;

import jp.pegasus.pivot.listener.SSFrameListener;
import org.apache.pivot.collections.LinkedList;
import org.apache.pivot.collections.Sequence;
import org.apache.pivot.util.ListenerList;
import org.apache.pivot.wtk.*;

public class SSFrame extends Window {
    private static class SSFrameListenerList extends ListenerList<SSFrameListener>
    implements SSFrameListener {
        @Override
        public void menuBarChanged(SSFrame frame, MenuBar previousMenuBar) {
            for (SSFrameListener listener : this) {
                listener.menuBarChanged(frame, previousMenuBar);
            }
        }
    }

    private MenuBar menuBar = null;

    private SSFrameListenerList frameListeners = new SSFrameListenerList();

    public SSFrame() {
        this(null, null);
    }

    public SSFrame(String title) {
        this(title, null);
    }

    public SSFrame(Component content) {
        this(null, content);
    }

    public SSFrame(String title, Component content) {
        super(content);

        setTitle(title);
        installSkin(SSFrame.class);
    }

    public MenuBar getMenuBar() {
        return menuBar;
    }

    public void setMenuBar(MenuBar menuBar) {
        MenuBar previousMenuBar = this.menuBar;

        if (previousMenuBar != menuBar) {
            this.menuBar = menuBar;

            if (previousMenuBar != null) {
                remove(previousMenuBar);
            }

            if (menuBar != null) {
                add(menuBar);
            }

            frameListeners.menuBarChanged(this, previousMenuBar);
        }
    }

    @Override
    public void moveToFront() {
        super.moveToFront();
        requestActive();
    }

    @Override
    protected void descendantGainedFocus(Component descendant, Component previousFocusedComponent) {
        if (menuBar != null) {
            LinkedList<Component> path = new LinkedList<Component>();

            Component ancestor = descendant;
            while (!(ancestor instanceof Display)) {
                path.insert(ancestor, 0);
                ancestor = ancestor.getParent();
            }

            for (Component component : path) {
                MenuHandler menuHandler = component.getMenuHandler();

                if (menuHandler != null) {
                    menuHandler.configureMenuBar(component, menuBar);
                }
            }
        }

        super.descendantGainedFocus(descendant, previousFocusedComponent);
    }

    @Override
    protected void descendantLostFocus(Component descendant) {
        if (menuBar != null) {
            LinkedList<Component> path = new LinkedList<Component>();

            Component ancestor = descendant;
            while (!(ancestor instanceof Display)) {
                path.insert(ancestor, 0);
                ancestor = ancestor.getParent();
            }

            for (Component component : path) {
                MenuHandler menuHandler = component.getMenuHandler();

                if (menuHandler != null) {
                    menuHandler.cleanupMenuBar(component, menuBar);
                }
            }
        }

        super.descendantLostFocus(descendant);
    }

    @Override
    public Sequence<Component> remove(int index, int count) {
        for (int i = index, n = index + count; i < n; i++) {
            Component component = get(i);
            if (component == menuBar) {
                throw new UnsupportedOperationException();
            }
        }

        // Call the base method to remove the components
        return super.remove(index, count);
    }

    public ListenerList<SSFrameListener> getFrameListeners() {
        return frameListeners;
    }
}
