/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.ss;

import jp.pegasus.pivot.content.SSButtonDataRenderer;
import org.apache.pivot.beans.DefaultProperty;
import org.apache.pivot.wtk.Button;

@DefaultProperty("buttonData")
public class SSPushButton extends Button {
    private static final Button.DataRenderer DEFAULT_DATA_RENDERER = new SSButtonDataRenderer();

    public SSPushButton() {
        this(false, null);
    }

    public SSPushButton(boolean toggleButton) {
        this(toggleButton, null);
    }

    public SSPushButton(Object buttonData) {
        this(false, buttonData);
    }

    public SSPushButton(boolean toggleButton, Object buttonData) {
        super(buttonData);

        setToggleButton(toggleButton);
        setDataRenderer(DEFAULT_DATA_RENDERER);

        installSkin(SSPushButton.class);
    }

    @Override
    public void press() {
        if (isToggleButton()) {
            State state = getState();

            if (getButtonGroup() == null) {
                if (state == State.SELECTED) {
                    setState(State.UNSELECTED);
                } else if (state == State.UNSELECTED) {
                    setState(isTriState() ? State.MIXED : State.SELECTED);
                } else {
                    setState(State.SELECTED);
                }
            } else {
                setSelected(true);
            }
        }

        super.press();
    }
}
