/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.ss;

import org.apache.pivot.util.ListenerList;
import org.apache.pivot.wtk.*;

public class SSScrollBar extends ScrollBar {

    private Orientation orientation;
    private int start = 0;
    private int end = 100;
    private int extent = 1;
    private int value = 0;
    private int unitIncrement = 1;
    private int blockIncrement = 1;

    public SSScrollBar() {
        this(Orientation.HORIZONTAL);
    }

    public SSScrollBar(Orientation orientation) {
        if (orientation == null) {
            throw new IllegalArgumentException("orientation is null");
        }

        this.orientation = orientation;
        installSkin(SSScrollBar.class);
    }

    @Override
    public Orientation getOrientation() {
        return orientation;
    }

    @Override
    public void setOrientation(Orientation orientation) {
        super.setOrientation(orientation);
    }

    @Override
    public int getStart() {
        return start;
    }

    @Override
    public void setStart(int start) {
        setScope(start, end, extent);
    }

    @Override
    public int getEnd() {
        return end;
    }

    @Override
    public void setEnd(int end) {
        setScope(start, end, extent);
    }

    @Override
    public Span getRange() {
        return new Span(start, end);
    }

    @Override
    public void setRange(int start, int end) {
        setScope(start, end, extent);
    }

    @Override
    public int getExtent() {
        return extent;
    }

    @Override
    public void setExtent(int extent) {
        setScope(start, end, extent);
    }

    @Override
    public Scope getScope() {
        return new Scope(start, end, extent);
    }

    @Override
    public void setScope(int start, int end, int extent) {
        int previousStart = this.start;
        int previousEnd = this.end;
        int previousExtent = this.extent;

        if (start != previousStart || end != previousEnd || extent != previousExtent) {
            if (start > value) {
                throw new IllegalArgumentException(String.format
                ("start (%d) is greater than value (%d)", start, value));
            }

            if (extent < 0) {
                throw new IllegalArgumentException(String.format
                ("extent (%d) is negative", extent));
            }

            if (end < value + extent) {
                throw new IllegalArgumentException(String.format
                ("end (%d) is less than value (%d) + extent (%d)", end, value, extent));
            }

            this.start = start;
            this.end = end;
            this.extent = extent;

//            scrollBarListeners.scopeChanged(this, previousStart, previousEnd,
//                previousExtent);
        }
    }

    @Override
    public int getValue() {
        return value;
    }

    @Override
    public void setValue(int value) {
        int previousValue = this.value;

        if (value != previousValue) {
            if (value < start) {
                throw new IllegalArgumentException(String.format
                ("value (%d) is less than start (%d)", value, start));
            }

            if (value + extent > end) {
                throw new IllegalArgumentException(String.format
                ("value (%d) + extent (%d) is greater than end (%d)", value, extent, end));
            }

            this.value = value;

//            scrollBarValueListeners.valueChanged(this, previousValue);
        }
    }

    @Override
    public int getUnitIncrement() {
        return unitIncrement;
    }

    @Override
    public void setUnitIncrement(int unitIncrement) {
        if (unitIncrement < 0) {
            throw new IllegalArgumentException("unitIncrement is negative");
        }

        int previousUnitIncrement = this.unitIncrement;

        if (unitIncrement != previousUnitIncrement) {
            this.unitIncrement = unitIncrement;
//            scrollBarListeners.unitIncrementChanged(this, previousUnitIncrement);
        }
    }

    @Override
    public int getBlockIncrement() {
        return blockIncrement;
    }

    @Override
    public void setBlockIncrement(int blockIncrement) {
        if (blockIncrement < 0) {
            throw new IllegalArgumentException("blockIncrement is negative");
        }

        int previousBlockIncrement = this.blockIncrement;

        if (blockIncrement != previousBlockIncrement) {
            this.blockIncrement = blockIncrement;

//            scrollBarListeners.blockIncrementChanged(this, previousBlockIncrement);
        }
    }

    @Override
    public ListenerList<ScrollBarListener> getScrollBarListeners() {
        return super.getScrollBarListeners();
    }

    @Override
    public ListenerList<ScrollBarValueListener> getScrollBarValueListeners() {
        return super.getScrollBarValueListeners();
    }
}
