/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.ss.menu;

import org.apache.pivot.wtk.Keyboard;
import org.apache.pivot.wtk.content.ButtonData;
import org.apache.pivot.wtk.media.Image;

public class SSMenuItemData extends ButtonData {
    private Keyboard.KeyStroke keyboardShortcut = null;

    /**
     * Constructor.
     */
    public SSMenuItemData() {
        this(null, null, null);
    }

    /**
     * Constructor.
     *
     * @param icon The icon to display in the menu item.
     */
    public SSMenuItemData(Image icon) {
        this(icon, null, null);
    }

    /**
     * Constructor.
     *
     * @param text The text to display in the menu item.
     */
    public SSMenuItemData(String text) {
        this(null, text, null);
    }

    /**
     * Constructor.
     *
     * @param icon The icon to display in the menu item.
     * @param text The text to display in the menu item.
     */
    public SSMenuItemData(Image icon, String text) {
        this(icon, text, null);
    }

    /**
     * Constructor.
     *
     * @param icon             The icon to display in the menu item.
     * @param text             The text to display in the menu item.
     * @param keyboardShortcut The keyboard shortcut associated with this menu item.
     */
    public SSMenuItemData(Image icon, String text, Keyboard.KeyStroke keyboardShortcut) {
        super(icon, text);

        this.keyboardShortcut = keyboardShortcut;
    }

    public Keyboard.KeyStroke getKeyboardShortcut() {
        return keyboardShortcut;
    }

    public void setKeyboardShortcut(Keyboard.KeyStroke keyboardShortcut) {
        this.keyboardShortcut = keyboardShortcut;
    }

    public void setKeyboardShortcut(String keyboardShortcut) {
        setKeyboardShortcut(Keyboard.KeyStroke.decode(keyboardShortcut));
    }
}
