/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.ss.tableview;

import jp.pegasus.log.Logger;
import jp.pegasus.pivot.content.SSTableViewCellRenderer;
import jp.pegasus.pivot.content.SSTableViewHeaderDataRenderer;
import jp.pegasus.pivot.editor.SSTableViewRowEditor;
import jp.pegasus.pivot.listener.tableview.*;
import jp.pegasus.pivot.ss.SSPalette;
import jp.pegasus.pivot.ss.SSRangeSelect;
import org.apache.pivot.beans.DefaultProperty;
import org.apache.pivot.collections.*;
import org.apache.pivot.collections.immutable.ImmutableList;
import org.apache.pivot.json.JSON;
import org.apache.pivot.json.JSONSerializer;
import org.apache.pivot.serialization.SerializationException;
import org.apache.pivot.util.Filter;
import org.apache.pivot.util.ImmutableIterator;
import org.apache.pivot.util.ListenerList;
import org.apache.pivot.wtk.*;

import java.io.IOException;
import java.net.URL;
import java.util.Comparator;
import java.util.Iterator;
import java.util.Locale;

@DefaultProperty("tableData")
public class SSTableView extends Component {

    public enum Type {
        Default, SpreadSheet, RowHeader, Property,
    }

    private Type tableViewType = Type.Default;
    private SSPalette palette = null;

    private SSRangeSelect rangeSelect = new SSRangeSelect();
    private SelectMode selectMode = SelectMode.SINGLE;

    private HashMap<String, SortDirection> sortMap = new HashMap<String, SortDirection>();
    private ArrayList<String> sortList = new ArrayList<String>();
    private SortDictionary sortDictionary = new SortDictionary();

    private Filter<?> disabledRowFilter = null;

    private BindType tableDataBindType = BindType.BOTH;
    private TableDataBindMapping tableDataBindMapping = null;

    private String selectRowKey = null;
    private BindType selectRowBindType = BindType.BOTH;
    private SelectRowBindMapping selectRowBindMapping = null;

    private String selectRowsKey = null;
    private BindType selectRowsBindType = BindType.BOTH;
    private SelectRowBindMapping selectRowsBindMapping = null;

    private List<?> tableData = null;
    private SSTableView columnSource = null;

    private SSTableViewRowEditor rowEditor = null;

    /**
     * int selColIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 48));
     * int selRowIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 32));
     * int curColIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet >> 16));
     * int curRowIndex = (int) (0x000000000000FFFFl & (selectCurrentIndexQuartet));
     */
    private long selectCurrentIndexQuartet = 0x0000000000000000l;

    /**
     * int selectColIndexFr = (int) (0x000000000000FFFFl & (selectIndexQuartet >> 48));
     * int selectRowIndexFr = (int) (0x000000000000FFFFl & (selectIndexQuartet >> 32));
     * int selectColIndexTo = (int) (0x000000000000FFFFl & (selectIndexQuartet >> 16));
     * int selectRowIndexTo = (int) (0x000000000000FFFFl & (selectIndexQuartet));
     */
    private long selectIndexQuartet = 0x0000000000000000l;

    /**
     * int copyColIndexFr = (int) (0x000000000000FFFFl & (copyIndexQuartet >> 48));
     * int copyRowIndexFr = (int) (0x000000000000FFFFl & (copyIndexQuartet >> 32));
     * int copyColIndexTo = (int) (0x000000000000FFFFl & (copyIndexQuartet >> 16));
     * int copyRowIndexTo = (int) (0x000000000000FFFFl & (copyIndexQuartet));
     */
    private long copyIndexQuartet = 0x0000000000000000l;

    /**
     * int cutColIndexFr = (int) (0x000000000000FFFFl & (cutIndexQuartet >> 48));
     * int cutRowIndexFr = (int) (0x000000000000FFFFl & (cutIndexQuartet >> 32));
     * int cutColIndexTo = (int) (0x000000000000FFFFl & (cutIndexQuartet >> 16));
     * int cutRowIndexTo = (int) (0x000000000000FFFFl & (cutIndexQuartet));
     */
    private long cutIndexQuartet = 0x0000000000000000l;

    /**
     * int pasteColIndexFr = (int) (0x000000000000FFFFl & (pasteIndexQuartet >> 48));
     * int pasteRowIndexFr = (int) (0x000000000000FFFFl & (pasteIndexQuartet >> 32));
     * int pasteColIndexTo = (int) (0x000000000000FFFFl & (pasteIndexQuartet >> 16));
     * int pasteRowIndexTo = (int) (0x000000000000FFFFl & (pasteIndexQuartet));
     */
    private long pasteIndexQuartet = 0x0000000000000000l;

    private int[] selectRange = {-1, -1, -1, -1};
    private int[] copyRange = {-1, -1, -1, -1};
    private int[] cutRange = {-1, -1, -1, -1};
    private int[] pasteRange = {-1, -1, -1, -1};

    /**
     * int cellColIndex = 0x0000FFFF & (selectCellIndexPair >> 16);
     * int cellRowIndex = 0x0000FFFF & (selectCellIndexPair);
     */
    private int selectCellIndexPair = 0x00000000;

    /**
     * int preColIndex = 0x0000FFFF & (preCellIndexPair >> 16);
     * int preRowIndex = 0x0000FFFF & (preCellIndexPair);
     */
    private int preCellIndexPair = 0x00000000;
//    private int preColumnIndex = -1;
//    private int preRowIndex = -1;


    /**
     * 0x0000000000000000
     * ->fedcba9876543210
     * <p/>
     * 0: isSelect
     * 1: isCopy
     * 2: isCutOut
     * 3: isPaste
     * 4: isHeaderPress
     * 5: isMouseDown
     * 6: isHighlighted
     * 7: isDisabled
     */
    //private long statusBits = 0x0000000000000000l;
    // status
    //---------------------------------------------------
    private boolean isSelect = false;
    private boolean isCopied = false;
    private boolean isCutOut = false;
    private boolean isPasted = false;
    private boolean isHeaderPressed = false;
    private boolean isMouseDown = false;
    //---------------------------------------------------

    private String tableDataKey = null;

    public static final class Column {
        private SSTableView tableView = null;
        private String name = null;
        private Object headerData = null;
        private HeaderDataRenderer headerDataRenderer = DEFAULT_HEADER_DATA_RENDERER;
        private int width = 0;
        private int minimumWidth = 0;
        private int maximumWidth = Integer.MAX_VALUE;
        private boolean relative = false;
        private Object filter = null;
        private CellRenderer cellRenderer = DEFAULT_CELL_RENDERER;

        private static final HeaderDataRenderer DEFAULT_HEADER_DATA_RENDERER = new SSTableViewHeaderDataRenderer();
        private static final CellRenderer DEFAULT_CELL_RENDERER = new SSTableViewCellRenderer();

        /**
         * Default column width.
         */
        public static final int DEFAULT_WIDTH = 100;

        /**
         * Creates an empty column.
         */
        public Column() {
            this(null, null, DEFAULT_WIDTH, false);
        }

        /**
         * Creates a new column with no header data and a fixed default width.
         *
         * @param name The column name.
         */
        public Column(String name) {
            this(name, null, DEFAULT_WIDTH, false);
        }

        /**
         * Creates a new column with a fixed default width.
         *
         * @param name       The column name.
         * @param headerData The column header data.
         */
        public Column(String name, Object headerData) {
            this(name, headerData, DEFAULT_WIDTH, false);
        }

        /**
         * Creates a new column with a fixed width.
         *
         * @param name       The column name.
         * @param headerData The column header data.
         * @param width      The width of the column.
         */
        public Column(String name, Object headerData, int width) {
            this(name, headerData, width, false);
        }

        /**
         * Creates a new column.
         *
         * @param name       The column name.
         * @param headerData The column header data.
         * @param width      The width of the column.
         * @param relative   If <tt>true</tt>, specifies a relative column width; otherwise,
         *                   specifies a fixed column width.
         */
        public Column(String name, Object headerData, int width, boolean relative) {
            setName(name);
            setHeaderData(headerData);
            setWidth(width, relative);
        }

        /**
         * Returns the table view with which this column is associated.
         *
         * @return The column's table view, or <tt>null</tt> if the column does not
         *         currently belong to a table.
         */
        public SSTableView getTableView() {
            return tableView;
        }

        /**
         * Returns the column name.
         *
         * @return The column name.
         */
        public String getName() {
            return name;
        }

        /**
         * Sets the column name.
         *
         * @param name The column name.
         */
        public void setName(String name) {
            String previousName = this.name;

            if (previousName != name) {
                this.name = name;
                if (tableView != null) {
                    tableView.tableViewColumnListeners.columnNameChanged(
                    this,
                    previousName);
                }
            }
        }

        /**
         * Returns the column header data.
         *
         * @return The column header data, or <tt>null</tt> if the column has no
         *         header data.
         */
        public Object getHeaderData() {
            return headerData;
        }

        /**
         * Sets the column header data.
         *
         * @param headerData The column header data, or <tt>null</tt> for no header data.
         */
        public void setHeaderData(Object headerData) {
            Object previousHeaderData = this.headerData;
            if (previousHeaderData != headerData) {
                this.headerData = headerData;
                if (tableView != null) {
                    tableView.tableViewColumnListeners.columnHeaderDataChanged(
                    this,
                    previousHeaderData);
                }
            }
        }

        /**
         * Returns the column header data renderer.
         */
        public HeaderDataRenderer getHeaderDataRenderer() {
            return headerDataRenderer;
        }

        /**
         * Sets the column header data renderer.
         *
         * @param headerDataRenderer
         */
        public void setHeaderDataRenderer(HeaderDataRenderer headerDataRenderer) {
            if (headerDataRenderer == null) {
                throw new IllegalArgumentException("headerDataRenderer is null.");
            }

            HeaderDataRenderer previousHeaderDataRenderer = this.headerDataRenderer;
            if (previousHeaderDataRenderer != headerDataRenderer) {
                this.headerDataRenderer = headerDataRenderer;

                if (tableView != null) {
                    tableView.tableViewColumnListeners.columnHeaderDataRendererChanged(
                    this,
                    previousHeaderDataRenderer);
                }
            }
        }

        /**
         * Returns the column width.
         *
         * @return The width of the column.
         */
        public int getWidth() {
            return width;
        }

        /**
         * Returns the relative flag.
         *
         * @return <tt>true</tt> if the column width is relative, <tt>false</tt> if it
         *         is fixed.
         */
        public boolean isRelative() {
            return relative;
        }

        /**
         * Set the column width.
         *
         * @param width The absolute width of the column.
         */
        public void setWidth(int width) {
            setWidth(width, false);
        }

        /**
         * Set the column width.
         *
         * @param width The encoded width of the row. If the string ends with the '*'
         *              character, it is treated as a relative value. Otherwise, it is
         *              considered an absolute value.
         */
        public void setWidth(String width) {
            boolean relative = false;

            if (width.endsWith("*")) {
                relative = true;
                width = width.substring(0, width.length() - 1);
            }

            setWidth(Integer.parseInt(width), relative);
        }

        /**
         * Sets the column width.
         *
         * @param width    The width of the column.
         * @param relative <tt>true</tt> if the column width is relative, <tt>false</tt> if it
         *                 is fixed.
         */
        public void setWidth(int width, boolean relative) {
            if (width < (relative ? 0 : -1)) {
                throw new IllegalArgumentException();
            }

            int previousWidth = this.width;
            boolean previousRelative = this.relative;

            if (previousWidth != width
            || previousRelative != relative) {
                this.width = width;
                this.relative = relative;

                if (tableView != null) {
                    tableView.tableViewColumnListeners.columnWidthChanged(this,
                    previousWidth, previousRelative);
                }
            }
        }

        /**
         * Gets the minimum and maximum widths to which the column can size.
         */
        public Limits getWidthLimits() {
            return new Limits(minimumWidth, maximumWidth);
        }

        /**
         * Sets the minimum and maximum widths the column can size to.
         *
         * @param minimumWidth Column width cannot be smaller than this size.
         * @param maximumWidth Column width cannot be greater than this size.
         */
        public void setWidthLimits(final int minimumWidth, final int maximumWidth) {
            if (minimumWidth < 0) {
                throw new IllegalArgumentException("Minimum width is negative.");
            }

            if (maximumWidth < minimumWidth) {
                throw new IllegalArgumentException("Maximum width is smaller than minimum width.");
            }

            final int previousMinimumWidth = this.minimumWidth;
            final int previousMaximumWidth = this.maximumWidth;

            if (minimumWidth != previousMinimumWidth
            || maximumWidth != previousMaximumWidth) {
                this.minimumWidth = minimumWidth;
                this.maximumWidth = maximumWidth;

                if (tableView != null) {
                    tableView.tableViewColumnListeners.columnWidthLimitsChanged(this,
                    previousMinimumWidth, previousMaximumWidth);
                }
            }
        }

        /**
         * Sets the minimum and maximum widths to which the column can size.
         *
         * @param widthLimits The new width limits.
         */
        public void setWidthLimits(Limits widthLimits) {
            setWidthLimits(widthLimits.minimum, widthLimits.maximum);
        }

        /**
         * Gets the minimum width the column is allowed to be.
         *
         * @return Minimum column width.
         */
        public int getMinimumWidth() {
            return minimumWidth;
        }

        /**
         * Sets the minimum width the column is allowed to be.
         *
         * @param minimumWidth Minimum column width.
         */
        public void setMinimumWidth(int minimumWidth) {
            setWidthLimits(minimumWidth, maximumWidth);
        }

        /**
         * Get the maximum width the column is allowed to be.
         *
         * @return Maximum column width.
         */
        public int getMaximumWidth() {
            return maximumWidth;
        }

        /**
         * Set the maximum width the column is allowed to be.
         *
         * @param maximumWidth Maximum column width.
         */
        public void setMaximumWidth(int maximumWidth) {
            setWidthLimits(minimumWidth, maximumWidth);
        }

        /**
         * Returns the column's filter.
         *
         * @return The column's filter, or <tt>null</tt> if the column does not have
         *         a filter.
         */
        public Object getFilter() {
            return filter;
        }

        /**
         * Sets the column's filter.
         *
         * @param filter The column's filter, or <tt>null</tt> for no filter.
         */
        public void setFilter(Object filter) {
            Object previousFilter = this.filter;

            if (previousFilter != filter) {
                this.filter = filter;

                if (tableView != null) {
                    tableView.tableViewColumnListeners.columnFilterChanged(this,
                    previousFilter);
                }
            }
        }

        /**
         * Returns the column's cell renderer.
         *
         * @return The cell renderer that is used to draw the contents of this column.
         */
        public CellRenderer getCellRenderer() {
            return cellRenderer;
        }

        /**
         * Sets the column's cell renderer.
         *
         * @param cellRenderer The cell renderer that is used to draw the contents of this column.
         */
        public void setCellRenderer(CellRenderer cellRenderer) {
            if (cellRenderer == null) {
                throw new IllegalArgumentException("cellRenderer is null.");
            }

            CellRenderer previousCellRenderer = this.cellRenderer;

            if (previousCellRenderer != cellRenderer) {
                this.cellRenderer = cellRenderer;

                if (tableView != null) {
                    tableView.tableViewColumnListeners.columnCellRendererChanged(this,
                    previousCellRenderer);
                }
            }
        }
    }

    public enum SelectMode {
        /**
         * Select is disabled.
         */
        NONE,

        /**
         * A single index may be select at a time.
         */
        SINGLE,

        /**
         * Multiple indexes may be concurrently select.
         */
        MULTI
    }

    public interface CellRenderer extends Renderer {
        /**
         * Prepares the renderer for layout or paint.
         *
         * @param row         The row to render, or <tt>null</tt> if called to calculate preferred height for
         *                    skins that assume a fixed renderer height.
         * @param rowIndex    The index of the row being rendered, or <tt>-1</tt> if <tt>value</tt>
         *                    is <tt>null</tt>.
         * @param columnIndex The index of the column being rendered.
         * @param tableView   The host component.
         * @param columnName  The name of the column being rendered.
         * @param isSelect    If <tt>true</tt>, the cell is select.
         * @param isCopied    If <tt>true</tt>, the cell is copied.
         * @param isCutOut    If <tt>true</tt>, the cell is cutout.
         * @param isPasted    If <tt>true</tt>, the cell is pasted.
         * @param highlighted If <tt>true</tt>, the cell is highlighted.
         * @param disabled    If <tt>true</tt>, the cell is disabled.
         */
        public void render(Object row, int rowIndex, int columnIndex, SSTableView tableView,
                           String columnName, boolean isSelect, boolean isCopied, boolean isCutOut, boolean isPasted, boolean highlighted, boolean disabled);

        /**
         * Converts table view cell data to a string representation.
         *
         * @param row
         * @param columnName
         * @return The cell data's string representation, or <tt>null</tt> if the data does not
         *         have a string representation.
         *         <p/>
         *         Note that this method may be called often during keyboard navigation, so
         *         implementations should avoid unnecessary string allocations.
         */
        public String toString(Object row, String columnName);
    }

    public interface HeaderDataRenderer extends Renderer {
        /**
         * Prepares the renderer for layout or paint.
         *
         * @param data            The data to render, or <tt>null</tt> if called to calculate preferred
         *                        height for skins that assume a fixed renderer height.
         * @param columnIndex     The index of the column being rendered.
         * @param tableViewHeader The host component.
         * @param columnName      The name of the column being rendered.
         * @param highlighted     If <tt>true</tt>, the item is highlighted.
         */
        public void render(Object data, int columnIndex, SSTableViewHeader tableViewHeader,
                           String columnName, boolean highlighted);

        /**
         * Converts table view header data to a string representation.
         *
         * @param item
         * @return The data's string representation, or <tt>null</tt> if the data does not
         *         have a string representation.
         *         <p/>
         *         Note that this method may be called often during keyboard navigation, so
         *         implementations should avoid unnecessary string allocations.
         */
        public String toString(Object item);
    }

    public interface RowEditor {
        /**
         * Called to begin editing a table row.
         *
         * @param tableView
         * @param rowIndex
         * @param columnIndex
         */
        public void beginEdit(SSTableView tableView, int rowIndex, int columnIndex);

        /**
         * Terminates an edit operation.
         *
         * @param result <tt>true</tt> to perform the edit; <tt>false</tt> to cancel it.
         */
        public void endEdit(boolean result);

        /**
         * Tests whether an edit is currently in progress.
         */
        public boolean isEditing();
    }

    public interface TableDataBindMapping {
        /**
         * Converts a context value to table data.
         *
         * @param value
         */
        public List<?> toTableData(Object value);

        /**
         * Converts table data to a context value.
         *
         * @param tableData
         */
        public Object valueOf(List<?> tableData);
    }

    public interface SelectRowBindMapping {
        /**
         * Returns the index of the row in the source list.
         *
         * @param tableData The source table data.
         * @param value     The value to locate.
         * @return The index of first occurrence of the value if it exists in the list;
         *         <tt>-1</tt>, otherwise.
         */
        public int indexOf(List<?> tableData, Object value);

        /**
         * Retrieves the value at the given index.
         *
         * @param tableData The source table data.
         * @param index     The index of the value to retrieve.
         */
        public Object get(List<?> tableData, int index);
    }

    public final class ColumnSequence implements Sequence<Column>, Iterable<Column> {
        @Override
        public int add(Column column) {
            int index = getLength();
            insert(column, index);

            return index;
        }

        @Override
        public void insert(Column column, int index) {
            if (column == null) {
                throw new IllegalArgumentException("column is null.");
            }

            if (column.tableView != null) {
                throw new IllegalArgumentException("column is already in use by another table view.");
            }

            columns.insert(column, index);
            column.tableView = SSTableView.this;

            tableViewColumnListeners.columnInserted(SSTableView.this, index);
        }

        @Override
        public Column update(int index, Column column) {
            throw new UnsupportedOperationException();
        }

        @Override
        public int remove(Column column) {
            int index = indexOf(column);
            if (index != -1) {
                remove(index, 1);
            }

            return index;
        }

        @Override
        public Sequence<Column> remove(int index, int count) {
            Sequence<Column> removed = columns.remove(index, count);

            if (count > 0) {
                for (int i = 0, n = removed.getLength(); i < n; i++) {
                    removed.get(i).tableView = null;
                }

                tableViewColumnListeners.columnsRemoved(SSTableView.this, index, removed);
            }

            return removed;
        }

        @Override
        public Column get(int index) {
            return columns.get(index);
        }

        @Override
        public int indexOf(Column column) {
            return columns.indexOf(column);
        }

        @Override
        public int getLength() {
            return columns.getLength();
        }

        @Override
        public Iterator<Column> iterator() {
            return new ImmutableIterator<Column>(columns.iterator());
        }
    }

    public final class SortDictionary implements Dictionary<String, SortDirection>, Iterable<String> {
        public SortDirection get(String columnName) {
            return sortMap.get(columnName);
        }

        public SortDirection put(String columnName, SortDirection sortDirection) {
            SortDirection previousSortDirection;

            if (sortDirection == null) {
                previousSortDirection = remove(columnName);
            } else {
                boolean update = containsKey(columnName);
                previousSortDirection = sortMap.put(columnName, sortDirection);

                if (update) {
                    tableViewSortListeners.sortUpdated(SSTableView.this, columnName, previousSortDirection);
                } else {
                    sortList.add(columnName);
                    tableViewSortListeners.sortAdded(SSTableView.this, columnName);
                }
            }

            return previousSortDirection;
        }

        public SortDirection remove(String columnName) {
            SortDirection sortDirection = null;

            if (containsKey(columnName)) {
                sortDirection = sortMap.remove(columnName);
                sortList.remove(columnName);
                tableViewSortListeners.sortRemoved(SSTableView.this, columnName, sortDirection);
            }

            return sortDirection;
        }

        public boolean containsKey(String columnName) {
            return sortMap.containsKey(columnName);
        }

        public boolean isEmpty() {
            return sortMap.isEmpty();
        }

        public Dictionary.Pair<String, SortDirection> get(int index) {
            String columnName = sortList.get(index);
            SortDirection sortDirection = sortMap.get(columnName);

            return new Dictionary.Pair<String, SortDirection>(columnName, sortDirection);
        }

        public int getLength() {
            return sortList.getLength();
        }

        public Iterator<String> iterator() {
            return sortList.iterator();
        }
    }

    private static class SSTableViewListenerList extends ListenerList<SSTableViewListener>
    implements SSTableViewListener {
        @Override
        public void tableDataChanged(SSTableView tableView, List<?> previousTableData) {
            for (SSTableViewListener listener : this) {
                listener.tableDataChanged(tableView, previousTableData);
            }
        }

        @Override
        public void columnSourceChanged(SSTableView tableView, SSTableView previousColumnSource) {
            for (SSTableViewListener listener : this) {
                listener.columnSourceChanged(tableView, previousColumnSource);
            }
        }

        @Override
        public void rowEditorChanged(SSTableView tableView,
                                     SSTableView.RowEditor previousRowEditor) {
            for (SSTableViewListener listener : this) {
                listener.rowEditorChanged(tableView, previousRowEditor);
            }
        }

        @Override
        public void selectModeChanged(SSTableView tableView, SelectMode previousSelectMode) {
            for (SSTableViewListener listener : this) {
                listener.selectModeChanged(tableView, previousSelectMode);
            }
        }

        @Override
        public void disabledRowFilterChanged(SSTableView tableView, Filter<?> previousDisabledRowFilter) {
            for (SSTableViewListener listener : this) {
                listener.disabledRowFilterChanged(tableView, previousDisabledRowFilter);
            }
        }
    }

    private static class SSTableViewColumnListenerList extends ListenerList<SSTableViewColumnListener>
    implements SSTableViewColumnListener {

        @Override
        public void columnInserted(SSTableView tableView, int index) {
            for (SSTableViewColumnListener listener : this) {
                listener.columnInserted(tableView, index);
            }
        }

        @Override
        public void columnsRemoved(SSTableView tableView, int index, Sequence<SSTableView.Column> columns) {
            for (SSTableViewColumnListener listener : this) {
                listener.columnsRemoved(tableView, index, columns);
            }
        }

        @Override
        public void columnNameChanged(Column column, String previousName) {
            for (SSTableViewColumnListener listener : this) {
                listener.columnNameChanged(column, previousName);
            }
        }

        @Override
        public void columnHeaderDataChanged(Column column, Object previousHeaderData) {
            for (SSTableViewColumnListener listener : this) {
                listener.columnHeaderDataChanged(column, previousHeaderData);
            }
        }

        @Override
        public void columnHeaderDataRendererChanged(SSTableView.Column column,
                                                    SSTableView.HeaderDataRenderer previousHeaderDataRenderer) {
            for (SSTableViewColumnListener listener : this) {
                listener.columnHeaderDataRendererChanged(column, previousHeaderDataRenderer);
            }
        }

        @Override
        public void columnWidthChanged(Column column, int previousWidth, boolean previousRelative) {
            for (SSTableViewColumnListener listener : this) {
                listener.columnWidthChanged(column, previousWidth, previousRelative);
            }
        }

        @Override
        public void columnWidthLimitsChanged(Column column, int previousMinimumWidth, int previousMaximumWidth) {
            for (SSTableViewColumnListener listener : this) {
                listener.columnWidthLimitsChanged(column, previousMinimumWidth, previousMaximumWidth);
            }
        }

        @Override
        public void columnFilterChanged(Column column, Object previousFilter) {
            for (SSTableViewColumnListener listener : this) {
                listener.columnFilterChanged(column, previousFilter);
            }
        }

        @Override
        public void columnCellRendererChanged(Column column, SSTableView.CellRenderer previousCellRenderer) {
            for (SSTableViewColumnListener listener : this) {
                listener.columnCellRendererChanged(column, previousCellRenderer);
            }
        }
    }

    private static class SSTableViewRowListenerList extends ListenerList<SSTableViewRowListener>
    implements SSTableViewRowListener {

        @Override
        public void rowInserted(SSTableView tableView, int index) {
            for (SSTableViewRowListener listener : this) {
                listener.rowInserted(tableView, index);
            }
        }

        @Override
        public void rowsRemoved(SSTableView tableView, int index, int count) {
            for (SSTableViewRowListener listener : this) {
                listener.rowsRemoved(tableView, index, count);
            }
        }

        @Override
        public void rowUpdated(SSTableView tableView, int index) {
            for (SSTableViewRowListener listener : this) {
                listener.rowUpdated(tableView, index);
            }
        }

        @Override
        public void rowsCleared(SSTableView tableView) {
            for (SSTableViewRowListener listener : this) {
                listener.rowsCleared(tableView);
            }
        }

        @Override
        public void rowsSorted(SSTableView tableView) {
            for (SSTableViewRowListener listener : this) {
                listener.rowsSorted(tableView);
            }
        }
    }

    private static class SSTableViewSelectListenerList extends ListenerList<SSTableViewSelectListener>
    implements SSTableViewSelectListener {

        @Override
        public void selectRangeAdded(SSTableView tableView, int rangeStart, int rangeEnd) {
            for (SSTableViewSelectListener listener : this) {
                listener.selectRangeAdded(tableView, rangeStart, rangeEnd);
            }
        }

        @Override
        public void selectRangeRemoved(SSTableView tableView, int rangeStart, int rangeEnd) {
            for (SSTableViewSelectListener listener : this) {
                listener.selectRangeRemoved(tableView, rangeStart, rangeEnd);
            }
        }

        @Override
        public void selectRangesChanged(SSTableView tableView, Sequence<Span> previousSelect) {
            for (SSTableViewSelectListener listener : this) {
                listener.selectRangesChanged(tableView, previousSelect);
            }
        }

        @Override
        public void selectRowChanged(SSTableView tableView, Object previousSelectRow) {
            for (SSTableViewSelectListener listener : this) {
                listener.selectRowChanged(tableView, previousSelectRow);
            }
        }
    }

    private static class SSTableViewSortListenerList extends ListenerList<SSTableViewSortListener>
    implements SSTableViewSortListener {
        public void sortAdded(SSTableView tableView, String columnName) {
            for (SSTableViewSortListener listener : this) {
                listener.sortAdded(tableView, columnName);
            }
        }

        public void sortUpdated(SSTableView tableView, String columnName,
                                SortDirection previousSortDirection) {
            for (SSTableViewSortListener listener : this) {
                listener.sortUpdated(tableView, columnName, previousSortDirection);
            }
        }

        public void sortRemoved(SSTableView tableView, String columnName,
                                SortDirection sortDirection) {
            for (SSTableViewSortListener listener : this) {
                listener.sortRemoved(tableView, columnName, sortDirection);
            }
        }

        public void sortChanged(SSTableView tableView) {
            for (SSTableViewSortListener listener : this) {
                listener.sortChanged(tableView);
            }
        }
    }

    private static class SSTableViewBindingListenerList extends ListenerList<SSTableViewBindingListener>
    implements SSTableViewBindingListener {
        @Override
        public void tableDataKeyChanged(SSTableView tableView, String previousTableDataKey) {
            for (SSTableViewBindingListener listener : this) {
                Logger.log.debug("previousTableDataKey:{}", previousTableDataKey);
                listener.tableDataKeyChanged(tableView, previousTableDataKey);
            }
        }

        @Override
        public void tableDataBindTypeChanged(SSTableView tableView, BindType previousTableDataBindType) {
            for (SSTableViewBindingListener listener : this) {
                listener.tableDataBindTypeChanged(tableView, previousTableDataBindType);
            }
        }

        @Override
        public void tableDataBindMappingChanged(SSTableView tableView,
                                                SSTableView.TableDataBindMapping previousTableDataBindMapping) {
            for (SSTableViewBindingListener listener : this) {
                listener.tableDataBindMappingChanged(tableView, previousTableDataBindMapping);
            }
        }

        @Override
        public void selectRowKeyChanged(SSTableView tableView, String previousSelectRowKey) {
            for (SSTableViewBindingListener listener : this) {
                Logger.log.debug("previousSelectRowKey:{}", previousSelectRowKey);
                listener.selectRowKeyChanged(tableView, previousSelectRowKey);
            }
        }

        @Override
        public void selectRowBindTypeChanged(SSTableView tableView, BindType previousSelectRowBindType) {
            for (SSTableViewBindingListener listener : this) {
                listener.selectRowBindTypeChanged(tableView, previousSelectRowBindType);
            }
        }

        @Override
        public void selectRowBindMappingChanged(SSTableView tableView, SelectRowBindMapping previousSelectRowBindMapping) {
            for (SSTableViewBindingListener listener : this) {
                listener.selectRowBindMappingChanged(tableView, previousSelectRowBindMapping);
            }
        }

        @Override
        public void selectRowsKeyChanged(SSTableView tableView, String previousSelectRowsKey) {
            for (SSTableViewBindingListener listener : this) {
                listener.selectRowsKeyChanged(tableView, previousSelectRowsKey);
            }
        }

        @Override
        public void selectRowsBindTypeChanged(SSTableView tableView, BindType previousSelectRowsBindType) {
            for (SSTableViewBindingListener listener : this) {
                listener.selectRowsBindTypeChanged(tableView, previousSelectRowsBindType);
            }
        }

        @Override
        public void selectRowsBindMappingChanged(SSTableView tableView,
                                                 SSTableView.SelectRowBindMapping previousSelectRowsBindMapping) {
            for (SSTableViewBindingListener listener : this) {
                listener.selectRowsBindMappingChanged(tableView, previousSelectRowsBindMapping);
            }
        }
    }

    private ArrayList<Column> columns = new ArrayList<Column>();
    private ColumnSequence columnSequence = new ColumnSequence();

    private SSTableViewListenerList tableViewListeners = new SSTableViewListenerList();
    private SSTableViewColumnListenerList tableViewColumnListeners = new SSTableViewColumnListenerList();
    private SSTableViewRowListenerList tableViewRowListeners = new SSTableViewRowListenerList();
    private SSTableViewSelectListenerList tableViewSelectListeners = new SSTableViewSelectListenerList();
    private SSTableViewSortListenerList tableViewSortListeners = new SSTableViewSortListenerList();
    private SSTableViewBindingListenerList tableViewBindingListeners = new SSTableViewBindingListenerList();

    public static final String COLUMN_NAME_KEY = "columnName";
    public static final String SORT_DIRECTION_KEY = "sortDirection";

    private ListListener<Object> tableDataListener = new ListListener<Object>() {
        @Override
        public void itemInserted(List<Object> list, int index) {
            //log.debug("");
            // Increment select ranges
            int updated = rangeSelect.insertIndex(index);

            tableViewRowListeners.rowInserted(SSTableView.this, index);

            if (updated > 0) {
                tableViewSelectListeners.selectRangesChanged(SSTableView.this, getSelectRanges());
            }// Notify listeners that items were inserted
            tableViewRowListeners.rowInserted(SSTableView.this, index);

            if (updated > 0) {
                tableViewSelectListeners.selectRangesChanged(SSTableView.this, getSelectRanges());
            }
        }

        @Override
        public void itemsRemoved(List<Object> list, int index, Sequence<Object> items) {
            int count = items.getLength();

            int previousSelectIndex;
            if (selectMode == SelectMode.SINGLE
            && rangeSelect.getLength() > 0) {
                previousSelectIndex = rangeSelect.get(0).start;
            } else {
                previousSelectIndex = -1;
            }

            // Decrement select ranges
            int updated = rangeSelect.removeIndexes(index, count);

            // Notify listeners that items were removed
            tableViewRowListeners.rowsRemoved(SSTableView.this, index, count);

            if (updated > 0) {
                tableViewSelectListeners.selectRangesChanged(SSTableView.this, getSelectRanges());

                if (selectMode == SelectMode.SINGLE
                && getSelectIndex() != previousSelectIndex) {
                    tableViewSelectListeners.selectRowChanged(SSTableView.this, null);
                }
            }
        }

        @Override
        public void itemUpdated(List<Object> list, int index, Object previousItem) {
//            tableViewRowListeners.rowUpdated(TableView.this, index);
        }

        @Override
        public void listCleared(List<Object> list) {
            int cleared = rangeSelect.getLength();
            rangeSelect.clear();

            tableViewRowListeners.rowsCleared(SSTableView.this);

            if (cleared > 0) {
                tableViewSelectListeners.selectRangesChanged(SSTableView.this, getSelectRanges());

                if (selectMode == SelectMode.SINGLE) {
                    tableViewSelectListeners.selectRowChanged(SSTableView.this, null);
                }
            }
        }

        @Override
        public void comparatorChanged(List<Object> list, Comparator<Object> previousComparator) {
            if (list.getComparator() != null) {
                int cleared = rangeSelect.getLength();
                rangeSelect.clear();
                tableViewRowListeners.rowsSorted(SSTableView.this);

                if (cleared > 0) {
                    tableViewSelectListeners.selectRangesChanged(SSTableView.this, getSelectRanges());

                    if (selectMode == SelectMode.SINGLE) {
                        tableViewSelectListeners.selectRowChanged(SSTableView.this, null);
                    }
                }
            }
        }
    };


    public SSTableView() {
        this(Type.Default, new ArrayList<Object>());
    }

    public SSTableView(Type type) {
        this(type, new ArrayList<Object>());
    }

    public SSTableView(Type type, List<?> tableData) {
        installSkin(SSTableView.class);
        setTableViewType(type);
        setTableData(tableData);
    }

    public interface Skin {
        public int getRowAt(int y);

        public int getColumnAt(int x);

        public Bounds getRowBounds(int rowIndex);

        public Bounds getColumnBounds(int columnIndex);

        public Bounds getCellBounds(int rowIndex, int columnIndex);

        public ArrayList<Integer> getRowBoundaries();

        public boolean isVariableRowHeight();

        public int getFixedRowHeight();

        public int getColumnBoundary(int columnIndex);
    }

    public List<?> getTableData() {
        return this.tableData;
    }

    public int[] getSelectRange() {
        return selectRange;
    }

    public void setSelectRange(int[] selectRange) {
        this.selectRange = selectRange;
    }

    public int[] getCopyRange() {
        return copyRange;
    }

    public void setCopyRange(int[] copyRange) {
        this.copyRange = copyRange;
    }

    public int[] getPasteRange() {
        return pasteRange;
    }

    public void setPasteRange(int[] pasteRange) {
        this.pasteRange = pasteRange;
    }

    @Override
    protected void setSkin(org.apache.pivot.wtk.Skin skin) {
        if (!(skin instanceof SSTableView.Skin)) {
            throw new IllegalArgumentException("Skin class must implement "
            + SSTableView.Skin.class.getName());
        }
        super.setSkin(skin);
    }

    public int getRowAt(int y) {
        SSTableView.Skin tableViewSkin = (SSTableView.Skin) getSkin();
        return tableViewSkin.getRowAt(y);
    }

    public int getColumnAt(int x) {
        SSTableView.Skin tableViewSkin = (SSTableView.Skin) getSkin();
        return tableViewSkin.getColumnAt(x);
    }

    public Bounds getCellBounds(int rowIndex, int columnIndex) {
        SSTableView.Skin tableViewSkin = (SSTableView.Skin) getSkin();
        return tableViewSkin.getCellBounds(rowIndex, columnIndex);
    }

    public Bounds getRowBounds(int rowIndex) {
        SSTableView.Skin tableViewSkin = (SSTableView.Skin) getSkin();
        return tableViewSkin.getRowBounds(rowIndex);
    }

    public ArrayList<Integer> getRowBoundaries() {
        SSTableView.Skin tableViewSkin = (SSTableView.Skin) getSkin();
        return tableViewSkin.getRowBoundaries();
    }

    public boolean isVariableRowHeight() {
        SSTableView.Skin tableViewSkin = (SSTableView.Skin) getSkin();
        return tableViewSkin.isVariableRowHeight();
    }

    public int getFixedRowHeight() {
        SSTableView.Skin tableViewSkin = (SSTableView.Skin) getSkin();
        return tableViewSkin.getFixedRowHeight();
    }

    public int getColumnBoundary(int columnIndex) {
        SSTableView.Skin tableViewSkin = (SSTableView.Skin) getSkin();
        return tableViewSkin.getColumnBoundary(columnIndex);
    }

    public long getSelectIndexQuartet() {
        return selectIndexQuartet;
    }

    public void setSelectIndexQuartet(long selectIndexQuartet) {
        this.selectIndexQuartet = selectIndexQuartet;
    }

    public void render(Object row, int rowIndex, int columnIndex, SSTableView tableView, String columnName, boolean select, boolean copied, boolean pasted, boolean highlighted, boolean disabled) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public String toString(Object row, String columnName) {
        return null;
    }

    public boolean isHeaderPressed() {
        return isHeaderPressed;
    }

    public void setHeaderPressed(boolean headerPressed) {
        isHeaderPressed = headerPressed;
    }

    public int getPreCellIndexPair() {
        return preCellIndexPair;
    }

    public void setPreCellIndexPair(int preCellIndexPair) {
        this.preCellIndexPair = preCellIndexPair;
    }

    public long getSelectCurrentIndexQuartet() {
        return selectCurrentIndexQuartet;
    }

    public void setSelectCurrentIndexQuartet(long selectCurrentIndexQuartet) {
        this.selectCurrentIndexQuartet = selectCurrentIndexQuartet;
    }

    public long getCopyIndexQuartet() {
        return copyIndexQuartet;
    }

    public void setCopyIndexQuartet(long copyIndexQuartet) {
        this.copyIndexQuartet = copyIndexQuartet;
    }

    public long getPasteIndexQuartet() {
        return pasteIndexQuartet;
    }

    public void setPasteIndexQuartet(long pasteIndexQuartet) {
        this.pasteIndexQuartet = pasteIndexQuartet;
    }

    public int getSelectCellIndexPair() {
        return selectCellIndexPair;
    }

    public void setSelectCellIndexPair(int selectCellIndexPair) {
        this.selectCellIndexPair = selectCellIndexPair;
    }

    public boolean isMouseDown() {
        return isMouseDown;
    }

    public void setMouseDown(boolean mouseDown) {
        isMouseDown = mouseDown;
    }

    public final boolean isSelect() {
        return isSelect;
    }

    public void setSelect(boolean select) {
        isSelect = select;
    }

    public final boolean isCopied() {
        return isCopied;
    }

    public void setCopied(boolean copied) {
        isCopied = copied;
    }

    public boolean isPasted() {
        return isPasted;
    }

    public void setPasted(boolean pasted) {
        isPasted = pasted;
    }

    public boolean isCutOut() {
        return isCutOut;
    }

    public void setCutOut(boolean cutOut) {
        isCutOut = cutOut;
    }

    /**
     * Sets the table data.
     *
     * @param tableData The data to be presented by the table view.
     */
    @SuppressWarnings("unchecked")
    public void setTableData(List<?> tableData) {
        if (tableData == null) {
            throw new IllegalArgumentException("tableData is null.");
        }

        List<?> previousTableData = this.tableData;

        if (previousTableData != tableData) {
            int cleared;
            if (previousTableData != null) {
                // Clear any existing selection
                cleared = rangeSelect.getLength();
                rangeSelect.clear();

                ((List<Object>) previousTableData).getListListeners().remove(tableDataListener);
            } else {
                cleared = 0;
            }

            ((List<Object>) tableData).getListListeners().add(tableDataListener);

            // Update the list data and fire change event
            this.tableData = tableData;
            tableViewListeners.tableDataChanged(this, previousTableData);

            if (cleared > 0) {
                tableViewSelectListeners.selectRangesChanged(this, getSelectRanges());

                if (selectMode == SelectMode.SINGLE) {
                    tableViewSelectListeners.selectRowChanged(this, null);
                }
            }
        }
    }

    @SuppressWarnings("unchecked")
    public void load(Object context) {
        Logger.log.debug("");
        if (tableDataKey != null
        && JSON.containsKey(context, tableDataKey)) {
            Object value = JSON.get(context, tableDataKey);
            List<?> tableData = (List<?>) value;

            setTableData(tableData);
        }

        Sequence<Object> rows = (Sequence<Object>) JSON.get(context, "");
        clearSelect();
        for (int i = 0, n = rows.getLength(); i < n; i++) {
            Object row = rows.get(i);
            int index = ((List<Object>) tableData).indexOf(row);
            if (index != -1) {
                addSelectIndex(index);
            }
        }
    }

    public void store(Object context) {
        Logger.log.info("selectRowsKey1:{}", selectRowsKey);
        if (tableDataKey != null) {
            Object value = tableData;
            JSON.put(context, tableDataKey, value);
        }

        ArrayList<Object> rows = new ArrayList<Object>();
        Sequence<Span> selectRanges = getSelectRanges();
        for (int i = 0, n = selectRanges.getLength(); i < n; i++) {
            Span range = selectRanges.get(i);
            for (int index = range.start; index <= range.end; index++) {
                Object row = tableData.get(index);
                rows.add(row);
            }
        }
        Logger.log.info("selectRowsKey2:{}", selectRowsKey);
        //JSON.put(context, selectRowsKey, rows);
    }

    public void clear() {
        if (tableDataKey != null) {
            setTableData(new ArrayList<Object>());
        }

        setSelectRow(null);
    }

    public ListenerList<SSTableViewListener> getTableViewListeners() {
        return tableViewListeners;
    }

    /**
     * Sets the table data.
     *
     * @param tableData A URL referring to a JSON file containing the data to be presented by
     *                  the table view.
     */
    public void setTableData(URL tableData) {
        if (tableData == null) {
            throw new IllegalArgumentException("tableData is null.");
        }

        JSONSerializer jsonSerializer = new JSONSerializer();

        try {
            setTableData((List<?>) jsonSerializer.readObject(tableData.openStream()));
        } catch (SerializationException exception) {
            throw new IllegalArgumentException(exception);
        } catch (IOException exception) {
            throw new IllegalArgumentException(exception);
        }
    }

    public SSTableView getColumnSource() {
        return columnSource;
    }

    public void setColumnSource(SSTableView columnSource) {
        SSTableView previousColumnSource = this.columnSource;

        if (previousColumnSource != columnSource) {
            this.columnSource = columnSource;
            tableViewListeners.columnSourceChanged(this, previousColumnSource);
        }
    }

    /**
     * Returns the editor used to edit rows in this table.
     *
     * @return The row editor, or <tt>null</tt> if no editor is installed.
     */
    public SSTableViewRowEditor getRowEditor() {
        return rowEditor;
    }

    /**
     * Sets the editor used to edit rows in this table.
     *
     * @param rowEditor The row editor for the list.
     */
    public void setRowEditor(SSTableViewRowEditor rowEditor) {
        SSTableViewRowEditor previousRowEditor = this.rowEditor;

        if (previousRowEditor != rowEditor) {
            this.rowEditor = rowEditor;
            tableViewListeners.rowEditorChanged(this, previousRowEditor);
        }
    }

    /**
     * Returns the table column sequence.
     *
     * @return The table column sequence.
     */
    public ColumnSequence getColumns() {
        ColumnSequence columnSequence = this.columnSequence;

        if (columnSource != null) {
            columnSequence = columnSource.getColumns();
        }

        return columnSequence;
    }

    /**
     * Sets the table data.
     *
     * @param tableData A JSON string (must begin with <tt>[</tt> and end with <tt>]</tt>)
     *                  denoting the data to be presented by the table view.
     */
    public final void setTableData(String tableData) {
        if (tableData == null) {
            throw new IllegalArgumentException("tableData is null.");
        }

        try {
            setTableData(JSONSerializer.parseList(tableData));
        } catch (SerializationException exception) {
            throw new IllegalArgumentException(exception);
        }
    }

    /**
     * When in single-select mode, returns the currently select index.
     *
     * @return The currently select index.
     */
    public int getSelectIndex() {
        if (selectMode != SelectMode.SINGLE) {
            throw new IllegalStateException("Table view is not in single-select mode.");
        }

        return (rangeSelect.getLength() == 0) ? -1 : rangeSelect.get(0).start;
    }

    /**
     * Sets the selection to a single index.
     *
     * @param index The index to select, or <tt>-1</tt> to clear the selection.
     */
    public void setSelectIndex(int index) {
        if (index == -1) {
            clearSelect();
        } else {
            setSelectRange(index, index);
        }
    }

    /**
     * Sets the selection to a single range.
     *
     * @param start
     * @param end
     */
    public void setSelectRange(int start, int end) {
        ArrayList<Span> selectRanges = new ArrayList<Span>();
        selectRanges.add(new Span(start, end));

        setSelectRanges(selectRanges);
    }

    /**
     * Returns the currently select ranges.
     *
     * @return An immutable list containing the currently select ranges. Note that the returned
     *         list is a wrapper around the actual selection, not a copy. Any changes made to the
     *         selection state will be reflected in the list, but events will not be fired.
     */
    public ImmutableList<Span> getSelectRanges() {
        return rangeSelect.getSelectRanges();
    }

    /**
     * Sets the selection to the given range sequence. Any overlapping or
     * connecting ranges will be consolidated, and the resulting selection will
     * be sorted in ascending order.
     *
     * @param selectRanges
     * @return The ranges that were actually set.
     */
    public Sequence<Span> setSelectRanges(Sequence<Span> selectRanges) {
        if (selectRanges == null) {
            throw new IllegalArgumentException("selectRanges is null.");
        }

        // When we're in mode NONE, the only thing we can do is to clear the selection
        if (selectMode == SelectMode.NONE
        && selectRanges.getLength() > 0) {
            throw new IllegalArgumentException("Select is not enabled.");
        }

        // Update the selection
        Sequence<Span> previousSelectRanges = this.rangeSelect.getSelectRanges();
        Object previousSelectRow = (selectMode == SelectMode.SINGLE) ? getSelectRow() : null;

        SSRangeSelect listSelect = new SSRangeSelect();
        for (int i = 0, n = selectRanges.getLength(); i < n; i++) {
            Span range = selectRanges.get(i);

            if (range == null) {
                throw new IllegalArgumentException("range is null.");
            }

            if (range.start < 0 || range.end >= tableData.getLength()) {
                throw new IndexOutOfBoundsException();
            }

            listSelect.addRange(range.start, range.end);
        }

        this.rangeSelect = listSelect;

        // Notify listeners
        tableViewSelectListeners.selectRangesChanged(this, previousSelectRanges);

        if (selectMode == SelectMode.SINGLE) {
            tableViewSelectListeners.selectRowChanged(this, previousSelectRow);
        }

        return getSelectRanges();
    }

    /**
     * Sets the selection to the given range sequence.
     *
     * @param selectRanges A JSON-formatted string containing the ranges to select.
     * @return The ranges that were actually set.
     * @see #setSelectRanges(Sequence)
     */
    public final Sequence<Span> setSelectRanges(String selectRanges) {
        if (selectRanges == null) {
            throw new IllegalArgumentException("selectRanges is null.");
        }

        try {
            setSelectRanges(parseSelectRanges(selectRanges));
        } catch (SerializationException exception) {
            throw new IllegalArgumentException(exception);
        }

        return getSelectRanges();
    }

    @SuppressWarnings("unchecked")
    private Sequence<Span> parseSelectRanges(String json)
    throws SerializationException {
        ArrayList<Span> selectRanges = new ArrayList<Span>();

        List<?> list = JSONSerializer.parseList(json);
        for (Object item : list) {
            Map<String, ?> map = (Map<String, ?>) item;
            selectRanges.add(new Span(map));
        }

        return selectRanges;
    }

    /**
     * Returns the first select index.
     *
     * @return The first select index, or <tt>-1</tt> if nothing is select.
     */
    public int getFirstSelectIndex() {
        return (rangeSelect.getLength() > 0) ? rangeSelect.get(0).start : -1;
    }

    /**
     * Returns the last select index.
     *
     * @return The last select index, or <tt>-1</tt> if nothing is select.
     */
    public int getLastSelectIndex() {
        return (rangeSelect.getLength() > 0) ?
        rangeSelect.get(rangeSelect.getLength() - 1).end : -1;
    }

    /**
     * Adds a single index to the selection.
     *
     * @param index The index to add.
     * @return <tt>true</tt> if the index was added to the selection; <tt>false</tt>,
     *         otherwise.
     */
    public boolean addSelectIndex(int index) {
        Sequence<Span> addedRanges = addSelectRange(index, index);
        return (addedRanges.getLength() > 0);
    }

    /**
     * Adds a range of indexes to the selection.
     *
     * @param start The first index in the range.
     * @param end   The last index in the range.
     * @return The ranges that were added to the selection.
     */
    public Sequence<Span> addSelectRange(int start, int end) {
        if (selectMode != SelectMode.MULTI) {
            throw new IllegalStateException("Table view is not in multi-select mode.");
        }

        if (start < 0 || end >= tableData.getLength()) {
            throw new IndexOutOfBoundsException();
        }

        Sequence<Span> addedRanges = rangeSelect.addRange(start, end);

        int n = addedRanges.getLength();
        for (int i = 0; i < n; i++) {
            Span addedRange = addedRanges.get(i);
            tableViewSelectListeners.selectRangeAdded(this, addedRange.start, addedRange.end);
        }

        if (n > 0) {
            tableViewSelectListeners.selectRangesChanged(this, null);
        }

        return addedRanges;
    }

    /**
     * Adds a range of indexes to the selection.
     *
     * @param range The range to add.
     * @return The ranges that were added to the selection.
     */
    public Sequence<Span> addSelectRange(Span range) {
        if (range == null) {
            throw new IllegalArgumentException("range is null.");
        }

        return addSelectRange(range.start, range.end);
    }

    /**
     * Removes a single index from the selection.
     *
     * @param index The index to remove.
     * @return <tt>true</tt> if the index was removed from the selection;
     *         <tt>false</tt>, otherwise.
     */
    public boolean removeSelectIndex(int index) {
        Sequence<Span> removedRanges = removeSelectRange(index, index);
        return (removedRanges.getLength() > 0);
    }

    /**
     * Removes a range of indexes from the selection.
     *
     * @param start The start of the range to remove.
     * @param end   The end of the range to remove.
     * @return The ranges that were removed from the selection.
     */
    public Sequence<Span> removeSelectRange(int start, int end) {
        if (selectMode != SelectMode.MULTI) {
            throw new IllegalStateException("Table view is not in multi-select mode.");
        }

        if (start < 0 || end >= tableData.getLength()) {
            throw new IndexOutOfBoundsException();
        }

        Sequence<Span> removedRanges = rangeSelect.removeRange(start, end);

        int n = removedRanges.getLength();
        for (int i = 0; i < n; i++) {
            Span removedRange = removedRanges.get(i);
            tableViewSelectListeners.selectRangeRemoved(this, removedRange.start, removedRange.end);
        }

        if (n > 0) {
            tableViewSelectListeners.selectRangesChanged(this, null);
        }

        return removedRanges;
    }

    /**
     * Removes a range of indexes from the selection.
     *
     * @param range The range to remove.
     * @return The ranges that were removed from the selection.
     */
    public Sequence<Span> removeSelectRange(Span range) {
        if (range == null) {
            throw new IllegalArgumentException("range is null.");
        }

        return removeSelectRange(range.start, range.end);
    }

    /**
     * Selects all rows in the table.
     */
    public void selectAll() {
        setSelectRange(0, tableData.getLength() - 1);
    }

    /**
     * Clears the selection.
     */
    public void clearSelect() {
        if (rangeSelect.getLength() > 0) {
            setSelectRanges(new ArrayList<Span>(0));
        }
    }

    /**
     * Returns the selection state of a given index.
     *
     * @param index The index whose selection state is to be tested.
     * @return <tt>true</tt> if the index is select; <tt>false</tt>,
     *         otherwise.
     */
    public boolean isRowSelect(int index) {
        if (index < 0 || index >= tableData.getLength()) {
            throw new IndexOutOfBoundsException();
        }

        return (rangeSelect.containsIndex(index));
    }

    public Object getSelectRow() {
        int index = getSelectIndex();
        Object row = null;

        if (index >= 0) {
            row = tableData.get(index);
        }

        return row;
    }

    @SuppressWarnings("unchecked")
    public void setSelectRow(Object row) {
        setSelectIndex((row == null) ? -1 : ((List<Object>) tableData).indexOf(row));
    }

    public Sequence<?> getSelectRows() {
        ArrayList<Object> rows = new ArrayList<Object>();

        for (int i = 0, n = rangeSelect.getLength(); i < n; i++) {
            Span range = rangeSelect.get(i);

            for (int index = range.start; index <= range.end; index++) {
                Object row = tableData.get(index);
                rows.add(row);
            }
        }

        return rows;
    }

    @SuppressWarnings("unchecked")
    public void setSelectRows(Sequence<Object> rows) {
        if (rows == null) {
            throw new IllegalArgumentException();
        }

        ArrayList<Span> selectRanges = new ArrayList<Span>();

        for (int i = 0, n = rows.getLength(); i < n; i++) {
            Object row = rows.get(i);
            if (row == null) {
                throw new IllegalArgumentException("item is null");
            }

            int index = ((List<Object>) tableData).indexOf(row);
            if (index == -1) {
                throw new IllegalArgumentException("\"" + row + "\" is not a valid selection.");
            }

            selectRanges.add(new Span(index));
        }

        setSelectRanges(selectRanges);
    }

    /**
     * Returns the current selection mode.
     */
    public SelectMode getSelectMode() {
        return selectMode;
    }

    /**
     * Sets the selection mode. Clears the selection if the mode has changed.
     *
     * @param selectMode The new selection mode.
     */
    public void setSelectMode(SelectMode selectMode) {
        if (selectMode == null) {
            throw new IllegalArgumentException("selectMode is null.");
        }

        SelectMode previousSelectMode = this.selectMode;

        if (previousSelectMode != selectMode) {
            // Clear any current selection
            clearSelect();

            // Update the selection mode
            this.selectMode = selectMode;

            // Fire select mode change event
            tableViewListeners.selectModeChanged(this, previousSelectMode);
        }
    }

    /**
     * Returns the table view's sort dictionary.
     */
    public SortDictionary getSort() {
        return sortDictionary;
    }

    /**
     * Sets the table view's sort.
     *
     * @param columnName
     * @param sortDirection
     */
    @SuppressWarnings("unchecked")
    public Dictionary<String, SortDirection> setSort(String columnName, SortDirection sortDirection) {
        Dictionary.Pair<String, SortDirection> sort =
        new Dictionary.Pair<String, SortDirection>(columnName, sortDirection);

        setSort(new ArrayList<Dictionary.Pair<String, SortDirection>>(sort));

        return getSort();
    }

    /**
     * Sets the table view's sort.
     *
     * @param sort A sequence of key/value pairs representing the sort. Keys represent column names and
     *             values represent sort direction.
     */
    public Dictionary<String, SortDirection> setSort(Sequence<Dictionary.Pair<String, SortDirection>> sort) {
        if (sort == null) {
            throw new IllegalArgumentException();
        }

        sortMap.clear();
        sortList.clear();

        for (int i = 0, n = sort.getLength(); i < n; i++) {
            Dictionary.Pair<String, SortDirection> pair = sort.get(i);

            if (!sortMap.containsKey(pair.key)) {
                sortMap.put(pair.key, pair.value);
                sortList.add(pair.key);
            }
        }

        tableViewSortListeners.sortChanged(this);

        return getSort();
    }

    /**
     * Sets the table view's sort.
     *
     * @param sort A JSON list containing JSON objects representing the sort.
     * @see #setSort(Sequence)
     */
    public final Dictionary<String, SortDirection> setSort(String sort) {
        if (sort == null) {
            throw new IllegalArgumentException();
        }

        try {
            setSort(parseSort(sort));
        } catch (SerializationException exception) {
            throw new IllegalArgumentException(exception);
        }

        return getSort();
    }

    @SuppressWarnings("unchecked")
    private Sequence<Dictionary.Pair<String, SortDirection>> parseSort(String json)
    throws SerializationException {
        ArrayList<Dictionary.Pair<String, SortDirection>> sort =
        new ArrayList<Dictionary.Pair<String, SortDirection>>();

        List<?> list = JSONSerializer.parseList(json);
        for (Object item : list) {
            Map<String, ?> map = (Map<String, ?>) item;

            Dictionary.Pair<String, SortDirection> pair =
            new Dictionary.Pair<String, SortDirection>((String) map.get(COLUMN_NAME_KEY),
            SortDirection.valueOf(((String) map.get(SORT_DIRECTION_KEY)).toUpperCase(Locale.ENGLISH)));
            sort.add(pair);
        }

        return sort;
    }

    /**
     * Clears the sort.
     */
    public void clearSort() {
        if (!sortMap.isEmpty()) {
            sortMap.clear();
            sortList.clear();
            tableViewSortListeners.sortChanged(this);
        }
    }

    /**
     * Returns the disabled state of a given row.
     *
     * @param index The index of the row whose disabled state is to be tested.
     * @return <tt>true</tt> if the row is disabled; <tt>false</tt>,
     *         otherwise.
     */
    @SuppressWarnings("unchecked")
    public boolean isRowDisabled(int index) {
        boolean disabled = false;

        if (disabledRowFilter != null) {
            Object row = tableData.get(index);
            disabled = ((Filter<Object>) disabledRowFilter).include(row);
        }

        return disabled;
    }

    /**
     * Returns the disabled row filter.
     *
     * @return The disabled row filter, or <tt>null</tt> if no disabled row filter is
     *         set.
     */
    public Filter<?> getDisabledRowFilter() {
        return disabledRowFilter;
    }

    /**
     * Sets the disabled row filter.
     *
     * @param disabledRowFilter The disabled row filter, or <tt>null</tt> for no disabled row filter.
     */
    public void setDisabledRowFilter(Filter<?> disabledRowFilter) {
        Filter<?> previousDisabledRowFilter = this.disabledRowFilter;

        if (previousDisabledRowFilter != disabledRowFilter) {
            this.disabledRowFilter = disabledRowFilter;
            tableViewListeners.disabledRowFilterChanged(this, previousDisabledRowFilter);
        }
    }

    public String getTableDataKey() {
        return tableDataKey;
    }

    public void setTableDataKey(String tableDataKey) {
        String previousTableDataKey = this.tableDataKey;

        if (previousTableDataKey != tableDataKey) {
            this.tableDataKey = tableDataKey;
            tableViewBindingListeners.tableDataKeyChanged(this, previousTableDataKey);
        }
    }

    public BindType getTableDataBindType() {
        return tableDataBindType;
    }

    public void setTableDataBindType(BindType tableDataBindType) {
        if (tableDataBindType == null) {
            throw new IllegalArgumentException();
        }

        BindType previousTableDataBindType = this.tableDataBindType;

        if (previousTableDataBindType != tableDataBindType) {
            this.tableDataBindType = tableDataBindType;
            tableViewBindingListeners.tableDataBindTypeChanged(this, previousTableDataBindType);
        }
    }

    public TableDataBindMapping getTableDataBindMapping() {
        return tableDataBindMapping;
    }

    public void setTableDataBindMapping(TableDataBindMapping tableDataBindMapping) {
        TableDataBindMapping previousTableDataBindMapping = this.tableDataBindMapping;

        if (previousTableDataBindMapping != tableDataBindMapping) {
            this.tableDataBindMapping = tableDataBindMapping;
            tableViewBindingListeners.tableDataBindMappingChanged(this, previousTableDataBindMapping);
        }
    }

    public String getSelectRowKey() {
        return selectRowKey;
    }

    public void setSelectRowKey(String selectRowKey) {
        String previousSelectRowKey = this.selectRowKey;
        Logger.log.info("selectRowKey:{}", selectRowKey);
        Logger.log.info("this.selectRowKey:{}", this.selectRowKey);

        if (previousSelectRowKey != selectRowKey) {
            this.selectRowKey = selectRowKey;
            tableViewBindingListeners.selectRowKeyChanged(this, previousSelectRowKey);
        }
    }

    public BindType getSelectRowBindType() {
        return selectRowBindType;
    }

    public void setSelectRowBindType(BindType selectRowBindType) {
        if (selectRowBindType == null) {
            throw new IllegalArgumentException();
        }

        BindType previousSelectRowBindType = this.selectRowBindType;
        if (previousSelectRowBindType != selectRowBindType) {
            this.selectRowBindType = selectRowBindType;
            tableViewBindingListeners.selectRowBindTypeChanged(this, previousSelectRowBindType);
        }
    }

    public SelectRowBindMapping getSelectRowBindMapping() {
        return selectRowBindMapping;
    }

    public void setSelectRowBindMapping(SelectRowBindMapping selectRowBindMapping) {
        SelectRowBindMapping previousSelectRowBindMapping = this.selectRowBindMapping;

        if (previousSelectRowBindMapping != selectRowBindMapping) {
            this.selectRowBindMapping = selectRowBindMapping;
            tableViewBindingListeners.selectRowBindMappingChanged(this, previousSelectRowBindMapping);
        }
    }

    public String getSelectRowsKey() {
        return selectRowsKey;
    }

    public void setSelectRowsKey(String selectRowsKey) {
        String previousSelectRowsKey = this.selectRowsKey;

        if (previousSelectRowsKey != selectRowsKey) {
            this.selectRowsKey = selectRowsKey;
            tableViewBindingListeners.selectRowsKeyChanged(this, previousSelectRowsKey);
        }
    }

    public BindType getSelectRowsBindType() {
        return selectRowsBindType;
    }

    public void setSelectRowsBindType(BindType selectRowsBindType) {
        if (selectRowsBindType == null) {
            throw new IllegalArgumentException();
        }

        BindType previousSelectRowsBindType = this.selectRowsBindType;
        if (previousSelectRowsBindType != selectRowsBindType) {
            this.selectRowsBindType = selectRowsBindType;
            tableViewBindingListeners.selectRowsBindTypeChanged(this, previousSelectRowsBindType);
        }
    }

    public SelectRowBindMapping getSelectRowsBindMapping() {
        return selectRowsBindMapping;
    }

    public void setSelectRowsBindMapping(SelectRowBindMapping selectRowsBindMapping) {
        SelectRowBindMapping previousSelectRowsBindMapping = this.selectRowsBindMapping;

        if (previousSelectRowsBindMapping != selectRowsBindMapping) {
            this.selectRowsBindMapping = selectRowsBindMapping;
            tableViewBindingListeners.selectRowsBindMappingChanged(this, previousSelectRowsBindMapping);
        }
    }

    /**
     * Returns the bounding area of a given column.
     *
     * @param columnIndex The column index.
     * @return The bounding area of the column.
     */
    public Bounds getColumnBounds(int columnIndex) {
        SSTableView.Skin tableViewSkin = (SSTableView.Skin) getSkin();
        return tableViewSkin.getColumnBounds(columnIndex);
    }

    public ListenerList<SSTableViewColumnListener> getTableViewColumnListeners() {
        return tableViewColumnListeners;
    }

    public ListenerList<SSTableViewRowListener> getTableViewRowListeners() {
        return tableViewRowListeners;
    }

    public ListenerList<SSTableViewSelectListener> getTableViewSelectListeners() {
        return tableViewSelectListeners;
    }

    public ListenerList<SSTableViewSortListener> getTableViewSortListeners() {
        return tableViewSortListeners;
    }

    public ListenerList<SSTableViewBindingListener> getTableViewBindingListeners() {
        return tableViewBindingListeners;
    }

    public SSPalette getPalette() {
        return palette;
    }

    public void setPalette(SSPalette palette) {
        this.palette = palette;
    }

    public Type getTableViewType() {
        return tableViewType;
    }

    public void setTableViewType(Type tableViewType) {
        this.tableViewType = tableViewType;
    }

    public long getCutIndexQuartet() {
        return cutIndexQuartet;
    }

    public void setCutIndexQuartet(long cutIndexQuartet) {
        this.cutIndexQuartet = cutIndexQuartet;
    }

    public int[] getCutRange() {
        return cutRange;
    }

    public void setCutRange(int[] cutRange) {
        this.cutRange = cutRange;
    }
}
