/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.ss.tableview;

import jp.pegasus.log.Logger;
import jp.pegasus.pivot.factory.SSFrameFactory;
import jp.pegasus.pivot.skin.*;
import jp.pegasus.pivot.ss.*;
import jp.pegasus.pivot.ss.tablepane.SSTablePane;
import org.apache.pivot.collections.Map;
import org.apache.pivot.wtk.*;

public class SSTableViewPopupMenu implements Application {
    private static long L00;
    private static long L02;

    private Display display = null;


    public SSTableViewPopupMenu() {
        super();
        Theme.getTheme().set(SSCardPane.class, SSCardPaneSkin.class);
        Theme.getTheme().set(SSCorner.class, SSCornerSkin.class);
        Theme.getTheme().set(SSScrollBar.class, SSScrollBarSkin.class);
        Theme.getTheme().set(SSScrollPane.class, ScrollPaneSkin.class);
        Theme.getTheme().set(SSTablePane.class, SSTablePaneSkin.class);
        Theme.getTheme().set(SSTableView.class, SSTableViewSkin.class);
        Theme.getTheme().set(SSTableViewHeader.class, TableViewHeaderSkin.class);
        Theme.getTheme().set(SSTextArea.class, SSTextAreaSkin.class);
    }

    @Override
    @SuppressWarnings("unchecked")
    public void startup(final Display display, Map<String, String> properties)
    throws Exception {
        this.display = display;

        SSFrame frame = SSFrameFactory.createFrame();
        frame.open(display);

        L02 = System.currentTimeMillis();
        Logger.log.debug("Execution time: {} s", (L02 - L00) / 1000.0);
    }

    @Override
    public boolean shutdown(boolean optional) {
        for (int i = display.getLength() - 1; i >= 0; i--) {
            Window window = (Window) display.get(i);
            window.close();
        }

        return false;
    }

    @Override
    public void suspend() {
    }

    @Override
    public void resume() {
    }

    public static void main(String[] args) {
        L00 = System.currentTimeMillis();
        DesktopApplicationContext.main(SSTableViewPopupMenu.class, args);
    }
}