/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.util;

import jp.pegasus.config.Config;
import jp.pegasus.log.Logger;
import jp.pegasus.pivot.model.SSElementalData;
import jp.pegasus.pivot.ss.tableview.SSTableView;
import org.apache.commons.lang.StringUtils;
import org.apache.pivot.collections.ArrayList;
import org.apache.pivot.collections.HashMap;
import org.apache.pivot.collections.List;
import org.apache.pivot.wtk.Clipboard;
import org.apache.pivot.wtk.LocalManifest;
import org.apache.pivot.wtk.Manifest;

import java.io.IOException;

public class SSTableViewUtil {

    private enum TakeType {
        Copy, Cut,
    }

    /**
     * Copy Operation
     *
     * @param tableView
     */
    @SuppressWarnings("unchecked")
    public static void copy(SSTableView tableView) {
        takeCellData(tableView, TakeType.Copy);
    }

    /**
     * Cut Operation
     *
     * @param tableView
     */
    @SuppressWarnings("unchecked")
    public static void cut(SSTableView tableView) {
        takeCellData(tableView, TakeType.Cut);
    }

    /**
     * Delete Operation
     *
     * @param tableView
     */
    @SuppressWarnings("unchecked")
    public static void delete(SSTableView tableView) {
        final long selectIndexQuartet = tableView.getSelectIndexQuartet();

        if (selectIndexQuartet == Config.SELECT_ALL_INDEX) {
            List<HashMap<String, SSElementalData>> tableData = new ArrayList<HashMap<String, SSElementalData>>();
            for (int i = 1, n = Config.MAX_ROW_NUMBER; i < n; i++) {
                tableData.add(new HashMap<String, SSElementalData>());
            }
            tableView.setTableData(tableData);
            return;
        }

        final int selectColIndexFr = (int) (0x000000000000FFFFl & (selectIndexQuartet >> 48));
        final int selectRowIndexFr = (int) (0x000000000000FFFFl & (selectIndexQuartet >> 32));
        final int selectColIndexTo = (int) (0x000000000000FFFFl & (selectIndexQuartet >> 16));
        final int selectRowIndexTo = (int) (0x000000000000FFFFl & (selectIndexQuartet));
        List<?> tableData = tableView.getTableData();
        for (int rowIndex = selectRowIndexFr; rowIndex <= selectRowIndexTo; rowIndex++) {
            HashMap<String, SSElementalData> rowData = (HashMap<String, SSElementalData>) tableData.get(rowIndex);
            for (int columnIndex = selectColIndexFr; columnIndex <= selectColIndexTo; columnIndex++) {
                rowData.put(SSTableViewUtil.fromNumber2Alphabet(columnIndex + 1), new SSElementalData());
            }
        }
        tableView.repaint();
    }

    /**
     * Paste Operation
     *
     * @param tableView
     */
    @SuppressWarnings("unchecked")
    public static void paste(SSTableView tableView) {
        if (tableView.isCutOut()) {
            final long cutIdxQuartet = tableView.getCutIndexQuartet();
            final int cutColIdxFr = (int) (0x000000000000FFFFl & (cutIdxQuartet >> 48));
            final int cutRowIdxFr = (int) (0x000000000000FFFFl & (cutIdxQuartet >> 32));
            final int cutColIdxTo = (int) (0x000000000000FFFFl & (cutIdxQuartet >> 16));
            final int cutRowIdxTo = (int) (0x000000000000FFFFl & (cutIdxQuartet));
            List<?> tableData = tableView.getTableData();
            for (int rowIndex = cutRowIdxFr; rowIndex <= cutRowIdxTo; rowIndex++) {
                HashMap<String, SSElementalData> rowData = (HashMap<String, SSElementalData>) tableData.get(rowIndex);
                for (int columnIndex = cutColIdxFr; columnIndex <= cutColIdxTo; columnIndex++) {
                    rowData.put(SSTableViewUtil.fromNumber2Alphabet(columnIndex + 1), new SSElementalData());
                }
            }
            tableView.setCutOut(false);
        }

        final long selectIndexQuartet = tableView.getSelectIndexQuartet();
        final int selColIdxFr = (int) (0x000000000000FFFFl & (selectIndexQuartet >> 48));
        final int selRowIdxFr = (int) (0x000000000000FFFFl & (selectIndexQuartet >> 32));
        final int selColIdxTo = (int) (0x000000000000FFFFl & (selectIndexQuartet >> 16));
        final int selRowIdxTo = (int) (0x000000000000FFFFl & (selectIndexQuartet));

        final int maxColNumber = Config.MAX_COL_NUMBER - 2;
        final int maxRowNumber = Config.MAX_ROW_NUMBER - 2;

        tableView.setPasted(true);
        tableView.setCopied(false);
        Manifest manifest = Clipboard.getContent();

        try {
            int rowIndex = selRowIdxFr;
            String pasteData = manifest.getText();
            if (pasteData == null) {
                Logger.log.warn("\"pasteData\" is null");
                return;
            }

            String[] rowPsteDataArr = pasteData.split("\n");
            if (rowPsteDataArr == null) {
                Logger.log.warn("\"rowPsteDataArr\" is null.");
                return;
            } else if (rowPsteDataArr.length == 0) {
                Logger.log.warn("\"rowPsteDataArr\" is 0.");
                return;
            }

            final int psteColWidthWork = StringUtils.splitPreserveAllTokens(rowPsteDataArr[0], "\t").length;
            final int psteColWidth = psteColWidthWork == 0 ? 1 : psteColWidthWork;
            final int psteRowWidth = rowPsteDataArr.length;
            final int workColIdxTo = selColIdxFr + psteColWidth - 1;
            final int workRowIdxTo = selRowIdxFr + psteRowWidth - 1;
            final int psteColIdxTo = (workColIdxTo > maxColNumber) ? maxColNumber : workColIdxTo;
            final int psteRowIdxTo = (workRowIdxTo > maxRowNumber) ? maxRowNumber : workRowIdxTo;
            Logger.log.debug("pasteColWidth:{}, pasteRowWidth:{}", psteColWidth, psteRowWidth);
//            Logger.log.debug("rowPsteDataArr[0]:{}", rowPsteDataArr[0]);

            final long pasteIndex = ((long) selColIdxFr << 48) | ((long) selRowIdxFr << 32) | ((long) psteColIdxTo << 16) | (long) psteRowIdxTo;
            tableView.setPasteIndexQuartet(pasteIndex);

            final int selctColWidth = selColIdxTo - selColIdxFr + 1;
            final int selctRowWidth = selRowIdxTo - selRowIdxFr + 1;
            Logger.log.debug("selctColWidth:{}, selctRowWidth:{}", selctColWidth, selctRowWidth);
            final int maxRowLoop = psteRowWidth < selctRowWidth ? selctRowWidth / psteRowWidth + 1 : 1;

            for (int rowWidthIdx = 0; rowWidthIdx < maxRowLoop; rowWidthIdx++) {
                for (String rowPasteData : rowPsteDataArr) {
                    if (maxRowNumber < rowIndex || (psteRowWidth < selctRowWidth && selRowIdxTo < rowIndex)) {
                        break;
                    }

                    org.apache.pivot.collections.List<?> tableData = tableView.getTableData();
                    HashMap<String, SSElementalData> rowData = (HashMap<String, SSElementalData>) tableData.get(rowIndex);

                    final String[] cellDataArray = StringUtils.splitPreserveAllTokens(rowPasteData, "\t");
                    int colIndex = selColIdxFr + 1;
                    final int maxColLoop = psteColWidth < selctColWidth ? selctColWidth / psteColWidth + 1 : 1;
                    for (int colWidthIdx = 0; colWidthIdx < maxColLoop; colWidthIdx++) {
                        for (final String cellData : cellDataArray) {
                            if (maxColNumber < colIndex || (psteColWidth <= selctColWidth && selColIdxTo < colIndex - 1)) {
                                break;
                            }
                            rowData.put(fromNumber2Alphabet(colIndex), new SSElementalData() {{
                                this.setText(cellData);
                                Logger.log.debug("cellData:{}", cellData);
                            }});
                            colIndex++;
                        }
                    }
                    rowIndex++;
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        tableView.repaint();
    }

    /**
     * Take Cell Data
     *
     * @param tableView
     * @param takeType
     */
    @SuppressWarnings("unchecked")
    private static void takeCellData(SSTableView tableView, TakeType takeType) {
        final long selectIndexQuartet = tableView.getSelectIndexQuartet();
        final int selectColIndexFr = (int) (0x000000000000FFFFl & (selectIndexQuartet >> 48));
        final int selectRowIndexFr = (int) (0x000000000000FFFFl & (selectIndexQuartet >> 32));
        final int selectColIndexTo = (int) (0x000000000000FFFFl & (selectIndexQuartet >> 16));
        final int selectRowIndexTo = (int) (0x000000000000FFFFl & (selectIndexQuartet));

        List<?> tableData = tableView.getTableData();
        StringBuilder dataBuilder = new StringBuilder();
        for (int rowIndex = selectRowIndexFr; rowIndex <= selectRowIndexTo; rowIndex++) {
            HashMap<String, ?> rowData = (HashMap<String, ?>) tableData.get(rowIndex);
            for (int columnIndex = selectColIndexFr; columnIndex <= selectColIndexTo; columnIndex++) {
                String colName = SSTableViewUtil.fromNumber2Alphabet(columnIndex + 1);

                Object object = rowData.get(colName);
                if (object instanceof SSElementalData) {
                    SSElementalData tempData = (SSElementalData) rowData.get(colName);
                    if (tempData == null) {
                        dataBuilder.append("");
                    } else {
                        String text = tempData.getText();
                        dataBuilder.append(text == null ? "" : text);
                    }
                    if (columnIndex != selectColIndexTo) {
                        dataBuilder.append("\t");
                    }
                } else if (object instanceof HashMap) {
                    HashMap tempData = (HashMap) rowData.get(colName);
                    if (tempData == null) {
                        dataBuilder.append("");
                    } else {
                        String text = (String) tempData.get("text");
                        dataBuilder.append(text == null ? "" : text);
                    }
                    if (columnIndex != selectColIndexTo) {
                        dataBuilder.append("\t");
                    }
                } else {
                    dataBuilder.append("");
                    if (columnIndex != selectColIndexTo) {
                        dataBuilder.append("\t");
                    }
                }

            }
            dataBuilder.append("\r\n");
        }

        String data = dataBuilder.toString();
        LocalManifest localManifest = new LocalManifest();
        localManifest.putText(data);
        Clipboard.setContent(localManifest);

        switch (takeType) {
            case Copy:
                tableView.setCopyIndexQuartet(selectIndexQuartet);
                tableView.setCopyRange(tableView.getSelectRange());
                tableView.setCopied(true);
                tableView.setCutOut(false);
                break;
            case Cut:
                tableView.setCutIndexQuartet(selectIndexQuartet);
                tableView.setCutRange(tableView.getSelectRange());
                tableView.setCopied(false);
                tableView.setCutOut(true);
                break;
            default:
                throw new IllegalArgumentException();
        }
        tableView.setPasted(false);
        tableView.repaint();
    }

    /**
     * Convert Numeric Values into the Alphabetical Values following the way of Excel Spread Sheet.
     * ex.) 1=>A, 2=>B, ... , 26=>Z, 27=>AA
     *
     * @param number
     * @return
     */
    public static String fromNumber2Alphabet(final int number) {
        StringBuilder column = new StringBuilder();

        if (number > 702) {
            column.append(String.valueOf((char) ((((number - 1) / 676) - 1) % 26 + 65)));
        }
        if (number > 26) {
            column.append(String.valueOf((char) (((number - 1) / 26 - 1) % 26 + 65)));
        }
        column.append(String.valueOf((char) ((number - 1) % 26 + 65)));

        return column.toString();
    }

    /**
     * Creates a new column.
     * Get Visible Range
     *
     * @param tableView
     * @param x
     * @param y
     * @param width
     * @param height
     * @return visibleRange
     *         <p/>
     *         final int columnBegin = (int) (0x000000000000FFFFl & (visibleRangeQuartet >> 48));
     *         final int rowBegin    = (int) (0x000000000000FFFFl & (visibleRangeQuartet >> 32));
     *         final int columnEnd   = (int) (0x000000000000FFFFl & (visibleRangeQuartet >> 16));
     *         final int rowEnd      = (int) (0x000000000000FFFFl & (visibleRangeQuartet));
     */
    @SuppressWarnings("unchecked")
    public static long getVisibleRange(SSTableView tableView, final int x, final int y, final int width, final int height) {
        SSTableView.ColumnSequence columns = tableView.getColumns();
        org.apache.pivot.collections.List<Object> tableData = (org.apache.pivot.collections.List<Object>) tableView.getTableData();

        final int colCount = columns.getLength();
        final int rowCount = tableData.getLength();
        int rowBgn = 0;
        int colBgn = tableView.getColumnAt(x + 1);
        int colEnd = colCount - 1;
        int rowEnd = rowCount - 1;

        if (colBgn == -1) {
            colBgn = colCount - 1;
        }

        if (colEnd != -1) {
            int clipRight = x + width - 1;
            clipRight = Math.min(clipRight, tableView.getColumnBoundary(colEnd) - 1);
            colEnd = tableView.getColumnAt(clipRight);
        }

        if (tableView.isVariableRowHeight()) {
            ArrayList<Integer> rowBoundaries = tableView.getRowBoundaries();
            rowBgn = tableView.getRowAt(y);
            if (rowBgn == -1) {
                rowBgn = rowCount - 1;
            }
            if (rowBgn != -1) {
                int clipBottom = y + height - 1;
                clipBottom = Math.min(clipBottom, rowBoundaries.get(rowEnd) - 1);
                rowBgn = tableView.getRowAt(clipBottom);
            }
        } else {
            rowBgn = Math.max(rowBgn,
            (int) Math.floor(y / (double) (tableView.getFixedRowHeight() + 1)));
            rowEnd = Math.min(rowEnd,
            (int) Math.ceil((y + height) / (double) (tableView.getFixedRowHeight() + 1)) - 1);
        }

        return ((long) rowBgn << 48) | ((long) rowBgn << 32) | ((long) colEnd << 16) | (long) rowEnd;
    }

    public static int getVisibleColumnRange(SSTableView tableView, final int x, final int width) {
        SSTableView.ColumnSequence columns = tableView.getColumns();

        final int columnCount = columns.getLength();
        int columnBgn = 0;
        int columnEnd = columnCount - 1;

        columnBgn = tableView.getColumnAt(x + 1);
        if (columnBgn == -1) {
            columnBgn = columnCount - 1;
        }

        if (columnEnd != -1) {
            int clipRight = x + width - 1;
            clipRight = Math.min(clipRight, tableView.getColumnBoundary(columnEnd) - 1);
            columnEnd = tableView.getColumnAt(clipRight);
        }

        return (columnBgn << 16) | columnEnd;
    }

    @SuppressWarnings("unchecked")
    public static int getVisibleRowRange(SSTableView tableView, final int y, final int height) {
        org.apache.pivot.collections.List<Object> tableData =
        (org.apache.pivot.collections.List<Object>) tableView.getTableData();

        final int rowCount = tableData.getLength();
        int rowBgn = 0;
        int rowEnd = rowCount - 1;

        if (tableView.isVariableRowHeight()) {
            ArrayList<Integer> rowBoundaries = tableView.getRowBoundaries();
            rowBgn = tableView.getRowAt(y);
            if (rowBgn == -1) {
                rowBgn = rowCount - 1;
            }
            if (rowBgn != -1) {
                int clipBottom = y + height - 1;
                clipBottom = Math.min(clipBottom, rowBoundaries.get(rowEnd) - 1);
                rowBgn = tableView.getRowAt(clipBottom);
            }
        } else {
            rowBgn = Math.max(rowBgn,
            (int) Math.floor(y / (double) (tableView.getFixedRowHeight() + 1)));
            rowEnd = Math.min(rowEnd,
            (int) Math.ceil((y + height) / (double) (tableView.getFixedRowHeight() + 1)) - 1);
        }

        return (rowBgn << 16) | rowEnd;
    }

}
