/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.pivot.util;

import jp.pegasus.pivot.factory.SSFrameFactory;
import jp.pegasus.pivot.factory.SSTreeBranchFactory;
import jp.pegasus.pivot.factory.tabpane.SSTabPaneContentFactory;
import jp.pegasus.pivot.factory.tabpane.SSTabPaneMenuFactory;
import jp.pegasus.pivot.model.SSData;
import jp.pegasus.pivot.model.SSFileData;
import jp.pegasus.pivot.ss.*;
import org.apache.pivot.collections.ArrayList;
import org.apache.pivot.collections.List;
import org.apache.pivot.wtk.SplitPane;

public class SSTreeViewUtil {

    /**
     * Copy File
     *
     * @param treeView
     */
    @SuppressWarnings("unchecked")
    public static void cpyFile(SSTreeView treeView) {
        SSTreeNode treeNode = (SSTreeNode) treeView.getSelectedNode();
        final int selectIndex = treeView.getTreeData().indexOf((SSTreeBranch) treeNode);

        SSFrame frame = (SSFrame) treeView.getWindow();
        SSData data = (SSData) frame.getAttribute(SSData.DEFAULT_TYPE);

        if (data.getFileDataList().getLength() == 0) {
            throw new RuntimeException();
        } else {
            SSFileData fileData = data.getFileDataList().get(selectIndex);
            SSFileData fileDataCopy = null;
            try {
                fileDataCopy = fileData.clone();
            } catch (CloneNotSupportedException e) {
                e.printStackTrace();
            }

            data.getFileDataList().add(fileDataCopy);
            final String copiedFileName = String.format(SSData.DEF_CPY_FIL_NAME, fileData.getFileName());
            fileDataCopy.setFileName(copiedFileName);

            final int sheetType = fileDataCopy.getSheetType();
            SSTreeNode.NodeType nodeType;
            switch (sheetType) {
                case 0:
                    nodeType = SSTreeNode.NodeType.MnuFilGrid;
                    break;
                case 1:
                    nodeType = SSTreeNode.NodeType.MnuFilText;
                    break;
                default:
                    throw new RuntimeException();
            }

            SSTreeBranch treeBranch = SSTreeBranchFactory.createTreeBranch(copiedFileName, nodeType, null);
            SSTabPane tabPane = SSTabPaneContentFactory.constructTabPaneContent(fileDataCopy);
            treeBranch.setTabPane(tabPane);
            treeView.getTreeData().add(treeBranch);
            treeView.repaint();

            SSFrameFactory.save(frame);
        }
    }

    /**
     * Delete File
     *
     * @param treeView
     */
    @SuppressWarnings("unchecked")
    public static void delFile(SSTreeView treeView) {
        SSTreeNode deleteTreeNode = (SSTreeNode) treeView.getSelectedNode();

        SSFrame frame = (SSFrame) treeView.getWindow();
        SSData data = (SSData) frame.getAttribute(SSData.DEFAULT_TYPE);

        ArrayList<SSTreeNode> treeNodeList = (ArrayList<SSTreeNode>) treeView.getTreeData();
        if (data.getFileDataList().getLength() == 0) {
            throw new RuntimeException();
        } else {
            final int selectIndex = treeNodeList.indexOf(deleteTreeNode);
            data.getFileDataList().remove(selectIndex, 1);
            treeNodeList.remove(selectIndex, 1);

            SplitPane splitPaneRigt = (SplitPane) frame.getContent();
            SplitPane splitPaneLeft = (SplitPane) splitPaneRigt.getLeft();
            if (treeNodeList.getLength() == 0) {
                splitPaneLeft.setRight(new SSTabPane());
            } else {
                List.ItemIterator<SSTreeNode> iterator = treeNodeList.iterator();
                SSTreeNode selectTreeNode = iterator.next();
                final int nextSelectIndex = treeNodeList.indexOf(selectTreeNode);
                data.setSelectFileIndex(nextSelectIndex);

                SSTabPane leftTab = SSTabPaneMenuFactory.constructTabPaneMenu(treeNodeList, nextSelectIndex);
                splitPaneLeft.setLeft(leftTab);

                SSTabPane centerTab = selectTreeNode.getTabPane();
                splitPaneLeft.setRight(centerTab);
            }
            treeView.repaint();

            SSFrameFactory.save(frame);
        }
    }

    /**
     * Add File
     *
     * @param treeView
     */
    @SuppressWarnings("unchecked")
    public static void addFile(SSTreeView treeView, SSTreeNode.NodeType nodeType) {
        if (treeView == null) {
            return;
        }
        SSFrame frame = (SSFrame) treeView.getWindow();

        SSData data = (SSData) frame.getAttribute(SSData.DEFAULT_TYPE);

        int sheetType;
        switch (nodeType) {
            case MnuFilGrid:
                sheetType = 0;
                break;
            case MnuFilText:
                sheetType = 1;
                break;
            default:
                throw new RuntimeException();
        }

        SSFileData fileData = SSData.createSheetFile(
                SSData.DEF_FIL_NAME,
                new String[]{
                        String.format(SSData.DEF_SHT_NAME, 1),
                        String.format(SSData.DEF_SHT_NAME, 2),
                        SSData.DEF_SHT_ADD,
                }, sheetType);
        data.getFileDataList().add(fileData);

        final String fileName = SSData.DEF_FIL_NAME;
        SSTreeBranch treeBranch = SSTreeBranchFactory.createTreeBranch(fileName, nodeType, null);
        SSTabPane tabPane = SSTabPaneContentFactory.constructTabPaneContent(fileData);
        treeBranch.setTabPane(tabPane);

        treeView.getTreeData().add(treeBranch);
        treeView.repaint();

        SSFrameFactory.save(frame);
    }

    /**
     * Add Folder
     *
     * @param treeView
     */
    @SuppressWarnings("unchecked")
    public static void addFolder(SSTreeView treeView) {
        //todo
    }

}
