/*
 * Copyright (C) 2010-2011 Toshiyuki Kitani
 *
 *  Pegasus is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Pegasus is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package jp.pegasus.util;

import jp.pegasus.log.Logger;
import jp.pegasus.main.Main;
import jp.pegasus.pivot.model.SSData;
import jp.pegasus.pivot.model.SSElementalData;
import org.apache.commons.compress.compressors.CompressorException;
import org.apache.commons.compress.compressors.CompressorInputStream;
import org.apache.commons.compress.compressors.CompressorOutputStream;
import org.apache.commons.compress.compressors.CompressorStreamFactory;
import org.apache.commons.lang.time.StopWatch;
import org.apache.pivot.collections.ArrayList;
import org.apache.pivot.collections.HashMap;
import org.apache.pivot.collections.Map;
import org.apache.pivot.json.JSONSerializer;
import org.apache.pivot.serialization.SerializationException;
import org.apache.pivot.wtk.Component;

import java.io.*;
import java.net.URL;
import java.util.Iterator;

public class JsonUtil {

    @SuppressWarnings("unchecked")
    public static void setStyles(Component component, String location) {
        try {
            URL jsonLocation = Main.class.getResource(location);
            try {
                InputStream inputStream = jsonLocation.openStream();
                JSONSerializer serializer = new JSONSerializer();
                try {
                    Map<String, ?> properties = (Map<String, ?>) serializer.readObject(inputStream);
                    Iterator<String> iterator = properties.iterator();
                    while (iterator.hasNext()) {
                        String key = iterator.next();
                        //log.debug("key:{}, value:{}", key, properties.get(key));
                        component.getStyles().put(key, properties.get(key));
                    }
                } catch (SerializationException e) {
                    e.printStackTrace();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        } catch (SecurityException e) {
            e.printStackTrace();
        }
    }

    @SuppressWarnings("unchecked")
    public static void setProperties(Map<Object, Object> propertyMap, String location) {
        try {
            URL jsonLocation = Main.class.getResource(location);
            try {
                InputStream inputStream = jsonLocation.openStream();
                JSONSerializer serializer = new JSONSerializer();
                try {
                    Map<Object, ?> properties = (Map<Object, ?>) serializer.readObject(inputStream);
                    Iterator<Object> iterator = properties.iterator();
                    while (iterator.hasNext()) {
                        Object key = iterator.next();
                        //log.debug("key:{}, value:{}", key, properties.get(key));
                        propertyMap.put(key, properties.get(key));
                    }
                } catch (SerializationException e) {
                    e.printStackTrace();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        } catch (SecurityException e) {
            e.printStackTrace();
        }
    }

    /**
     * Read GrandData
     *
     * @param location
     * @return
     */
    @SuppressWarnings("unchecked")
    public static SSData readGrandData(String location) {
        SSData data = null;

        File dataFile = new File(location);
        if (dataFile.exists() && dataFile.length() > 0) {
            try {
                FileInputStream fileInput = new FileInputStream(location);
                BufferedInputStream buffInput = new BufferedInputStream(fileInput);
                CompressorInputStream gzInput = new CompressorStreamFactory().createCompressorInputStream(CompressorStreamFactory.GZIP, buffInput);

                JSONSerializer jsonSerializer = new JSONSerializer(SSData.class);
                try {
                    data = (SSData) jsonSerializer.readObject(gzInput);
                } catch (SerializationException e) {
                    e.printStackTrace();
                }
                gzInput.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            data = SSData.initializeData();
        }
        return data;
    }

    /**
     * Write GrandData
     *
     * @param data
     * @param location
     */
    @SuppressWarnings("unchecked")
    public static void writeGrandData(SSData data, String location) {
        JSONSerializer jsonSerializer = new JSONSerializer(SSData.class);
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        long startTime = stopWatch.getTime();

        try {
            FileOutputStream fileOutput = new FileOutputStream(location);
            BufferedOutputStream buffOutput = new BufferedOutputStream(fileOutput);
            CompressorOutputStream gzOutput = new CompressorStreamFactory().createCompressorOutputStream(CompressorStreamFactory.GZIP, buffOutput);
            jsonSerializer.writeObject(data, gzOutput);
            gzOutput.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        stopWatch.stop();
        Logger.log.info("Write GrandData Time: {}", stopWatch.getTime() - startTime);
    }

    /**
     * Raad CellDataList
     *
     * @param location
     * @return
     */
    @SuppressWarnings("unchecked")
    public static ArrayList<HashMap<String, SSElementalData>> readCellDataList(String location) {
        ArrayList<HashMap<String, SSElementalData>> data = null;
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        long startTime = stopWatch.getTime();

        File dataFile = new File(location);
        if (dataFile.exists() && dataFile.length() > 0) {

            try {
                FileInputStream fileInput = new FileInputStream(location);
                BufferedInputStream buffInput = new BufferedInputStream(fileInput);
                CompressorInputStream gzInput = new CompressorStreamFactory().createCompressorInputStream(CompressorStreamFactory.GZIP, buffInput);

                JSONSerializer jsonSerializer = new JSONSerializer();
                try {
                    data = (ArrayList<HashMap<String, SSElementalData>>) jsonSerializer.readObject(gzInput);
                } catch (SerializationException e) {
                    e.printStackTrace();
                }
                gzInput.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            data = SSElementalData.createCellDataList();
        }

        stopWatch.stop();
        Logger.log.info("Read CellDataList Time: {}", stopWatch.getTime() - startTime);
        return data;
    }

    /**
     * Raad TextData
     *
     * @param location
     * @return
     */
    @SuppressWarnings("unchecked")
    public static SSElementalData readTextData(String location) {
        SSElementalData data = null;
        File dataFile = new File(location);
        if (dataFile.exists() && dataFile.length() > 0) {
            try {
                FileInputStream fileInput = new FileInputStream(location);
                BufferedInputStream buffInput = new BufferedInputStream(fileInput);
                CompressorInputStream gzInput = new CompressorStreamFactory().createCompressorInputStream(CompressorStreamFactory.GZIP, buffInput);

                JSONSerializer jsonSerializer = new JSONSerializer(SSElementalData.class);
                try {
                    data = (SSElementalData) jsonSerializer.readObject(gzInput);
                } catch (SerializationException e) {
                    e.printStackTrace();
                }
                gzInput.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            data = new SSElementalData();
        }
        return data;
    }

    public static String getDigest(Object objectData, String algorithm) {
        String digestS = null;
        try {
            String jsonData = JSONSerializer.toString(objectData);
            byte[] digestB = DigestUtil.generateDigest(jsonData, algorithm);
            digestS = DigestUtil.getStringDigest(digestB);
        } catch (SerializationException e) {
            e.printStackTrace();
        }

        return digestS;
    }


    /**
     * Write CellDataList
     *
     * @param objectData
     */
    @SuppressWarnings("unchecked")
    public static void writeSheetData(final Object objectData, final String filePath) {
        if (!new File(filePath).exists()) {
            Logger.log.info("Create filePath: {}", filePath);
            try {
                new File(filePath).createNewFile();
                JSONSerializer jsonSerializer = new JSONSerializer();

                FileOutputStream fileOutput = new FileOutputStream(filePath);
                BufferedOutputStream buffOutput = new BufferedOutputStream(fileOutput);
                CompressorOutputStream gzOutput = new CompressorStreamFactory().createCompressorOutputStream(CompressorStreamFactory.GZIP, buffOutput);
                jsonSerializer.writeObject(objectData, gzOutput);
                gzOutput.close();
            } catch (CompressorException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } catch (SerializationException e) {
                e.printStackTrace();
            }
        }
    }

}
